@@ -3,20 +3,81 @@ import json
 import os
 import re
 import socket
+import time
 import urllib.error
 import urllib.parse
 import urllib.request
 from collections import OrderedDict
+from gettext import gettext as _
+from typing import Any, Dict
 
 import requests
 
 from lutris import settings
 from lutris.util import http, system
+from lutris.util.display import get_gpus_info
+from lutris.util.http import HTTPError, Request
+from lutris.util.linux import LINUX_SYSTEM
 from lutris.util.log import logger
+from lutris.util.strings import time_ago
 
 API_KEY_FILE_PATH = os.path.join(settings.CACHE_DIR, "auth-token")
 USER_INFO_FILE_PATH = os.path.join(settings.CACHE_DIR, "user.json")
-USER_ICON_FILE_PATH = os.path.join(settings.CACHE_DIR, "user.png")
+
+
+def get_time_from_api_date(date_string):
+    """Convert a date string originating from the API and convert it to a datetime object"""
+    return time.strptime(date_string[:date_string.find(".")], "%Y-%m-%dT%H:%M:%S")
+
+
+def get_runtime_versions_date() -> float:
+    return os.path.getmtime(settings.RUNTIME_VERSIONS_PATH)
+
+
+def get_runtime_versions_date_time_ago() -> str:
+    try:
+        return time_ago(get_runtime_versions_date())
+    except FileNotFoundError:
+        return _("never")
+
+
+def check_stale_runtime_versions() -> bool:
+    """True if runtime versions file that download_runtime_versions() creates
+    is missing or stale; if true we must call that function."""
+    try:
+        threshold = time.time() + 6 * 60 * 60  # 6 hours from now
+        modified_at = get_runtime_versions_date()
+        return threshold < modified_at
+    except FileNotFoundError:
+        return True
+
+
+def download_runtime_versions() -> Dict[str, Any]:
+    """Queries runtime + runners + current client versions and stores the result
+    in a file; the mdate of this file is used to decide when it is stale and should
+    be replaced."""
+    gpus_info = get_gpus_info()
+    pci_ids = [" ".join([gpu["PCI_ID"], gpu["PCI_SUBSYS_ID"]]) for gpu in gpus_info.values()]
+
+    url = settings.SITE_URL + "/api/runtimes/versions?pci_ids=" + ",".join(pci_ids)
+    response = http.Request(url, headers={"Content-Type": "application/json"})
+    try:
+        response.get()
+    except http.HTTPError as ex:
+        logger.error("Unable to get runtimes from API: %s", ex)
+        return {}
+    with open(settings.RUNTIME_VERSIONS_PATH, mode="w", encoding="utf-8") as runtime_file:
+        json.dump(response.json, runtime_file, indent=2)
+    return response.json
+
+
+def get_runtime_versions() -> Dict[str, Any]:
+    """Load runtime versions from the json file that is created at startup
+    if it is missing or stale."""
+    if not system.path_exists(settings.RUNTIME_VERSIONS_PATH):
+        return {}
+    with open(settings.RUNTIME_VERSIONS_PATH, mode="r", encoding="utf-8") as runtime_file:
+        return json.load(runtime_file)
 
 
 def read_api_key():
@@ -77,6 +138,7 @@ def get_user_info():
 
 def get_runners(runner_name):
     """Return the available runners for a given runner name"""
+    logger.debug("Retrieving runners")
     api_url = settings.SITE_URL + "/api/runners/" + runner_name
     host = settings.SITE_URL.split("//")[1]
 
@@ -91,6 +153,84 @@ def get_runners(runner_name):
     return response.json()
 
 
+def download_runner_versions(runner_name: str) -> list:
+    try:
+        request = Request("{}/api/runners/{}".format(settings.SITE_URL, runner_name))
+        runner_info = request.get().json
+        if not runner_info:
+            logger.error("Failed to get runner information")
+    except HTTPError as ex:
+        logger.error("Unable to get runner information: %s", ex)
+        runner_info = None
+    if not runner_info:
+        return []
+    versions = runner_info.get("versions") or []
+    return versions
+
+
+def format_runner_version(version_info: Dict[str, str]) -> str:
+    version = version_info.get("version")
+    if not version:
+        return ""
+    arch = version_info.get("architecture")
+    if arch:
+        return "{}-{}".format(version, arch)
+
+    return version
+
+
+def get_default_runner_version_info(runner_name: str, version: str = None) -> Dict[str, str]:
+    """Get the appropriate version for a runner
+
+    Params:
+        version: Optional version to lookup, will return this one if found
+
+    Returns:
+        Dict containing version, architecture and url for the runner, an empty dict
+        if the data can't be retrieved. If a pseudo-version is accepted, may be
+        a dict containing only the version itself.
+    """
+
+    if not version:
+        runtime_versions = get_runtime_versions()
+        if runtime_versions:
+            try:
+                runner_versions = runtime_versions["runners"][runner_name]
+            except KeyError:
+                runner_versions = []
+            for runner_version in runner_versions:
+                if runner_version["architecture"] in (LINUX_SYSTEM.arch, "all"):
+                    return runner_version
+    logger.info(
+        "Getting runner information for %s%s",
+        runner_name,
+        " (version: %s)" % version if version else "",
+    )
+    arch = LINUX_SYSTEM.arch
+    versions = download_runner_versions(runner_name)
+    # Please someone clean up the abomination that is the code below.
+    if version:
+        if version.endswith("-i386") or version.endswith("-x86_64"):
+            version, arch = version.rsplit("-", 1)
+        versions = [v for v in versions if v["version"] == version]
+    versions_for_arch = [v for v in versions if v["architecture"] == arch]
+    if len(versions_for_arch) == 1:
+        return versions_for_arch[0]
+
+    if len(versions_for_arch) > 1:
+        default_version = [v for v in versions_for_arch if v["default"] is True]
+        if default_version:
+            return default_version[0]
+    elif len(versions) > 1 and LINUX_SYSTEM.is_64_bit:
+        default_version = [v for v in versions if v["default"] is True]
+        if default_version:
+            return default_version[0]
+    # If we didn't find a proper version yet, return the first available.
+    if len(versions_for_arch) >= 1:
+        return versions_for_arch[0]
+    return {}
+
+
 def get_http_post_response(url, payload):
     response = http.Request(url, headers={"Content-Type": "application/json"})
     try:
@@ -183,7 +323,7 @@ def get_game_installers(game_slug, revision=None):
     return [normalize_installer(i) for i in installers]
 
 
-def get_game_details(slug):
+def get_game_details(slug: str) -> dict:
     url = settings.SITE_URL + "/api/games/%s" % slug
     request = http.Request(url)
     try:
@@ -194,9 +334,10 @@ def get_game_details(slug):
     return response.json
 
 
-def normalize_installer(installer):
+def normalize_installer(installer: dict) -> dict:
     """Adjusts an installer dict so it is in the correct form, with values
     of the expected types."""
+
     def must_be_str(key):
         if key in installer:
             installer[key] = str(installer[key])
@@ -211,7 +352,7 @@ def normalize_installer(installer):
     return installer
 
 
-def search_games(query):
+def search_games(query) -> dict:
     if not query:
         return {}
     query = query.lower().strip()[:255]
@@ -225,19 +366,6 @@ def search_games(query):
     return response.json
 
 
-def get_bundle(bundle):
-    """Retrieve a lutris bundle from the API"""
-    url = "/api/bundles/%s" % bundle
-    response = http.Request(settings.SITE_URL + url, headers={"Content-Type": "application/json"})
-    try:
-        response.get()
-    except http.HTTPError as ex:
-        logger.error("Unable to get bundle from API: %s", ex)
-        return None
-    response_data = response.json
-    return response_data.get("games", [])
-
-
 def parse_installer_url(url):
     """
     Parses `lutris:` urls, extracting any info necessary to install or run a game.
@@ -246,14 +374,14 @@ def parse_installer_url(url):
     launch_config_name = None
     try:
         parsed_url = urllib.parse.urlparse(url, scheme="lutris")
-    except Exception:  # pylint: disable=broad-except
+    except Exception as ex:
         logger.warning("Unable to parse url %s", url)
-        return False
+        raise ValueError("Invalid lutris url %s" % url) from ex
     if parsed_url.scheme != "lutris":
-        return False
+        raise ValueError("Invalid lutris url %s" % url)
     url_path = parsed_url.path
     if not url_path:
-        return False
+        raise ValueError("Invalid lutris url %s" % url)
     # urlparse can't parse if the path only contain numbers
     # workaround to remove the scheme manually:
     if url_path.startswith("lutris:"):
