@@ -7,6 +7,7 @@ import shlex
 import subprocess
 import sys
 import uuid
+from copy import copy
 
 from gi.repository import GLib
 
@@ -36,7 +37,6 @@ WRAPPER_SCRIPT = get_wrapper_script_location()
 
 
 class MonitoredCommand:
-
     """Exexcutes a commmand while keeping track of its state"""
 
     fallback_cwd = "/tmp"
@@ -98,8 +98,6 @@ class MonitoredCommand:
             return wrapper_command + self.command
 
         terminal_path = system.find_executable(self.terminal)
-        if not terminal_path:
-            raise RuntimeError("Couldn't find terminal %s" % self.terminal)
         script_path = get_terminal_script(self.command, self.cwd, self.env)
         return wrapper_command + [terminal_path, "-e", script_path]
 
@@ -120,25 +118,39 @@ class MonitoredCommand:
     @staticmethod
     def get_environment(user_env):
         """Process the user provided environment variables for use as self.env"""
-        env = user_env or {}
+        env = copy(user_env) if user_env else {}
+
         # not clear why this needs to be added, the path is already added in
         # the wrappper script.
         env['PYTHONPATH'] = ':'.join(sys.path)
         # Drop bad values of environment keys, those will confuse the Python
         # interpreter.
         env["LUTRIS_GAME_UUID"] = str(uuid.uuid4())
-        return {key: value for key, value in env.items() if "=" not in key}
+
+        cleaned = {}
+        for key, value in env.items():
+            if "=" in key:
+                logger.warning("Environment variable name '%s' contains '=' so it can't be used; skipping.", key)
+            elif value is None:
+                logger.warning("Environment variable '%s' has None for its value; skipping.", key)
+            elif not isinstance(value, str):
+                logger.warning("Environment variable '%s' value '%s' is not a string; converting.", key, value)
+                cleaned[key] = str(value)
+            else:
+                cleaned[key] = value
+        return cleaned
 
     def get_child_environment(self):
         """Returns the calculated environment for the child process."""
-        env = os.environ.copy()
+        env = system.get_environment()
         env.update(self.env)
         return env
 
     def start(self):
         """Run the thread."""
-        for key, value in self.env.items():
-            logger.debug("%s=\"%s\"", key, value)
+        if os.environ.get("LUTRIS_DEBUG_ENV") == "1":
+            for key, value in self.env.items():
+                logger.debug("%s=\"%s\"", key, value)
         wrapper_command = self.get_wrapper_command()
         env = self.get_child_environment()
         self.game_process = self.execute_process(wrapper_command, env)
@@ -159,16 +171,28 @@ class MonitoredCommand:
             self.on_stdout_output,
         )
 
+    def log_filter(self, line):
+        """Filter out some message we don't want to show to the user."""
+        if "GStreamer-WARNING **" in line:
+            return False
+        if "Bad file descriptor" in line:
+            return False
+        return True
+
     def log_handler_stdout(self, line):
         """Add the line to this command's stdout attribute"""
+        if not self.log_filter(line):
+            return
         self._stdout.write(line)
 
     def log_handler_buffer(self, line):
         """Add the line to the associated LogBuffer object"""
         self.log_buffer.insert(self.log_buffer.get_end_iter(), line, -1)
 
-    def log_handler_console_output(self, line):  # pylint: disable=no-self-use
+    def log_handler_console_output(self, line):
         """Print the line to stdout"""
+        if not self.log_filter(line):
+            return
         with contextlib.suppress(BlockingIOError):
             sys.stdout.write(line)
             sys.stdout.flush()
