@@ -102,6 +102,15 @@ def get_game_for_service(service, appid):
         return existing_games[0]
 
 
+def get_all_installed_game_for_service(service):
+    if service == "lutris":
+        db_games = get_games(filters={"installed": 1})
+        return {g["slug"]: g for g in db_games}
+
+    db_games = get_games(filters={"service": service, "installed": 1})
+    return {g["service_id"]: g for g in db_games}
+
+
 def get_service_games(service):
     """Return the list of all installed games for a service"""
     global _SERVICE_CACHE_ACCESSED
@@ -199,7 +208,7 @@ def get_matching_game(params):
             if game["configpath"] == params.get("configpath"):
                 return game["id"]
         else:
-            if (game["runner"] == params.get("runner") or not all([params.get("runner"), game["runner"]])):
+            if game["runner"] == params.get("runner") or not all([params.get("runner"), game["runner"]]):
                 return game["id"]
     return None
 
@@ -230,24 +239,6 @@ def get_used_platforms():
     return [result[0] for result in results if result[0]]
 
 
-def get_unusued_game_name(game_name):
-    """Returns the given name, but if this name is already used by an installed
-    game, this adds a number to it to make it unique."""
-    def is_name_in_use(name):
-        """Queries the database to see if a given is in use by an installed
-        game."""
-        existing_game = get_game_by_field(assigned_name, "name")
-        return existing_game and existing_game["installed"]
-
-    assigned_name = game_name
-    assigned_index = 1
-    while is_name_in_use(assigned_name):
-        assigned_index += 1
-        assigned_name = f"{game_name} {assigned_index}"
-
-    return assigned_name
-
-
 def get_game_count(param, value):
     res = sql.db_select(settings.PGA_DB, "games", fields=("COUNT(id)",), condition=(param, value))
     if res:
