@@ -3,8 +3,7 @@ from gettext import gettext as _
 
 from gi.repository import Gio, GLib, Gtk
 
-from lutris import api
-from lutris.exceptions import watch_errors
+from lutris import api, sysoptions
 from lutris.gui.config.add_game_dialog import AddGameDialog
 from lutris.gui.dialogs import ErrorDialog, ModelessDialog
 from lutris.gui.dialogs.game_import import ImportGameDialog
@@ -30,8 +29,8 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
         (
             "folder-new-symbolic",
             "go-next-symbolic",
-            _("Scan a folder for games"),
-            _("Mass-import a folder of games"),
+            _("Import previously installed Lutris games"),
+            _("Scan a folder for games installed from a previous Lutris installation"),
             "scan_folder"
         ),
         (
@@ -125,6 +124,8 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
         self.install_from_setup_game_slug_entry = Gtk.Entry(sensitive=False)
         self.installer_presets = Gtk.ListStore(str, str)
         self.install_preset_dropdown = Gtk.ComboBox.new_with_model(self.installer_presets)
+        self.installer_locale = Gtk.ListStore(str, str)
+        self.install_locale_dropdown = Gtk.ComboBox.new_with_model(self.installer_locale)
 
         self.install_script_file_chooser = FileChooserEntry(
             title=_("Select script"), action=Gtk.FileChooserAction.OPEN
@@ -147,21 +148,15 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
 
         self.load_initial_page()
 
-    @watch_errors()
     def on_back_clicked(self, _widget):
         self.stack.navigate_back()
 
-    @watch_errors()
     def on_navigate_home(self, _accel_group, _window, _keyval, _modifier):
         self.stack.navigate_home()
 
-    @watch_errors()
     def on_cancel_clicked(self, _widget):
         self.destroy()
 
-    def on_watched_error(self, error):
-        ErrorDialog(str(error), parent=self)
-
     # Initial Page
 
     def load_initial_page(self):
@@ -185,7 +180,6 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
         self.stack.present_page("initial")
         self.display_cancel_button()
 
-    @watch_errors()
     def on_row_activated(self, listbox, row):
         if row.callback_name:
             callback = getattr(self, row.callback_name)
@@ -243,14 +237,12 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
         self.search_entry.grab_focus()
         self.display_cancel_button()
 
-    @watch_errors()
     def _on_search_updated(self, entry):
         if self.search_timer_id:
             GLib.source_remove(self.search_timer_id)
         self.text_query = entry.get_text().strip()
         self.search_timer_id = GLib.timeout_add(750, self.update_search_results)
 
-    @watch_errors()
     def update_search_results(self):
         # Don't start a search while another is going; defer it instead.
         if self.search_spinner.get_visible():
@@ -264,7 +256,6 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
             self.search_spinner.start()
             AsyncCall(api.search_games, self.update_search_results_cb, self.text_query)
 
-    @watch_errors()
     def update_search_results_cb(self, api_games, error):
         if error:
             raise error
@@ -277,9 +268,9 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
         if not count:
             self.search_result_label.set_markup(_("No results"))
         elif count == total_count:
-            self.search_result_label.set_markup(_(f"Showing <b>{count}</b> results"))
+            self.search_result_label.set_markup(_("Showing <b>%s</b> results") % count)
         else:
-            self.search_result_label.set_markup(_(f"<b>{total_count}</b> results, only displaying first {count}"))
+            self.search_result_label.set_markup(_("<b>%s</b> results, only displaying first {count}") % total_count)
         for row in self.search_listbox.get_children():
             row.destroy()
         for game in api_games.get("results", []):
@@ -295,12 +286,10 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
         self.search_frame.show()
         self.search_explanation_label.hide()
 
-    @watch_errors()
     def _on_game_selected(self, listbox, row):
         game_slug = row.api_info["slug"]
-        installers = get_installers(game_slug=game_slug)
         application = Gio.Application.get_default()
-        application.show_installer_window(installers)
+        application.show_lutris_installer_window(game_slug=game_slug)
         self.destroy()
 
     # Scan Folder Page
@@ -317,10 +306,10 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
         self.scan_directory_chooser.set_hexpand(True)
 
         explanation = _(
-            "Lutris will search this folder for sub-folders that contain games it recognizes.\n\n"
-            "Any games it finds that are not already in Lutris will be added.\n\n"
-            "When you click 'Continue' below, the search will begin, and any games found will "
-            "be added at once."
+            "This folder will be scanned for games previously installed with Lutris.\n\n"
+            "Folder names have to match their corresponding Lutris ID, each matching ID"
+            "will be queried for existing install script to provide for exe locations.\n\n"
+            "Click 'Continue' to start scanning and import games"
         )
 
         grid.attach(self._get_explanation_label(explanation), 0, 1, 2, 1)
@@ -331,7 +320,6 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
         self.stack.present_page("scan_folder")
         self.display_continue_button(self.on_continue_scan_folder_clicked)
 
-    @watch_errors()
     def on_continue_scan_folder_clicked(self, _widget):
         path = self.scan_directory_chooser.get_text()
         if not path:
@@ -357,7 +345,6 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
         spinner.start()
         return spinner
 
-    @watch_errors()
     def _on_folder_scanned(self, result, error):
         def present_installed_games_page():
             if installed or missing:
@@ -370,7 +357,7 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
             self.display_cancel_button(label=_("_Close"))
 
         if error:
-            ErrorDialog(str(error), parent=self)
+            ErrorDialog(error, parent=self)
             self.stack.navigation_reset()
             return
 
@@ -457,6 +444,23 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
         grid.attach(self.install_preset_dropdown, 1, 3, 1, 1)
         self.install_preset_dropdown.set_halign(Gtk.Align.START)
 
+        locale_label = Gtk.Label(_("Locale:"), visible=True)
+        locale_label.set_xalign(0)
+        grid.attach(locale_label, 0, 4, 1, 1)
+
+        locale_list = sysoptions.get_locale_choices()
+        for locale_humanized, locale in locale_list:
+            self.installer_locale.append([locale, _(locale_humanized)])
+
+        locale_renderer_text = Gtk.CellRendererText()
+        self.install_locale_dropdown.pack_start(locale_renderer_text, True)
+        self.install_locale_dropdown.add_attribute(locale_renderer_text, "text", 1)
+        self.install_locale_dropdown.set_id_column(0)
+        self.install_locale_dropdown.set_active(0)
+
+        grid.attach(self.install_locale_dropdown, 1, 4, 1, 1)
+        self.install_locale_dropdown.set_halign(Gtk.Align.START)
+
         grid.set_vexpand(True)
         return grid
 
@@ -465,19 +469,16 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
         self.stack.present_page("install_from_setup")
         self.display_continue_button(self._on_install_setup_continue, label=_("_Install"))
 
-    @watch_errors()
     def on_install_from_setup_game_slug_toggled(self, checkbutton):
         self.install_from_setup_game_slug_entry.set_sensitive(checkbutton.get_active())
         self.on_install_from_setup_game_name_changed()
 
-    @watch_errors()
     def on_install_from_setup_game_name_changed(self, *_args):
         if not self.install_from_setup_game_slug_checkbox.get_active():
             name = self.install_from_setup_game_name_entry.get_text()
             proposed_slug = slugify(name) if name else ""
             self.install_from_setup_game_slug_entry.set_text(proposed_slug)
 
-    @watch_errors()
     def _on_install_setup_continue(self, button):
         name = self.install_from_setup_game_name_entry.get_text().strip()
 
@@ -498,6 +499,8 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
         else:
             win_ver_task = None
 
+        locale_selected = self.installer_locale[self.install_locale_dropdown.get_active()][0]
+
         installer = {
             "name": name,
             "version": _("Setup file"),
@@ -513,7 +516,12 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
                 ],
                 "installer": [
                     {"task": {"name": "wineexec", "executable": "setupfile", "arch": arch}}
-                ]
+                ],
+                "system": {
+                    "env": {
+                        "LC_ALL": locale_selected
+                    }
+                }
             }
         }
         if win_ver_task:
@@ -553,7 +561,6 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
         self.stack.present_page("install_from_script")
         self.display_continue_button(self.on_continue_install_from_script_clicked, label=_("_Install"))
 
-    @watch_errors()
     def on_continue_install_from_script_clicked(self, _widget):
         path = self.install_script_file_chooser.get_text()
         if not path:
@@ -595,7 +602,6 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
         self.stack.present_page("import_rom")
         self.display_continue_button(self.on_continue_import_rom_clicked, label=_("_Install"))
 
-    @watch_errors()
     def on_continue_import_rom_clicked(self, _widget):
         path = self.import_rom_file_chooser.get_text()
         if not path:
@@ -612,8 +618,14 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
 
     def add_local_game(self):
         """Manually configure game"""
-        AddGameDialog(parent=self)
-        GLib.idle_add(self.destroy)  # defer destory so the game dialog can be centered first
+        # We use the LutrisWindow as the parent because we would
+        # destroy this window before the AddGameDialog could disconnect.
+        # We've tried to be clever here, but it didn't work reliably.
+        # This does center the AddGameDialog over the main window, which
+        # isn't terrible.
+        application = Gio.Application.get_default()
+        AddGameDialog(parent=application.window)
+        self.destroy()
 
     # Subtitle Label
 
