@@ -2,7 +2,7 @@ from gettext import gettext as _
 
 from gi.repository import Gtk
 
-from lutris.cache import get_cache_path, save_cache_path
+from lutris.cache import get_cache_path, has_custom_cache_path, save_custom_cache_path
 from lutris.gui.dialogs import ModalDialog
 from lutris.gui.widgets.common import FileChooserEntry
 
@@ -18,7 +18,7 @@ class CacheConfigurationDialog(ModalDialog):
         self.timer_id = None
         self.set_size_request(480, 150)
 
-        self.cache_path = get_cache_path() or ""
+        self.cache_path = get_cache_path() if has_custom_cache_path() else ""
         self.get_content_area().add(self.get_cache_config())
 
         self.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
@@ -28,7 +28,7 @@ class CacheConfigurationDialog(ModalDialog):
         result = self.run()
 
         if result == Gtk.ResponseType.OK:
-            save_cache_path(self.cache_path)
+            save_custom_cache_path(self.cache_path)
 
         self.destroy()
 
@@ -42,7 +42,8 @@ class CacheConfigurationDialog(ModalDialog):
         path_chooser = FileChooserEntry(
             title=_("Set the folder for the cache path"),
             action=Gtk.FileChooserAction.SELECT_FOLDER,
-            path=self.cache_path,
+            warn_if_non_writable_parent=True,
+            text=self.cache_path,
             activates_default=True
         )
         path_chooser.connect("changed", self._on_cache_path_set)
