@@ -1,20 +1,16 @@
 """GUI dialog for reporting issues"""
-# Standard Library
 import json
 import os
 from gettext import gettext as _
 
-# Third Party Libraries
 from gi.repository import Gtk
 
-# Lutris Modules
 from lutris.gui.dialogs import NoticeDialog
 from lutris.gui.widgets.window import BaseApplicationWindow
 from lutris.util.linux import gather_system_info
 
 
 class IssueReportWindow(BaseApplicationWindow):
-
     """Window for collecting and sending issue reports"""
 
     def __init__(self, application):
@@ -49,7 +45,7 @@ class IssueReportWindow(BaseApplicationWindow):
         action_buttons_alignment.add(self.action_buttons)
         self.vbox.pack_start(action_buttons_alignment, False, True, 0)
 
-        cancel_button = self.get_action_button(_("C_ancel"), handler=self.on_destroy)
+        cancel_button = self.get_action_button(_("C_ancel"), handler=lambda *x: self.destroy())
         self.action_buttons.add(cancel_button)
 
         save_button = self.get_action_button(_("_Save"), handler=self.on_save)
