@@ -1,9 +1,9 @@
 """Main window for the Lutris interface."""
 # pylint:disable=too-many-lines
 import os
-import re
 from collections import namedtuple
 from gettext import gettext as _
+from typing import List
 from urllib.parse import unquote, urlparse
 
 from gi.repository import Gdk, Gio, GLib, GObject, Gtk
@@ -12,14 +12,14 @@ from lutris import services, settings
 from lutris.database import categories as categories_db
 from lutris.database import games as games_db
 from lutris.database.services import ServiceGameCollection
-from lutris.exceptions import watch_errors
+from lutris.exceptions import EsyncLimitError, FsyncUnsupportedError
 from lutris.game import Game
 from lutris.gui import dialogs
 from lutris.gui.addgameswindow import AddGamesWindow
 from lutris.gui.config.preferences_dialog import PreferencesDialog
 from lutris.gui.dialogs.delegates import DialogInstallUIDelegate, DialogLaunchUIDelegate
 from lutris.gui.dialogs.game_import import ImportGameDialog
-from lutris.gui.views import COL_ID, COL_NAME
+from lutris.gui.download_queue import DownloadQueue
 from lutris.gui.views.grid import GameGridView
 from lutris.gui.views.list import GameListView
 from lutris.gui.views.store import GameStore
@@ -27,13 +27,17 @@ from lutris.gui.widgets.game_bar import GameBar
 from lutris.gui.widgets.gi_composites import GtkTemplate
 from lutris.gui.widgets.sidebar import LutrisSidebar
 from lutris.gui.widgets.utils import load_icon_theme, open_uri
+from lutris.runtime import ComponentUpdater, RuntimeUpdater
 from lutris.scanners.lutris import add_to_path_cache, get_missing_game_ids, remove_from_path_cache
 # pylint: disable=no-member
 from lutris.services.base import BaseService
 from lutris.services.lutris import LutrisService
 from lutris.util import datapath
+from lutris.util.jobs import AsyncCall
 from lutris.util.log import logger
+from lutris.util.strings import get_natural_sort_key
 from lutris.util.system import update_desktop_icons
+from lutris.util.wine.wine import esync_display_limit_warning, fsync_display_support_warning
 
 
 @GtkTemplate(ui=os.path.join(datapath.get(), "ui", "lutris-window.ui"))
@@ -50,7 +54,6 @@ class LutrisWindow(Gtk.ApplicationWindow,
     __gsignals__ = {
         "view-updated": (GObject.SIGNAL_RUN_FIRST, None, ()),
     }
-
     games_stack = GtkTemplate.Child()
     sidebar_revealer = GtkTemplate.Child()
     sidebar_scrolled = GtkTemplate.Child()
@@ -59,6 +62,7 @@ class LutrisWindow(Gtk.ApplicationWindow,
     zoom_adjustment = GtkTemplate.Child()
     blank_overlay = GtkTemplate.Child()
     viewtype_icon = GtkTemplate.Child()
+    download_revealer: Gtk.Revealer = GtkTemplate.Child()
 
     def __init__(self, application, **kwargs):
         width = int(settings.read_setting("width") or self.default_width)
@@ -86,6 +90,7 @@ class LutrisWindow(Gtk.ApplicationWindow,
         self.set_service(self.filters.get("service"))
         self.icon_type = self.load_icon_type()
         self.game_store = GameStore(self.service, self.service_media)
+        self._game_store_generation = 0
         self.current_view = Gtk.Box()
         self.views = {}
 
@@ -99,9 +104,9 @@ class LutrisWindow(Gtk.ApplicationWindow,
         self.connect("configure-event", self.on_window_configure)
         self.connect("realize", self.on_load)
         self.connect("drag-data-received", self.on_drag_data_received)
+        self.connect("notify::visible", self.on_visible_changed)
         if self.maximized:
             self.maximize()
-
         self.init_template()
         self._init_actions()
 
@@ -129,7 +134,7 @@ class LutrisWindow(Gtk.ApplicationWindow,
         self.game_bar = None
         self.revealer_box = Gtk.HBox(visible=True)
         self.game_revealer.add(self.revealer_box)
-        self.get_missing_games()
+
         self.update_action_state()
 
         self.connect("view-updated", self.update_store)
@@ -145,7 +150,7 @@ class LutrisWindow(Gtk.ApplicationWindow,
 
         # Finally trigger the initialization of the view here
         selected_category = settings.read_setting("selected_category", default="runner:all")
-        self.sidebar.selected_category = selected_category.split(":") if selected_category else None
+        self.sidebar.selected_category = selected_category.split(":", maxsplit=1) if selected_category else None
 
     def _init_actions(self):
         Action = namedtuple("Action", ("callback", "type", "enabled", "default", "accel"))
@@ -173,19 +178,19 @@ class LutrisWindow(Gtk.ApplicationWindow,
                 self.on_view_sorting_state_change,
                 type="s",
                 default=self.view_sorting,
-                enabled=lambda: self.is_view_sort_active
+                enabled=lambda: self.is_view_sort_sensitive
             ),
             "view-sorting-installed-first": Action(
                 self.on_view_sorting_installed_first_change,
                 type="b",
                 default=self.view_sorting_installed_first,
-                enabled=lambda: self.is_view_sort_active
+                enabled=lambda: self.is_view_sort_sensitive
             ),
             "view-sorting-ascending": Action(
                 self.on_view_sorting_direction_change,
                 type="b",
                 default=self.view_sorting_ascending,
-                enabled=lambda: self.is_view_sort_active
+                enabled=lambda: self.is_view_sort_sensitive
             ),
             "show-side-panel": Action(
                 self.on_side_panel_state_change,
@@ -197,6 +202,7 @@ class LutrisWindow(Gtk.ApplicationWindow,
                 self.hidden_state_change,
                 type="b",
                 default=self.show_hidden_games,
+                enabled=lambda: self.is_show_hidden_sensitive,
                 accel="<Primary>h",
             ),
             "open-forums": Action(lambda *x: open_uri("https://forums.lutris.net/")),
@@ -224,6 +230,7 @@ class LutrisWindow(Gtk.ApplicationWindow,
             if value.enabled:
                 def updater(action=action, value=value):
                     action.props.enabled = value.enabled()
+
                 self.action_state_updaters.append(updater)
             self.add_action(action)
             if value.accel:
@@ -246,13 +253,14 @@ class LutrisWindow(Gtk.ApplicationWindow,
     def on_load(self, widget, data=None):
         """Finish initializing the view"""
         self._bind_zoom_adjustment()
+        AsyncCall(get_missing_game_ids, self.on_get_missing_game_ids)
         self.current_view.grab_focus()
 
     def on_sidebar_realize(self, widget, data=None):
         """Grab the initial focus after the sidebar is initialized - so the view is ready."""
         self.current_view.grab_focus()
 
-    def on_drag_data_received(self, widget, drag_context, x, y, data, info, time):
+    def on_drag_data_received(self, _widget, _drag_context, _x, _y, data, _info, _time):
         """Handler for drop event"""
         file_paths = [unquote(urlparse(uri).path) for uri in data.get_uris()]
         dialog = ImportGameDialog(file_paths, parent=self)
@@ -267,6 +275,11 @@ class LutrisWindow(Gtk.ApplicationWindow,
         }
         return filters
 
+    @property
+    def is_show_hidden_sensitive(self):
+        """True if the hidden checkbox will be effective; service views ignore it."""
+        return not self.filters.get("service")
+
     def hidden_state_change(self, action, value):
         """Hides or shows the hidden games"""
         action.set_state(value)
@@ -323,8 +336,13 @@ class LutrisWindow(Gtk.ApplicationWindow,
         if self.view_sorting_installed_first:
             params.append(("installed", "COLLATE NOCASE DESC"))
 
+        if self.view_sorting == "name":
+            key = "CASE WHEN sortname <> '' THEN sortname ELSE name END"
+        else:
+            key = self.view_sorting
+
         params.append((
-            self.view_sorting,
+            key,
             "COLLATE NOCASE ASC"
             if self.view_sorting_ascending
             else "COLLATE NOCASE DESC"
@@ -333,7 +351,7 @@ class LutrisWindow(Gtk.ApplicationWindow,
         return params
 
     @property
-    def is_view_sort_active(self):
+    def is_view_sort_sensitive(self):
         """True if the iew sorting options will be effective; dynamic categories ignore them."""
         return self.filters.get("dynamic_category") not in self.dynamic_categories_game_factories
 
@@ -359,13 +377,6 @@ class LutrisWindow(Gtk.ApplicationWindow,
             "playtime": 0.0,
         }
 
-        def natural_sort_key(value):
-            def pad_numbers(text):
-                return text.zfill(16) if text.isdigit() else text
-
-            key = [pad_numbers(c) for c in re.split('([0-9]+)', value)]
-            return key
-
         def get_sort_value(item):
             db_game = resolver(item)
             if not db_game:
@@ -373,10 +384,17 @@ class LutrisWindow(Gtk.ApplicationWindow,
                 value = sort_defaults.get(view_sorting, "")
             else:
                 installation_flag = bool(db_game.get("installed"))
-                value = db_game.get(view_sorting)
+
+                # When sorting by name, check for a valid sortname first, then fall back
+                # on name if valid sortname is not available.
+                sortname = db_game.get("sortname")
+                if view_sorting == "name" and sortname:
+                    value = sortname
+                else:
+                    value = db_game.get(view_sorting)
 
                 if view_sorting == "name":
-                    value = natural_sort_key(value)
+                    value = get_natural_sort_key(value)
 
             # Users may have obsolete view_sorting settings, so
             # we must tolerate them. We treat them all as blank.
@@ -396,10 +414,17 @@ class LutrisWindow(Gtk.ApplicationWindow,
 
     def get_running_games(self):
         """Return a list of currently running games"""
-        return games_db.get_games_by_ids([game.id for game in self.application.running_games])
+        return games_db.get_games_by_ids(self.application.get_running_game_ids())
 
-    def get_missing_games(self):
-        missing_ids = get_missing_game_ids()
+    def on_get_missing_game_ids(self, missing_ids, error):
+        if error:
+            logger.error(str(error))
+            return
+        self.get_missing_games(missing_ids)
+
+    def get_missing_games(self, missing_ids: list = None) -> list:
+        if missing_ids is None:
+            missing_ids = get_missing_game_ids()
         missing_games = games_db.get_games_by_ids(missing_ids)
         if missing_games:
             self.sidebar.missing_row.show()
@@ -448,10 +473,10 @@ class LutrisWindow(Gtk.ApplicationWindow,
                 service_game[field] = lutris_game[field]
         return service_game
 
-    def get_service_games(self, service_name):
-        """Switch the current service to service_name and return games if available"""
-        service_games = ServiceGameCollection.get_for_service(service_name)
-        if service_name == "lutris":
+    def get_service_games(self, service_id):
+        """Return games for the service indicated."""
+        service_games = ServiceGameCollection.get_for_service(service_id)
+        if service_id == "lutris":
             lutris_games = {g["slug"]: g for g in games_db.get_games()}
         else:
             lutris_games = {g["service_id"]: g for g in games_db.get_games(filters={"service": self.service.id})}
@@ -464,12 +489,12 @@ class LutrisWindow(Gtk.ApplicationWindow,
         ]
 
     def get_games_from_filters(self):
-        service_name = self.filters.get("service")
-        if service_name in services.SERVICES:
+        service_id = self.filters.get("service")
+        if service_id in services.SERVICES:
             if self.service.online and not self.service.is_authenticated():
                 self.show_label(_("Connect your %s account to access your games") % self.service.name)
                 return []
-            return self.get_service_games(service_name)
+            return self.get_service_games(service_id)
         if self.filters.get("dynamic_category") in self.dynamic_categories_game_factories:
             return self.dynamic_categories_game_factories[self.filters["dynamic_category"]]()
         if self.filters.get("category") and self.filters["category"] != "all":
@@ -513,13 +538,15 @@ class LutrisWindow(Gtk.ApplicationWindow,
             return medias[icon_type]()
         return medias[service.default_format]()
 
-    def update_revealer(self, game=None):
-        if game:
+    def update_revealer(self, games=None):
+        if games:
             if self.game_bar:
                 self.game_bar.destroy()
-
-            self.game_bar = GameBar(game, self.application, self)
-            self.revealer_box.pack_start(self.game_bar, True, True, 0)
+            if len(games) == 1 and games[0]:
+                self.game_bar = GameBar(games[0], self.application, self)
+                self.revealer_box.pack_start(self.game_bar, True, True, 0)
+            else:
+                self.game_bar = None
         elif self.game_bar:
             # The game bar can't be destroyed here because the game gets unselected on Wayland
             # whenever the game bar is interacted with. Instead, we keep the current game bar open
@@ -566,24 +593,69 @@ class LutrisWindow(Gtk.ApplicationWindow,
                 self.show_label(_("No games found"))
 
     def update_store(self, *_args, **_kwargs):
-        self.game_store.store.clear()
-        self.hide_overlay()
-        games = self.get_games_from_filters()
-        if games:
-            if len(games) > 1:
-                self.search_entry.set_placeholder_text(_("Search %s games") % len(games))
+        service_id = self.filters.get("service")
+        service = self.service
+        service_media = self.service_media
+        self._game_store_generation += 1
+        generation = self._game_store_generation
+
+        def make_game_store(games):
+            game_store = GameStore(service, service_media)
+            game_store.add_preloaded_games(games, service_id)
+            return games, game_store
+
+        def on_games_ready(games, error):
+            if generation != self._game_store_generation:
+                return  # no longer applicable, we got switched again!
+
+            if error:
+                raise error  # bounce any error against the backstop
+
+            # Since get_games_from_filters() seems to be much faster than making a GameStore,
+            # we defer the spinner to here, when we know how many games we will show. If there
+            # are "many" we show a spinner while the store is built.
+            if len(games) > 512:
+                self.show_spinner()
+
+            AsyncCall(make_game_store, apply_store, games)
+
+        def apply_store(result, error):
+            if generation != self._game_store_generation:
+                return  # no longer applicable, we got switched again!
+
+            if error:
+                raise error  # bounce any error against the backstop
+
+            games, game_store = result
+
+            if games:
+                if len(games) > 1:
+                    self.search_entry.set_placeholder_text(_("Search %s games") % len(games))
+                else:
+                    self.search_entry.set_placeholder_text(_("Search 1 game"))
             else:
-                self.search_entry.set_placeholder_text(_("Search 1 game"))
-        else:
-            self.search_entry.set_placeholder_text(_("Search games"))
-        for view in self.views.values():
-            view.service = self.service
-        GLib.idle_add(self.update_revealer)
-        for game in games:
-            self.game_store.add_game(game)
-        if not games:
-            self.show_empty_label()
+                self.search_entry.set_placeholder_text(_("Search games"))
+
+            for view in self.views.values():
+                view.service = self.service
+
+            GLib.idle_add(self.update_revealer)
+            self.game_store = game_store
+
+            view_type = self.current_view_type
+
+            if view_type in self.views:
+                self.current_view = self.views[view_type]
+                self.current_view.set_game_store(self.game_store)
+
+            if games:
+                self.hide_overlay()
+            else:
+                self.show_empty_label()
+
         self.search_timer_id = None
+
+        AsyncCall(self.get_games_from_filters, on_games_ready)
         return False
 
     def _bind_zoom_adjustment(self):
@@ -612,15 +684,6 @@ class LutrisWindow(Gtk.ApplicationWindow,
         if icon_type != self.icon_type:
             GLib.idle_add(self.save_icon_type, icon_type)
 
-    def show_overlay(self, widget, halign=Gtk.Align.FILL, valign=Gtk.Align.FILL):
-        """Display a widget in the blank overlay"""
-        for child in self.blank_overlay.get_children():
-            child.destroy()
-        self.blank_overlay.set_halign(halign)
-        self.blank_overlay.set_valign(valign)
-        self.blank_overlay.add(widget)
-        self.blank_overlay.props.visible = True
-
     def show_label(self, message):
         """Display a label in the middle of the UI"""
         self.show_overlay(Gtk.Label(message, visible=True))
@@ -641,15 +704,23 @@ class LutrisWindow(Gtk.ApplicationWindow,
         self.show_overlay(splash_box, Gtk.Align.FILL, Gtk.Align.FILL)
 
     def show_spinner(self):
-        spinner = Gtk.Spinner(visible=True)
+        spinner = Gtk.Spinner(visible=True, width_request=32, height_request=32)
         spinner.start()
+        self.show_overlay(spinner, halign=Gtk.Align.CENTER, valign=Gtk.Align.CENTER)
+
+    def show_overlay(self, widget, halign=Gtk.Align.FILL, valign=Gtk.Align.FILL):
+        """Display a widget in the blank overlay"""
         for child in self.blank_overlay.get_children():
             child.destroy()
-        self.blank_overlay.add(spinner)
-        self.blank_overlay.props.visible = True
+        self.blank_overlay.set_halign(halign)
+        self.blank_overlay.set_valign(valign)
+        self.blank_overlay.add(widget)
+        self.blank_overlay.show()
+        self.games_stack.hide()
 
     def hide_overlay(self):
-        self.blank_overlay.props.visible = False
+        self.blank_overlay.hide()
+        self.games_stack.show()
         for child in self.blank_overlay.get_children():
             child.destroy()
 
@@ -684,10 +755,16 @@ class LutrisWindow(Gtk.ApplicationWindow,
 
     def load_icon_type(self):
         """Return the icon style depending on the type of view."""
+        default_icon_types = {
+            "icon_type_grid": "coverart_med",
+        }
         setting_key = "icon_type_%sview" % self.current_view_type
         if self.service and self.service.id != "lutris":
             setting_key += "_%s" % self.service.id
-        self.icon_type = settings.read_setting(setting_key)
+        self.icon_type = settings.read_setting(
+            setting_key,
+            default=default_icon_types.get(setting_key, "")
+        )
         return self.icon_type
 
     def save_icon_type(self, icon_type):
@@ -704,18 +781,15 @@ class LutrisWindow(Gtk.ApplicationWindow,
         if not self.game_store:
             logger.error("No game store yet")
             return
-        self.game_store = GameStore(self.service, self.service_media)
 
         view_type = self.current_view_type
 
-        if view_type in self.views:
-            self.current_view = self.views[view_type]
-            self.current_view.set_game_store(self.game_store)
-        else:
+        if view_type not in self.views:
+            self.game_store = GameStore(self.service, self.service_media)
             if view_type == "grid":
                 self.current_view = GameGridView(
                     self.game_store,
-                    hide_text=settings.read_setting("hide_text_under_icons") == "True"
+                    hide_text=settings.read_bool_setting("hide_text_under_icons")
                 )
             else:
                 self.current_view = GameListView(self.game_store)
@@ -736,19 +810,23 @@ class LutrisWindow(Gtk.ApplicationWindow,
 
         self.update_view_settings()
         self.games_stack.set_visible_child_name(view_type)
-        self.update_store()
         self.update_action_state()
+        self.emit("view-updated")
 
     def rebuild_view(self, view_type):
         """Discards the view named by 'view_type' and if it is the current view,
         regenerates it. This is used to update view settings that can only be
         set during view construction, and not updated later."""
         if view_type in self.views:
+            view = self.views[view_type]
             scrolledwindow = self.games_stack.get_child_by_name(view_type)
-            scrolledwindow.remove(self.views[view_type])
-            del self.views["grid"]
+            scrolledwindow.remove(view)
+            del self.views[view_type]
             if self.current_view_type == view_type:
                 self.redraw_view()
+            # Because the view has hooks and such hooked up, it must be explicitly
+            # destroyed to disconnect everything.
+            view.destroy()
 
     def update_view_settings(self):
         if self.current_view and self.current_view_type == "grid":
@@ -794,7 +872,7 @@ class LutrisWindow(Gtk.ApplicationWindow,
 
     def _service_reloaded_cb(self, error):
         if error:
-            dialogs.ErrorDialog(str(error), parent=self)
+            dialogs.ErrorDialog(error, parent=self)
 
     def on_service_logout(self, service):
         if self.service and service.id == self.service.id:
@@ -815,9 +893,17 @@ class LutrisWindow(Gtk.ApplicationWindow,
         self.search_entry.set_size_request(min(max(50, size[0] - 470), 800), -1)
 
     def on_window_delete(self, *_args):
-        if self.application.running_games.get_n_items():
+        app = self.application
+        if app.running_games:
             self.hide()
             return True
+        if app.has_tray_icon():
+            self.hide()
+            return True
+
+    def on_visible_changed(self, window, param):
+        if self.application.tray:
+            self.application.tray.update_present_menu()
 
     def on_window_configure(self, *_args):
         """Callback triggered when the window is moved, resized..."""
@@ -841,7 +927,7 @@ class LutrisWindow(Gtk.ApplicationWindow,
     @GtkTemplate.Callback
     def on_preferences_activate(self, *_args):
         """Callback when preferences is activated."""
-        self.application.show_window(PreferencesDialog)
+        self.application.show_window(PreferencesDialog, parent=self)
 
     def on_show_installed_state_change(self, action, value):
         """Callback to handle uninstalled game filter switch"""
@@ -877,13 +963,19 @@ class LutrisWindow(Gtk.ApplicationWindow,
 
     def on_game_unhandled_error(self, game, error):
         """Called when a game has sent the 'game-error' signal"""
-        dialogs.ErrorDialog(str(error), parent=self)
+
+        if isinstance(error, FsyncUnsupportedError):
+            fsync_display_support_warning(parent=self)
+        elif isinstance(error, EsyncLimitError):
+            esync_display_limit_warning(parent=self)
+        else:
+            dialogs.ErrorDialog(error, parent=self)
         return True
 
     @GtkTemplate.Callback
     def on_add_game_button_clicked(self, *_args):
         """Add a new game manually with the AddGameDialog."""
-        self.application.show_window(AddGamesWindow)
+        self.application.show_window(AddGamesWindow, parent=self)
         return True
 
     def on_toggle_viewtype(self, *args):
@@ -928,6 +1020,8 @@ class LutrisWindow(Gtk.ApplicationWindow,
                 self.filters.pop(filter_type)
 
         row_type, row_id = widget.selected_category
+        if row_type == "user_category":
+            row_type = "category"
         self.filters[row_type] = row_id
 
         service_name = self.filters.get("service")
@@ -936,38 +1030,35 @@ class LutrisWindow(Gtk.ApplicationWindow,
         self.redraw_view()
 
     def on_game_selection_changed(self, view, selection):
-        if not selection:
-            GLib.idle_add(self.update_revealer)
-            return False
-        game_id = view.get_model().get_value(selection, COL_ID)
-        if not game_id:
-            GLib.idle_add(self.update_revealer)
-            return False
-        if self.service:
-            game = ServiceGameCollection.get_game(self.service.id, game_id)
-        else:
-            game = games_db.get_game_by_field(int(game_id), "id")
-        if not game:
-            game = {
-                "id": game_id,
-                "appid": game_id,
-                "name": view.get_model().get_value(selection, COL_NAME),
-                "slug": game_id,
-                "service": self.service.id if self.service else None,
-            }
-            logger.warning("No game found. Replacing with placeholder %s", game)
+        game_ids = [view.get_game_id_for_path(path) for path in selection]
 
-        GLib.idle_add(self.update_revealer, game)
+        if not game_ids:
+            GLib.idle_add(self.update_revealer)
+            return False
+
+        games = []
+        for game_id in game_ids:
+            if self.service:
+                game = ServiceGameCollection.get_game(self.service.id, game_id)
+            else:
+                game = games_db.get_game_by_field(game_id, "id")
+
+            # There can be no game found if you are removing a game; it will
+            # still have a selected icon in the UI just long enough to get here.
+            if game:
+                games.append(game)
+
+        GLib.idle_add(self.update_revealer, games)
         return False
 
     def on_toggle_badges(self, _widget, _data):
         """Event handler to toggle badge visibility"""
         state = settings.read_setting("hide_badges_on_icons").lower() == "true"
         settings.write_setting("hide_badges_on_icons", not state)
-        self.on_settings_changed(None, "hide_badges_on_icons")
+        self.on_settings_changed(None, not state, "hide_badges_on_icons")
 
-    def on_settings_changed(self, dialog, settings_key):
-        if settings_key == "hide_text_under_icons":
+    def on_settings_changed(self, dialog, state, setting_key):
+        if setting_key == "hide_text_under_icons":
             self.rebuild_view("grid")
         else:
             self.update_view_settings()
@@ -978,36 +1069,38 @@ class LutrisWindow(Gtk.ApplicationWindow,
         if game.is_hidden and not self.show_hidden_games:
             return False
 
-        # Stopped games do not get displayed on the running page
-        if game.state == game.STATE_STOPPED:
-            selected_row = self.sidebar.get_selected_row()
-            if selected_row and selected_row.id == "running":
+        row = self.sidebar.get_selected_row()
+
+        if row:
+            # Stopped games do not get displayed on the running page
+            if row.type == "dynamic_category" and row.id == "running" and game.state == game.STATE_STOPPED:
                 return False
+
+            # If the update took the row out of this view's category, we'll need
+            # to update the view to reflect that.
+            if row.type in ("category", "user_category"):
+                if row.id != "all" and row.id not in game.get_categories():
+                    return False
+
         return True
 
     def on_game_updated(self, game):
         """Updates an individual entry in the view when a game is updated"""
         add_to_path_cache(game)
-        if game.appid and self.service:
-            db_game = ServiceGameCollection.get_game(self.service.id, game.appid)
+        if self.service:
+            db_game = self.service.get_service_db_game(game)
         else:
             db_game = games_db.get_game_by_field(game.id, "id")
-        if not self.is_game_displayed(game):
-            self.game_store.remove_game(db_game["id"])
-            return True
+
+            if db_game and not self.is_game_displayed(game) and "id" in db_game:
+                self.game_store.remove_game(db_game["id"])
+                return True
+
         if db_game:
             updated = self.game_store.update(db_game)
             if not updated:
                 self.update_store()
-        else:
-            logger.debug("Can't get DB game for %s (service: %s)", game, self.service)
 
-        # If the update took the row out of this view's category, we'll need
-        # to update the view to reflect that.
-        selected_row = self.sidebar.get_selected_row()
-        if selected_row and selected_row.type == "category" and \
-                selected_row.id != "all" and selected_row.id not in game.get_categories():
-            self.game_store.remove_game(game.id)
         return True
 
     def on_game_stopped(self, game):
@@ -1029,26 +1122,17 @@ class LutrisWindow(Gtk.ApplicationWindow,
         self.emit("view-updated")
         return True
 
-    @watch_errors()
-    def on_game_activated(self, view, game_id):
+    def on_game_activated(self, _view, game_id):
         """Handles view activations (double click, enter press)"""
         if self.service:
             logger.debug("Looking up %s game %s", self.service.id, game_id)
             db_game = games_db.get_game_for_service(self.service.id, game_id)
-            if self.service.id == "lutris":
-                if not db_game or not db_game["installed"]:
-                    self.service.install(game_id)
-                    return
+
+            if db_game and db_game["installed"]:
                 game_id = db_game["id"]
             else:
-                if db_game and db_game["installed"]:
-                    game_id = db_game["id"]
-                else:
-                    service_game = ServiceGameCollection.get_game(self.service.id, game_id)
-                    if not service_game:
-                        logger.error("No game %s found for %s", game_id, self.service.id)
-                        return
-                    game_id = self.service.install(service_game)
+                game_id = self.service.install_by_id(game_id)
+
         if game_id:
             game = Game(game_id)
             if game.is_installed:
@@ -1056,5 +1140,50 @@ class LutrisWindow(Gtk.ApplicationWindow,
             else:
                 game.emit("game-install")
 
-    def on_watched_error(self, error):
-        dialogs.ErrorDialog(str(error), parent=self)
+    @property
+    def download_queue(self) -> DownloadQueue:
+        queue = self.download_revealer.get_child()
+        if not queue:
+            queue = DownloadQueue(self.download_revealer)
+            self.download_revealer.add(queue)
+        return queue
+
+    def start_runtime_updates(self, force_updates: bool) -> None:
+        """Starts the process of applying runtime updates, asynchronously. No UI appears until
+        we can determine that there are updates to perform."""
+
+        def create_runtime_updater():
+            runtime_updater = RuntimeUpdater(force=force_updates)
+            component_updaters = runtime_updater.create_component_updaters()
+            return component_updaters, runtime_updater
+
+        def create_runtime_updater_cb(result, error):
+            if error:
+                logger.exception("Failed to obtain updates from Lutris.net: %s", error)
+            else:
+                component_updaters, runtime_updater = result
+                if component_updaters:
+                    self.install_runtime_component_updates(component_updaters, runtime_updater)
+
+        AsyncCall(create_runtime_updater, create_runtime_updater_cb)
+
+    def install_runtime_component_updates(self, updaters: List[ComponentUpdater],
+                                          runtime_updater: RuntimeUpdater,
+                                          completion_function: DownloadQueue.CompletionFunction = None,
+                                          error_function: DownloadQueue.ErrorFunction = None) -> bool:
+        """Installs a list of component updates. This displays progress bars
+        in the sidebar as it installs updates, one at a time."""
+
+        queue = self.download_queue
+        operation_names = [f"component_update:{u.name}" for u in updaters]
+
+        def install_updates():
+            for updater in updaters:
+                updater.install_update(runtime_updater)
+            for updater in updaters:
+                updater.join()
+
+        return queue.start_multiple(install_updates, (u.get_progress for u in updaters),
+                                    completion_function=completion_function,
+                                    error_function=error_function,
+                                    operation_names=operation_names)
