@@ -1,16 +1,17 @@
 """Misc widgets used in the GUI."""
 # Standard Library
 import os
+import shlex
 import urllib.parse
 from gettext import gettext as _
 
 # Third Party Libraries
 from gi.repository import GLib, GObject, Gtk, Pango
 
+from lutris.gui.widgets.utils import open_uri
 # Lutris Modules
 from lutris.util import system
 from lutris.util.linux import LINUX_SYSTEM
-from lutris.util.log import logger
 
 
 class SlugEntry(Gtk.Entry, Gtk.Editable):
@@ -35,7 +36,6 @@ class NumberEntry(Gtk.Entry, Gtk.Editable):
 
 
 class FileChooserEntry(Gtk.Box):
-
     """Editable entry with a file picker button"""
 
     max_completion_items = 15  # Maximum number of items to display in the autocompletion dropdown.
@@ -48,12 +48,14 @@ class FileChooserEntry(Gtk.Box):
         self,
         title=_("Select file"),
         action=Gtk.FileChooserAction.OPEN,
-        path=None,
+        text=None,
         default_path=None,
         warn_if_non_empty=False,
+        warn_if_non_writable_parent=False,
         warn_if_ntfs=False,
         activates_default=False,
-    ):
+        shell_quoting=False
+    ):  # pylint: disable=too-many-arguments
         super().__init__(
             orientation=Gtk.Orientation.VERTICAL,
             spacing=0,
@@ -61,14 +63,18 @@ class FileChooserEntry(Gtk.Box):
         )
         self.title = title
         self.action = action
-        self.path = os.path.expanduser(path) if path else None
-        self.default_path = os.path.expanduser(default_path) if default_path else path
         self.warn_if_non_empty = warn_if_non_empty
+        self.warn_if_non_writable_parent = warn_if_non_writable_parent
         self.warn_if_ntfs = warn_if_ntfs
+        self.shell_quoting = shell_quoting
 
         self.path_completion = Gtk.ListStore(str)
 
         self.entry = Gtk.Entry(visible=True)
+        self.set_text(text)  # do before set up signal handlers
+        self.original_text = self.get_text()
+        self.default_path = os.path.expanduser(default_path) if default_path else self.get_path()
+
         self.entry.set_activates_default(activates_default)
         self.entry.set_completion(self.get_completion())
         self.entry.connect("changed", self.on_entry_changed)
@@ -76,29 +82,71 @@ class FileChooserEntry(Gtk.Box):
         self.entry.connect("focus-out-event", self.on_focus_out)
         self.entry.connect("backspace", self.on_backspace)
 
-        if path:
-            self.entry.set_text(path)
-
-        browse_button = Gtk.Button(_("Browse..."), visible=True)
+        browse_button = Gtk.Button.new_from_icon_name("view-more-horizontal-symbolic", Gtk.IconSize.BUTTON)
+        browse_button.show()
+        if action == Gtk.FileChooserAction.SELECT_FOLDER:
+            browse_button.set_tooltip_text(_("Select a folder"))
+        else:
+            browse_button.set_tooltip_text(_("Select a file"))
+        browse_button.get_style_context().add_class("circular")
         browse_button.connect("clicked", self.on_browse_clicked)
 
+        self.open_button = Gtk.Button.new_from_icon_name("folder-symbolic", Gtk.IconSize.BUTTON)
+        self.open_button.show()
+        self.open_button.set_tooltip_text(_("Open in file browser"))
+        self.open_button.get_style_context().add_class("circular")
+        self.open_button.connect("clicked", self.on_open_clicked)
+        self.open_button.set_sensitive(bool(self.get_open_directory()))
+
         box = Gtk.Box(spacing=6, visible=True)
         box.pack_start(self.entry, True, True, 0)
-        box.add(browse_button)
+        box.pack_end(self.open_button, False, False, 0)
+        box.pack_end(browse_button, False, False, 0)
         self.pack_start(box, False, False, 0)
 
-    def set_text(self, path):
-        self.path = os.path.expanduser(path)
-        self.entry.set_text(self.path)
+    def set_text(self, text):
+        if self.shell_quoting and text:
+            command_array = shlex.split(text)
+            if command_array:
+                expanded = os.path.expanduser(command_array[0])
+                command_array[0] = expanded
+                rejoined = shlex.join(command_array)
+                self.original_text = rejoined
+                self.entry.set_text(rejoined)
+                return
+
+        expanded = os.path.expanduser(text) if text else ""
+        self.original_text = expanded
+        self.entry.set_text(expanded)
+
+    def set_path(self, path):
+        if self.shell_quoting:
+            command_array = shlex.split(self.get_text())
+            if command_array:
+                command_array[0] = os.path.expanduser(path) if path else ""
+                rejoined = shlex.join(command_array)
+                self.original_text = rejoined
+                self.entry.set_text(rejoined)
+                return
+
+        expanded = os.path.expanduser(path) if path else ""
+        self.original_text = expanded
+        self.entry.set_text(expanded)
 
     def get_text(self):
-        """Return the entry's text"""
+        """Return the entry's text. If shell_quoting is one, this is actually a command
+        line (with argument quoting) and not a simple path."""
         return self.entry.get_text()
 
-    def get_filename(self):
-        """Deprecated"""
-        logger.warning("Just use get_text")
-        return self.get_text()
+    def get_path(self):
+        """Returns the path in the entry; if shell_quoting is set, this extracts
+        the command from the text and returns only that."""
+        text = self.get_text()
+        if self.shell_quoting:
+            command_array = shlex.split(text)
+            return command_array[0] if command_array else ""
+
+        return text
 
     def get_completion(self):
         """Return an EntryCompletion widget"""
@@ -116,7 +164,7 @@ class FileChooserEntry(Gtk.Box):
 
     def get_default_folder(self):
         """Return the default folder for the file picker"""
-        default_path = self.path or self.default_path or ""
+        default_path = self.get_path() or self.default_path or ""
         if not default_path or not system.path_exists(default_path):
             current_entry = self.get_text()
             if system.path_exists(current_entry):
@@ -132,17 +180,35 @@ class FileChooserEntry(Gtk.Box):
 
         if response == Gtk.ResponseType.ACCEPT:
             target_path = file_chooser_dialog.get_filename()
-            if target_path:
-                self.entry.set_text(system.reverse_expanduser(target_path))
+
+            if target_path and self.shell_quoting:
+                command_array = shlex.split(self.entry.get_text())
+                text = shlex.join([target_path] + command_array[1:])
+            else:
+                text = target_path
+
+            self.original_text = text
+            self.entry.set_text(text)
 
         file_chooser_dialog.destroy()
 
+    def on_open_clicked(self, _widget):
+        path = self.get_open_directory()
+
+        if path:
+            open_uri(path)
+
+    def get_open_directory(self):
+        path = self.get_path()
+
+        while path and not os.path.isdir(path):
+            path = os.path.dirname(path)
+
+        return path
+
     def on_entry_changed(self, widget):
         """Entry changed callback"""
         self.clear_warnings()
-        path = widget.get_text()
-        if not path:
-            return
 
         # If the user isn't editing this entry, we'll apply updates
         # immediately upon any change
@@ -152,7 +218,9 @@ class FileChooserEntry(Gtk.Box):
                 # We changed the text on commit, so we return here to avoid a double changed signal
                 return
 
-        self.path = path
+        text = self.get_text()
+        path = self.get_path()
+        self.original_text = text
 
         if self.warn_if_ntfs and LINUX_SYSTEM.get_fs_type_for_path(path) == "ntfs":
             ntfs_box = Gtk.Box(spacing=6, visible=True)
@@ -162,7 +230,7 @@ class FileChooserEntry(Gtk.Box):
             ntfs_label = Gtk.Label(visible=True)
             ntfs_label.set_markup(_(
                 "<b>Warning!</b> The selected path is located on a drive formatted by Windows.\n"
-                "Games and programs installed on Windows drives usually <b>don't work</b>."
+                "Games and programs installed on Windows drives <b>don't work</b>."
             ))
             ntfs_box.add(ntfs_label)
             self.pack_end(ntfs_box, False, False, 10)
@@ -170,17 +238,20 @@ class FileChooserEntry(Gtk.Box):
             non_empty_label = Gtk.Label(visible=True)
             non_empty_label.set_markup(_(
                 "<b>Warning!</b> The selected path "
-                "contains files. Installation might not work properly."
+                "contains files. Installation will not work properly."
             ))
             self.pack_end(non_empty_label, False, False, 10)
-        parent = system.get_existing_parent(path)
-        if parent is not None and not os.access(parent, os.W_OK):
-            non_writable_destination_label = Gtk.Label(visible=True)
-            non_writable_destination_label.set_markup(_(
-                "<b>Warning</b> The destination folder "
-                "is not writable by the current user."
-            ))
-            self.pack_end(non_writable_destination_label, False, False, 10)
+        if self.warn_if_non_writable_parent:
+            parent = system.get_existing_parent(path)
+            if parent is not None and not os.access(parent, os.W_OK):
+                non_writable_destination_label = Gtk.Label(visible=True)
+                non_writable_destination_label.set_markup(_(
+                    "<b>Warning</b> The destination folder "
+                    "is not writable by the current user."
+                ))
+                self.pack_end(non_writable_destination_label, False, False, 10)
+
+        self.open_button.set_sensitive(bool(self.get_open_directory()))
 
         self.emit("changed")
 
@@ -196,17 +267,17 @@ class FileChooserEntry(Gtk.Box):
         GLib.idle_add(self.detect_changes)
 
     def detect_changes(self):
-        """Detects if the text has changed and updates self.path and fires
+        """Detects if the text has changed and updates self.original_text and fires
         the changed signal. Lame, but Gtk.Entry does not always fire its
         changed event when edited!"""
-        new_path = self.get_text()
-        if self.path != new_path:
-            self.path = new_path
+        new_text = self.get_text()
+        if self.original_text != new_text:
+            self.original_text = new_text
             self.emit("changed")
         return False  # used as idle function
 
     def normalize_path(self):
-        original_path = self.get_text()
+        original_path = self.get_path()
         path = original_path.strip("\r\n")
 
         if path.startswith('file:///'):
@@ -217,7 +288,7 @@ class FileChooserEntry(Gtk.Box):
         self.update_completion(path)
 
         if path != original_path:
-            self.entry.set_text(path)
+            self.set_path(path)
             return True
 
         return False
@@ -255,7 +326,7 @@ class Label(Gtk.Label):
 
     def __init__(self, message=None, width_request=230):
         """Custom init of label."""
-        super().__init__(label=message)
+        super().__init__(label=message, visible=True)
         self.set_line_wrap(True)
         self.set_max_width_chars(22)
         self.set_line_wrap_mode(Pango.WrapMode.WORD_CHAR)
@@ -269,16 +340,12 @@ class VBox(Gtk.Box):
         super().__init__(orientation=Gtk.Orientation.VERTICAL, margin_top=18, **kwargs)
 
 
-class EditableGrid(Gtk.Grid):
+class EditableGrid(Gtk.Box):
     __gsignals__ = {"changed": (GObject.SIGNAL_RUN_FIRST, None, ())}
 
     def __init__(self, data, columns):
         self.columns = columns
-        super().__init__()
-        self.set_column_homogeneous(True)
-        self.set_row_homogeneous(True)
-        self.set_row_spacing(6)
-        self.set_column_spacing(6)
+        super().__init__(orientation=Gtk.Orientation.VERTICAL, spacing=6)
 
         self.liststore = Gtk.ListStore(str, str)
         for item in data:
@@ -307,14 +374,16 @@ class EditableGrid(Gtk.Grid):
         self.delete_button.connect("clicked", self.on_delete)
 
         self.scrollable_treelist = Gtk.ScrolledWindow()
-        self.scrollable_treelist.set_vexpand(True)
+        self.scrollable_treelist.set_size_request(-1, 209)
         self.scrollable_treelist.add(self.treeview)
         self.scrollable_treelist.set_shadow_type(Gtk.ShadowType.IN)
 
-        self.attach(self.scrollable_treelist, 0, 0, 5, 5)
-        self.attach(self.add_button, 5 - len(self.buttons), 6, 1, 1)
-        for i, button in enumerate(self.buttons[1:]):
-            self.attach_next_to(button, self.buttons[i], Gtk.PositionType.RIGHT, 1, 1)
+        self.pack_start(self.scrollable_treelist, True, True, 0)
+        button_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=6)
+        for button in reversed(self.buttons):
+            button.set_size_request(80, -1)
+            button_box.pack_end(button, False, False, 0)
+        self.pack_end(button_box, False, False, 0)
         self.show_all()
 
     def on_add(self, widget):  # pylint: disable=unused-argument
