@@ -2,7 +2,7 @@
 # pylint: disable=unsubscriptable-object
 from gi.repository import GLib, GObject, Gtk
 
-from lutris.util.jobs import AsyncCall
+from lutris.gui.dialogs import ErrorDialog
 
 
 class SearchableCombobox(Gtk.Bin):
@@ -64,14 +64,14 @@ class SearchableCombobox(Gtk.Bin):
         self.combobox.set_active_id(model[_iter][1])
 
     def _populate_combobox_choices(self, choice_func):
-        AsyncCall(self._do_populate_combobox_choices, None, choice_func)
-
-    def _do_populate_combobox_choices(self, choice_func):
-        for choice in choice_func():
-            self.liststore.append(choice)
-        entry = self.combobox.get_child()
-        entry.set_icon_from_icon_name(Gtk.EntryIconPosition.PRIMARY, None)
-        self.combobox.set_active_id(self.initial)
+        try:
+            for choice in choice_func():
+                self.liststore.append(choice)
+            entry = self.combobox.get_child()
+            entry.set_icon_from_icon_name(Gtk.EntryIconPosition.PRIMARY, None)
+            self.combobox.set_active_id(self.initial)
+        except Exception as ex:
+            ErrorDialog(ex, parent=self.get_toplevel())
 
     @staticmethod
     def _on_combobox_scroll(combobox, _event):
