@@ -1,13 +1,15 @@
 """Sidebar for the main window"""
+import locale
 from gettext import gettext as _
 
 from gi.repository import GLib, GObject, Gtk, Pango
 
 from lutris import runners, services
+from lutris.config import LutrisConfig
+from lutris.database import categories as categories_db
 from lutris.database import games as games_db
-from lutris.exceptions import watch_errors
 from lutris.game import Game
-from lutris.gui import dialogs
+from lutris.gui.config.edit_category_games import EditCategoryGamesDialog
 from lutris.gui.config.runner import RunnerConfigDialog
 from lutris.gui.config.runner_box import RunnerBox
 from lutris.gui.config.services_box import ServicesBox
@@ -15,8 +17,9 @@ from lutris.gui.dialogs import ErrorDialog
 from lutris.gui.dialogs.runner_install import RunnerInstallDialog
 from lutris.gui.widgets.utils import has_stock_icon
 from lutris.installer.interpreter import ScriptInterpreter
+from lutris.runners import InvalidRunnerError
 from lutris.services import SERVICES
-from lutris.services.base import AuthTokenExpired, BaseService
+from lutris.services.base import AuthTokenExpiredError, BaseService
 
 TYPE = 0
 SLUG = 1
@@ -46,7 +49,6 @@ class SidebarRow(Gtk.ListBoxRow):
         self.application = application
         self.type = type_
         self.id = id_
-        self.runner = None
         self.name = name
         self.is_updating = False
         self.buttons = {}
@@ -162,11 +164,11 @@ class ServiceSidebarRow(SidebarRow):
 
     def service_reloaded_cb(self, error):
         if error:
-            if isinstance(error, AuthTokenExpired):
+            if isinstance(error, AuthTokenExpiredError):
                 self.service.logout()
                 self.service.login(parent=self.get_toplevel())
             else:
-                ErrorDialog(str(error), parent=self.get_toplevel())
+                ErrorDialog(error, parent=self.get_toplevel())
         GLib.timeout_add(2000, self.enable_refresh_button)
 
     def enable_refresh_button(self):
@@ -212,38 +214,79 @@ class RunnerSidebarRow(SidebarRow):
 
         # Creation is delayed because only installed runners can be imported
         # and all visible boxes should be installed.
-        self.runner = runners.import_runner(self.id)()
-        if self.runner.multiple_versions:
+        try:
+            runner = self.get_runner()
+        except InvalidRunnerError:
+            return entries
+
+        if runner.multiple_versions:
             entries.append((
                 "system-software-install-symbolic",
                 _("Manage Versions"),
                 self.on_manage_versions,
                 "manage-versions"
             ))
-        if self.runner.runnable_alone:
+        if runner.runnable_alone:
             entries.append(("media-playback-start-symbolic", _("Run"), self.on_run_runner, "run"))
         entries.append(("emblem-system-symbolic", _("Configure"), self.on_configure_runner, "configure"))
         return entries
 
-    @watch_errors()
+    def get_runner(self):
+        return runners.import_runner(self.id)()
+
     def on_run_runner(self, *_args):
         """Runs the runner without no game."""
-        self.runner.run(self.get_toplevel())
+        runner = self.get_runner()
+        runner.run(self.get_toplevel())
 
-    @watch_errors()
     def on_configure_runner(self, *_args):
         """Show runner configuration"""
-        self.application.show_window(RunnerConfigDialog, runner=self.runner)
+        runner = self.get_runner()
+        self.application.show_window(RunnerConfigDialog, runner=runner, parent=self.get_toplevel())
 
-    @watch_errors()
     def on_manage_versions(self, *_args):
         """Manage runner versions"""
-        dlg_title = _("Manage %s versions") % self.runner.name
+        runner = self.get_runner()
+        dlg_title = _("Manage %s versions") % runner.human_name
         self.application.show_window(RunnerInstallDialog, title=dlg_title,
-                                     runner=self.runner, parent=self.get_toplevel())
+                                     runner=runner, parent=self.get_toplevel())
 
-    def on_watched_error(self, error):
-        dialogs.ErrorDialog(str(error), parent=self.get_toplevel())
+
+class CategorySidebarRow(SidebarRow):
+
+    def __init__(self, category, application):
+        super().__init__(
+            category['name'],
+            "user_category",
+            category['name'],
+            Gtk.Image.new_from_icon_name("folder-symbolic", Gtk.IconSize.MENU),
+            application=application
+        )
+        self.category = category
+
+        self._sort_name = locale.strxfrm(category['name'])
+
+    def get_actions(self):
+        """Return the definition of buttons to be added to the row"""
+        return [
+            ("applications-system-symbolic", _("Edit Games"), self.on_category_clicked, "manage-category-games")
+        ]
+
+    def on_category_clicked(self, button):
+        self.application.show_window(EditCategoryGamesDialog, category=self.category, parent=self.get_toplevel())
+        return True
+
+    def __lt__(self, other):
+        if not isinstance(other, CategorySidebarRow):
+            raise ValueError('Cannot compare %s to %s' % (self.__class__.__name__, other.__class__.__name__))
+
+        return self._sort_name < other._sort_name
+
+    def __gt__(self, other):
+        if not isinstance(other, CategorySidebarRow):
+            raise ValueError('Cannot compare %s to %s' % (self.__class__.__name__, other.__class__.__name__))
+
+        return self._sort_name > other._sort_name
 
 
 class SidebarHeader(Gtk.Box):
@@ -285,28 +328,33 @@ class LutrisSidebar(Gtk.ListBox):
         super().__init__()
         self.set_size_request(200, -1)
         self.application = application
-        self.get_style_context().add_class("sidebar")
+        self.get_style_context().add_class("lutrissidebar")
 
         # Empty values until LutrisWindow explicitly initializes the rows
         # at the right time.
         self.installed_runners = []
+        self.runner_visibility_cache = {}
+        self.used_categories = set()
         self.active_services = {}
         self.active_platforms = []
         self.service_rows = {}
         self.runner_rows = {}
         self.platform_rows = {}
 
+        self.category_rows = {}
         # A dummy objects that allows inspecting why/when we have a show() call on the object.
         self.running_row = DummyRow()
         self.missing_row = DummyRow()
         self.row_headers = {
             "library": SidebarHeader(_("Library"), header_index=0),
-            "service": SidebarHeader(_("Sources"), header_index=1),
-            "runner": SidebarHeader(_("Runners"), header_index=2),
-            "platform": SidebarHeader(_("Platforms"), header_index=3),
+            "user_category": SidebarHeader(_("Categories"), header_index=1),
+            "service": SidebarHeader(_("Sources"), header_index=2),
+            "runner": SidebarHeader(_("Runners"), header_index=3),
+            "platform": SidebarHeader(_("Platforms"), header_index=4),
         }
         GObject.add_emission_hook(RunnerBox, "runner-installed", self.update_rows)
         GObject.add_emission_hook(RunnerBox, "runner-removed", self.update_rows)
+        GObject.add_emission_hook(RunnerConfigDialog, "runner-updated", self.update_runner_rows)
         GObject.add_emission_hook(ScriptInterpreter, "runners-installed", self.update_rows)
         GObject.add_emission_hook(ServicesBox, "services-changed", self.update_rows)
         GObject.add_emission_hook(Game, "game-start", self.on_game_start)
@@ -393,14 +441,14 @@ class LutrisSidebar(Gtk.ListBox):
 
     @property
     def selected_category(self):
-        """The selected sidebar row, as a tuple of category and category value,
+        """The selected sidebar row, as a tuple of category type and category value,
         like ('service', 'lutris')."""
         row = self.get_selected_row()
         return (row.type, row.id) if row else ("category", "all")
 
     @selected_category.setter
     def selected_category(self, value):
-        """Selects the rrow for the category indicated by a category tuple,
+        """Selects the row for the category indicated by a category tuple,
         like ('service', 'lutris')"""
         selected_row_type, selected_row_id = value or ("category", "all")
         for row in self.get_children():
@@ -409,20 +457,36 @@ class LutrisSidebar(Gtk.ListBox):
                 break
 
     def _filter_func(self, row):
+        def is_runner_visible(runner_name):
+            if runner_name not in self.runner_visibility_cache:
+                runner_config = LutrisConfig(runner_slug=row.id)
+                self.runner_visibility_cache[runner_name] = runner_config.runner_config.get(
+                    "visible_in_side_panel", True)
+            return self.runner_visibility_cache[runner_name]
+
         if not row or not row.id or row.type in ("category", "dynamic_category"):
             return True
-        if row.type == "service":
-            return row.id in self.active_services
+
         if row.type == "runner":
             if row.id is None:
                 return True  # 'All'
-            return row.id in self.installed_runners
-        return row.id in self.active_platforms
+            return row.id in self.installed_runners and is_runner_visible(row.id)
+
+        if row.type == "user_category":
+            allowed_ids = self.used_categories
+        elif row.type == "service":
+            allowed_ids = self.active_services
+        else:
+            allowed_ids = self.active_platforms
+
+        return row.id in allowed_ids
 
     def _header_func(self, row, before):
         if not before:
             header = self.row_headers["library"]
-        elif before.type in ("category", "dynamic_category") and row.type == "service":
+        elif before.type in ("category", "dynamic_category") and row.type == "user_category":
+            header = self.row_headers[row.type]
+        elif before.type in ("category", "dynamic_category", "user_category") and row.type == "service":
             header = self.row_headers[row.type]
         elif before.type == "service" and row.type == "runner":
             header = self.row_headers[row.type]
@@ -431,7 +495,7 @@ class LutrisSidebar(Gtk.ListBox):
         else:
             header = None
 
-        if row.get_header() != header:
+        if header and row.get_header() != header:
             # GTK is messy here; a header can't belong to two rows at once,
             # so we must remove it from the one that owns it, if any, and
             # also from the sidebar itself. Then we can reuse it.
@@ -442,6 +506,11 @@ class LutrisSidebar(Gtk.ListBox):
             header.first_row = row
             row.set_header(header)
 
+    def update_runner_rows(self, *_args):
+        self.runner_visibility_cache.clear()
+        self.update_rows()
+        return True
+
     def update_rows(self, *_args):
         """Generates any missing rows that are now needed, and re-evaluate the filter to hide
         any no longer needed. GTK has a lot of trouble dynamically updating and re-arranging
@@ -467,6 +536,10 @@ class LutrisSidebar(Gtk.ListBox):
             row.show_all()
             self.insert(row, index)
 
+        categories_db.remove_unused_categories()
+        categories = [c for c in categories_db.get_categories() if not categories_db.is_reserved_category(c["name"])]
+
+        self.used_categories = {c["name"] for c in categories}
         self.active_services = services.get_enabled_services()
         self.installed_runners = [runner.name for runner in runners.get_installed()]
         self.active_platforms = games_db.get_used_platforms()
@@ -506,6 +579,12 @@ class LutrisSidebar(Gtk.ListBox):
                 self.platform_rows[platform] = platform_row
                 insert_row(platform_row)
 
+        for category in categories:
+            if category["name"] not in self.category_rows:
+                new_category_row = CategorySidebarRow(category, application=self.application)
+                self.category_rows[category["name"]] = new_category_row
+                insert_row(new_category_row)
+
         self.invalidate_filter()
         return True
 
@@ -516,7 +595,7 @@ class LutrisSidebar(Gtk.ListBox):
 
     def on_game_stop(self, _game):
         """Hide the "running" section when no games are running"""
-        if not self.application.running_games.get_n_items():
+        if not self.application.running_games:
             self.running_row.hide()
 
             if self.get_selected_row() == self.running_row:
