@@ -1,14 +1,11 @@
 """AppIndicator based tray icon"""
 from gettext import gettext as _
 
-# Third Party Libraries
 import gi
 from gi.repository import Gtk
 
-# Lutris Modules
-from lutris import pga
+from lutris.database.games import get_games
 from lutris.game import Game
-from lutris.gui.widgets.utils import get_pixbuf_for_game
 
 try:
     gi.require_version('AppIndicator3', '0.1')
@@ -43,7 +40,7 @@ class LutrisStatusIcon:
         """Whether the icon is visible"""
         if APP_INDICATOR_SUPPORTED:
             return self.icon.get_status() != AppIndicator.IndicatorStatus.PASSIVE
-        return self.icon.is_visible()
+        return self.icon.get_visible()
 
     def set_visible(self, value):
         """Set the visibility of the icon"""
@@ -51,13 +48,13 @@ class LutrisStatusIcon:
             if value:
                 visible = AppIndicator.IndicatorStatus.ACTIVE
             else:
-                visible = AppIndicator.IndicatorStatus.ACTIVE
+                visible = AppIndicator.IndicatorStatus.PASSIVE
             self.icon.set_status(visible)
         else:
             self.icon.set_visible(value)
 
     def get_menu(self):
-        """Instanciates the menu attached to the tray icon"""
+        """Instantiates the menu attached to the tray icon"""
         menu = Gtk.Menu()
         installed_games = self.add_games()
         number_of_games_in_menu = 10
@@ -65,11 +62,11 @@ class LutrisStatusIcon:
             menu.append(self._make_menu_item_for_game(game))
         menu.append(Gtk.SeparatorMenuItem())
 
-        present_menu = Gtk.ImageMenuItem()
-        present_menu.set_image(Gtk.Image.new_from_icon_name("lutris", Gtk.IconSize.MENU))
-        present_menu.set_label(_("Show Lutris"))
-        present_menu.connect("activate", self.on_activate)
-        menu.append(present_menu)
+        self.present_menu = Gtk.ImageMenuItem()
+        self.present_menu.set_image(Gtk.Image.new_from_icon_name("lutris", Gtk.IconSize.MENU))
+        self.present_menu.set_label(_("Show Lutris"))
+        self.present_menu.connect("activate", self.on_activate)
+        menu.append(self.present_menu)
 
         quit_menu = Gtk.MenuItem()
         quit_menu.set_label(_("Quit"))
@@ -78,9 +75,28 @@ class LutrisStatusIcon:
         menu.show_all()
         return menu
 
+    def update_present_menu(self):
+        app_window = self.application.window
+        if app_window:
+            if app_window.get_visible():
+                self.present_menu.set_label(_("Hide Lutris"))
+            else:
+                self.present_menu.set_label(_("Show Lutris"))
+
     def on_activate(self, _status_icon, _event=None):
         """Callback to show or hide the window"""
-        self.application.window.present()
+        app_window = self.application.window
+        if app_window.get_visible():
+            # If the window has any transients, hiding it will hide them too
+            # never to be shown again, which is broken. So we don't allow that.
+            windows = Gtk.Window.list_toplevels()
+            for w in windows:
+                if w.get_visible() and w.get_transient_for() == app_window:
+                    return
+
+            app_window.hide()
+        else:
+            app_window.show()
 
     def on_menu_popup(self, _status_icon, button, time):
         """Callback to show the contextual menu"""
@@ -88,20 +104,18 @@ class LutrisStatusIcon:
 
     def on_quit_application(self, _widget):
         """Callback to quit the program"""
-        self.application.do_shutdown()
+        self.application.quit()
 
     def _make_menu_item_for_game(self, game):
-        menu_item = Gtk.ImageMenuItem()
+        menu_item = Gtk.MenuItem()
         menu_item.set_label(game["name"])
-        game_icon = get_pixbuf_for_game(game["slug"], "icon_small")
-        menu_item.set_image(Gtk.Image.new_from_pixbuf(game_icon))
         menu_item.connect("activate", self.on_game_selected, game["id"])
         return menu_item
 
     @staticmethod
     def add_games():
         """Adds installed games in order of last use"""
-        installed_games = pga.get_games(filter_installed=True)
+        installed_games = get_games(filters={"installed": 1})
         installed_games.sort(
             key=lambda game: max(game["lastplayed"] or 0, game["installed_at"] or 0),
             reverse=True,
@@ -109,7 +123,8 @@ class LutrisStatusIcon:
         return installed_games
 
     def on_game_selected(self, _widget, game_id):
-        self.application.launch(Game(game_id))
+        launch_ui_delegate = self.application.get_launch_ui_delegate()
+        Game(game_id).launch(launch_ui_delegate)
 
 
 class LutrisTray(Gtk.StatusIcon):
