@@ -1,12 +1,9 @@
-# Standard Library
 import logging
 import os.path
 from gettext import gettext as _
 
-# Lutris Modules
 from lutris.config import LutrisConfig
-from lutris.gui.dialogs import ErrorDialog
-from lutris.gui.dialogs.download import DownloadDialog
+from lutris.exceptions import MissingBiosError, MissingGameExecutableError
 from lutris.runners.runner import Runner
 from lutris.util import display, extract, system
 
@@ -20,7 +17,6 @@ def get_resolutions():
     return screen_resolutions
 
 
-# pylint: disable=C0103
 class atari800(Runner):
     human_name = _("Atari800")
     platforms = [_("Atari 8bit computers")]  # FIXME try to determine the actual computer used
@@ -49,12 +45,9 @@ class atari800(Runner):
 
     runner_options = [
         {
-            "option":
-            "bios_path",
-            "type":
-            "directory_chooser",
-            "label":
-            _("BIOS location"),
+            "option": "bios_path",
+            "type": "directory_chooser",
+            "label": _("BIOS location"),
             "help": _(
                 "A folder containing the Atari 800 BIOS files.\n"
                 "They are provided by Lutris so you shouldn't have to "
@@ -62,10 +55,8 @@ class atari800(Runner):
             ),
         },
         {
-            "option":
-            "machine",
-            "type":
-            "choice",
+            "option": "machine",
+            "type": "choice",
             "choices": [
                 (_("Emulate Atari 800"), "atari"),
                 (_("Emulate Atari 800 XL"), "xl"),
@@ -73,15 +64,14 @@ class atari800(Runner):
                 (_("Emulate Atari 320 XE (Rambo)"), "rambo"),
                 (_("Emulate Atari 5200"), "5200"),
             ],
-            "default":
-            "atari",
-            "label":
-            _("Machine"),
+            "default": "atari",
+            "label": _("Machine"),
         },
         {
             "option": "fullscreen",
             "type": "bool",
             "default": False,
+            "section": _("Graphics"),
             "label": _("Fullscreen"),
         },
         {
@@ -89,20 +79,19 @@ class atari800(Runner):
             "type": "choice",
             "choices": get_resolutions(),
             "default": "desktop",
+            "section": _("Graphics"),
             "label": _("Fullscreen resolution"),
         },
     ]
 
-    def install(self, version=None, downloader=None, callback=None):
+    def install(self, install_ui_delegate, version=None, callback=None):
 
-        def on_runner_installed(*args):  # pylint: disable=unused-argument
+        def on_runner_installed(*_args):
             config_path = system.create_folder("~/.atari800")
             bios_archive = os.path.join(config_path, "atari800-bioses.zip")
-            dlg = DownloadDialog(self.bios_url, bios_archive)
-            dlg.run()
+            install_ui_delegate.download_install_file(self.bios_url, bios_archive)
             if not system.path_exists(bios_archive):
-                ErrorDialog(_("Could not download Atari 800 BIOS archive"))
-                return
+                raise RuntimeError(_("Could not download Atari 800 BIOS archive"))
             extract.extract_archive(bios_archive, config_path)
             os.remove(bios_archive)
             config = LutrisConfig(runner_slug="atari800")
@@ -111,7 +100,7 @@ class atari800(Runner):
             if callback:
                 callback()
 
-        super().install(version, downloader, on_runner_installed)
+        super().install(install_ui_delegate, version, on_runner_installed)
 
     def find_good_bioses(self, bios_path):
         """ Check for correct bios files """
@@ -125,7 +114,7 @@ class atari800(Runner):
         return good_bios
 
     def play(self):
-        arguments = [self.get_executable()]
+        arguments = self.get_command()
         if self.runner_config.get("fullscreen"):
             arguments.append("-fullscreen")
         else:
@@ -144,7 +133,7 @@ class atari800(Runner):
 
         bios_path = self.runner_config.get("bios_path")
         if not system.path_exists(bios_path):
-            return {"error": "NO_BIOS"}
+            raise MissingBiosError()
         good_bios = self.find_good_bioses(bios_path)
         for bios, filename in good_bios.items():
             arguments.append("-%s" % bios)
@@ -152,7 +141,7 @@ class atari800(Runner):
 
         rom = self.game_config.get("main_file") or ""
         if not system.path_exists(rom):
-            return {"error": "FILE_NOT_FOUND", "file": rom}
+            raise MissingGameExecutableError(filename=rom)
         arguments.append(rom)
 
         return {"command": arguments}
