@@ -124,14 +124,15 @@ def scan_dir_for_bios(path):
 class fsuae(Runner):
     human_name = _("FS-UAE")
     description = _("Amiga emulator")
+    flatpak_id = "net.fsuae.FS-UAE"
     platforms = [
         AMIGAS["A500"]["name"],
         AMIGAS["A500+"]["name"],
         AMIGAS["A600"]["name"],
-        AMIGAS["A1000"]["name"],
-        AMIGAS["A1200"]["name"],
         AMIGAS["A1200"]["name"],
+        AMIGAS["A3000"]["name"],
         AMIGAS["A4000"]["name"],
+        AMIGAS["A1000"]["name"],
         AMIGAS["CD32"]["name"],
         AMIGAS["CDTV"]["name"],
     ]
@@ -479,11 +480,11 @@ class fsuae(Runner):
                 params.append(param % option_value)
 
         if self.runner_config.get("gfx_fullscreen_amiga"):
-            width = int(DISPLAY_MANAGER.get_current_resolution()[0])
+            width = DISPLAY_MANAGER.get_current_resolution()[0]
             params.append("--fullscreen")
             # params.append("--fullscreen_mode=fullscreen-window")
             params.append("--fullscreen_mode=fullscreen")
-            params.append("--fullscreen_width=%d" % width)
+            params.append("--fullscreen_width=%s" % width)
         if self.runner_config.get("jitcompiler"):
             params.append("--jit_compiler=1")
         if self.runner_config.get("bsdsocket"):
@@ -497,4 +498,4 @@ class fsuae(Runner):
         return params
 
     def play(self):
-        return {"command": [self.get_executable()] + self.get_params() + self.insert_floppies()}
+        return {"command": self.get_command() + self.get_params() + self.insert_floppies()}
