@@ -1,23 +1,30 @@
+# Standard Library
 import os
 import shutil
+from gettext import gettext as _
+
+# Lutris Modules
 from lutris.config import LutrisConfig
-from lutris.gui.dialogs import QuestionDialog, FileDialog
+from lutris.exceptions import MissingBiosError, MissingGameExecutableError
 from lutris.runners.runner import Runner
 from lutris.util import system
 
 
 class hatari(Runner):
-    human_name = "Hatari"
-    description = "Atari ST computers emulator"
-    platforms = ["Atari ST"]
+    human_name = _("Hatari")
+    description = _("Atari ST computers emulator")
+    platforms = [_("Atari ST")]
     runnable_alone = True
+    flatpak_id = "org.tuxfamily.hatari"
     runner_executable = "hatari/bin/hatari"
+    entry_point_option = "disk-a"
+
     game_options = [
         {
             "option": "disk-a",
             "type": "file",
-            "label": "Floppy Disk A",
-            "help": (
+            "label": _("Floppy Disk A"),
+            "help": _(
                 "Hatari supports floppy disk images in the following "
                 "formats: ST, DIM, MSA, STX, IPF, RAW and CRT. The last "
                 "three require the caps library (capslib). ZIP is "
@@ -27,8 +34,8 @@ class hatari(Runner):
         {
             "option": "disk-b",
             "type": "file",
-            "label": "Floppy Disk B",
-            "help": (
+            "label": _("Floppy Disk B"),
+            "help": _(
                 "Hatari supports floppy disk images in the following "
                 "formats: ST, DIM, MSA, STX, IPF, RAW and CRT. The last "
                 "three require the caps library (capslib). ZIP is "
@@ -37,14 +44,14 @@ class hatari(Runner):
         },
     ]
 
-    joystick_choices = [("None", "none"), ("Keyboard", "keys"), ("Joystick", "real")]
+    joystick_choices = [(_("None"), "none"), (_("Keyboard"), "keys"), (_("Joystick"), "real")]
 
     runner_options = [
         {
             "option": "bios_file",
             "type": "file",
-            "label": "Bios file (TOS)",
-            "help": (
+            "label": _("Bios file (TOS)"),
+            "help": _(
                 "TOS is the operating system of the Atari ST "
                 "and is necessary to run applications with the best "
                 "fidelity, minimizing risks of issues.\n"
@@ -54,22 +61,25 @@ class hatari(Runner):
         {
             "option": "fullscreen",
             "type": "bool",
-            "label": "Fullscreen",
+            "section": _("Graphics"),
+            "label": _("Fullscreen"),
             "default": False,
         },
         {
             "option": "zoom",
             "type": "bool",
-            "label": "Scale up display by 2 (Atari ST/STE)",
+            "section": _("Graphics"),
+            "label": _("Scale up display by 2 (Atari ST/STE)"),
             "default": True,
-            "help": "Double the screen size in windowed mode.",
+            "help": _("Double the screen size in windowed mode."),
         },
         {
             "option": "borders",
             "type": "bool",
-            "label": "Add borders to display",
+            "section": _("Graphics"),
+            "label": _("Add borders to display"),
             "default": False,
-            "help": (
+            "help": _(
                 "Useful for some games and demos using the overscan "
                 "technique. The Atari ST displayed borders around the "
                 "screen because it was not powerful enough to display "
@@ -81,9 +91,10 @@ class hatari(Runner):
         {
             "option": "status",
             "type": "bool",
-            "label": "Display status bar",
+            "section": _("Graphics"),
+            "label": _("Display status bar"),
             "default": False,
-            "help": (
+            "help": _(
                 "Displays a status bar with some useful information, "
                 "like green leds lighting up when the floppy disks are "
                 "read."
@@ -92,33 +103,32 @@ class hatari(Runner):
         {
             "option": "joy0",
             "type": "choice",
-            "label": "Joystick 1",
+            "section": _("Joysticks"),
+            "label": _("Joystick 0"),
             "choices": joystick_choices,
             "default": "none",
         },
         {
             "option": "joy1",
             "type": "choice",
-            "label": "Joystick 2",
+            "section": _("Joysticks"),
+            "label": _("Joystick 1"),
             "choices": joystick_choices,
-            "default": "none",
+            "default": "real",
         },
     ]
 
-    def install(self, version=None, downloader=None, callback=None):
+    def install(self, install_ui_delegate, version=None, callback=None):
+
         def on_runner_installed(*args):
             bios_path = system.create_folder("~/.hatari/bios")
-            dlg = QuestionDialog(
-                {
-                    "question": "Do you want to select an Atari ST BIOS file?",
-                    "title": "Use BIOS file?",
-                }
-            )
-            if dlg.result == dlg.YES:
-                bios_dlg = FileDialog("Select a BIOS file")
-                bios_filename = bios_dlg.filename
-                if not bios_filename:
-                    return
+
+            bios_filename = install_ui_delegate.show_install_file_inquiry(
+                question=_("Do you want to select an Atari ST BIOS file?"),
+                title=_("Use BIOS file?"),
+                message=_("Select a BIOS file"))
+
+            if bios_filename:
                 shutil.copy(bios_filename, bios_path)
                 bios_path = os.path.join(bios_path, os.path.basename(bios_filename))
                 config = LutrisConfig(runner_slug="hatari")
@@ -127,12 +137,10 @@ class hatari(Runner):
             if callback:
                 callback()
 
-        super(hatari, self).install(
-            version=version, downloader=downloader, callback=on_runner_installed
-        )
+        super().install(install_ui_delegate, version=version, callback=on_runner_installed)
 
-    def play(self):
-        params = [self.get_executable()]
+    def play(self):  # pylint: disable=too-many-branches
+        params = self.get_command()
         if self.runner_config.get("fullscreen"):
             params.append("--fullscreen")
         else:
@@ -168,11 +176,15 @@ class hatari(Runner):
             params.append("--tos")
             params.append(self.runner_config["bios_file"])
         else:
-            return {"error": "NO_BIOS"}
+            raise MissingBiosError()
         diska = self.game_config.get("disk-a")
         if not system.path_exists(diska):
-            return {"error": "FILE_NOT_FOUND", "file": diska}
+            raise MissingGameExecutableError(filename=diska)
         params.append("--disk-a")
         params.append(diska)
 
+        diskb = self.game_config.get("disk-b")
+        params.append("--disk-b")
+        params.append(diskb)
+
         return {"command": params}
