@@ -7,6 +7,7 @@ from zipfile import ZipFile
 import requests
 
 from lutris import settings
+from lutris.exceptions import GameConfigError, MissingGameExecutableError, UnspecifiedVersionError
 from lutris.runners.runner import Runner
 from lutris.util import system
 from lutris.util.libretro import RetroConfig
@@ -26,7 +27,11 @@ def get_libretro_cores():
     # Get core identifiers from info dir
     info_path = get_default_config_path("info")
     if not os.path.exists(info_path):
-        req = requests.get("http://buildbot.libretro.com/assets/frontend/info.zip", allow_redirects=True)
+        req = requests.get(
+            "http://buildbot.libretro.com/assets/frontend/info.zip",
+            allow_redirects=True,
+            timeout=5
+        )
         if req.status_code == requests.codes.ok:  # pylint: disable=no-member
             with open(get_default_config_path('info.zip'), 'wb') as info_zip:
                 info_zip.write(req.content)
@@ -65,6 +70,8 @@ class libretro(Runner):
     description = _("Multi-system emulator")
     runnable_alone = True
     runner_executable = "retroarch/retroarch"
+    flatpak_id = "org.libretro.RetroArch"
+    has_runner_versions = True
 
     game_options = [
         {
@@ -101,6 +108,10 @@ class libretro(Runner):
         },
     ]
 
+    @property
+    def directory(self):
+        return os.path.join(settings.RUNNER_DIR, "retroarch")
+
     @property
     def platforms(self):
         return [core[2] for core in LIBRETRO_CORES]
@@ -118,7 +129,7 @@ class libretro(Runner):
 
     def get_core_path(self, core):
         """Return the path of a core, prioritizing Retroarch cores"""
-        lutris_cores_folder = os.path.join(settings.RUNNER_DIR, "retroarch", "cores")
+        lutris_cores_folder = os.path.join(self.directory, "cores")
         retroarch_core_folder = os.path.join(os.path.expanduser("~/.config/retroarch/cores"))
         core_filename = "{}_libretro.so".format(core)
         retroarch_core = os.path.join(retroarch_core_folder, core_filename)
@@ -129,18 +140,25 @@ class libretro(Runner):
     def get_version(self, use_default=True):
         return self.game_config["core"]
 
-    def is_retroarch_installed(self):
-        return system.path_exists(self.get_executable())
-
-    def is_installed(self, core=None):
-        if not core and self.config and self.game_config.get("core"):
+    def is_installed(self, flatpak_allowed: bool = True, core=None) -> bool:
+        if not core and self.has_explicit_config and self.game_config.get("core"):
             core = self.game_config["core"]
         if not core or self.runner_config.get("runner_executable"):
-            return self.is_retroarch_installed()
+            return super().is_installed(flatpak_allowed=flatpak_allowed)
         is_core_installed = system.path_exists(self.get_core_path(core))
-        return self.is_retroarch_installed() and is_core_installed
+        return super().is_installed(flatpak_allowed=flatpak_allowed) and is_core_installed
 
-    def install(self, version=None, downloader=None, callback=None):
+    def is_installed_for(self, interpreter):
+        core = interpreter.installer.script["game"].get("core")
+        return self.is_installed(core=core)
+
+    def get_installer_runner_version(self, installer, use_runner_config: bool = True) -> str:
+        version = installer.script["game"].get("core")
+        if not version:
+            raise UnspecifiedVersionError(_("The installer does not specify the libretro 'core' version."))
+        return version
+
+    def install(self, install_ui_delegate, version=None, callback=None):
         captured_super = super()  # super() does not work inside install_core()
 
         def install_core():
@@ -148,16 +166,16 @@ class libretro(Runner):
                 if callback:
                     callback()
             else:
-                captured_super.install(version, downloader, callback)
+                captured_super.install(install_ui_delegate, version, callback)
 
-        if not self.is_retroarch_installed():
-            captured_super.install(version=None, downloader=downloader, callback=install_core)
+        if not super().is_installed():
+            captured_super.install(install_ui_delegate, version=None, callback=install_core)
         else:
-            captured_super.install(version, downloader, callback)
+            captured_super.install(install_ui_delegate, version, callback)
 
     def get_run_data(self):
         return {
-            "command": [self.get_executable()] + self.get_runner_parameters(),
+            "command": self.get_command() + self.get_runner_parameters(),
             "env": self.get_env(),
         }
 
@@ -258,40 +276,19 @@ class libretro(Runner):
         return parameters
 
     def play(self):
-        command = [self.get_executable()]
-
-        command += self.get_runner_parameters()
+        command = self.get_command() + self.get_runner_parameters()
 
         # Core
         core = self.game_config.get("core")
         if not core:
-            return {
-                "error": "CUSTOM",
-                "text": _("No core has been selected for this game"),
-            }
+            raise GameConfigError(_("No core has been selected for this game"))
         command.append("--libretro={}".format(self.get_core_path(core)))
 
-        # Ensure the core is available
-        if not self.is_installed(core):
-            self.install(core)
-
         # Main file
         file = self.game_config.get("main_file")
         if not file:
-            return {"error": "CUSTOM", "text": _("No game file specified")}
+            raise GameConfigError(_("No game file specified"))
         if not system.path_exists(file):
-            return {"error": "FILE_NOT_FOUND", "file": file}
+            raise MissingGameExecutableError(filename=file)
         command.append(file)
         return {"command": command}
-
-    # Checks whether the retroarch or libretro directories can be uninstalled.
-    def can_uninstall(self):
-        retroarch_path = os.path.join(settings.RUNNER_DIR, 'retroarch')
-        return os.path.isdir(retroarch_path) or super().can_uninstall()
-
-    # Remove the `retroarch` directory.
-    def uninstall(self):
-        retroarch_path = os.path.join(settings.RUNNER_DIR, 'retroarch')
-        if os.path.isdir(retroarch_path):
-            system.remove_folder(retroarch_path)
-        super().uninstall()
