@@ -2,6 +2,7 @@
 import os
 from gettext import gettext as _
 
+from lutris.exceptions import MissingGameExecutableError
 # Lutris Modules
 from lutris.runners.runner import Runner
 from lutris.util import system
@@ -59,6 +60,7 @@ class o2em(Runner):
             "option": "controller1",
             "type": "choice",
             "choices": controller_choices,
+            "section": _("Controllers"),
             "label": _("First controller"),
             "default": "2",
         },
@@ -66,18 +68,21 @@ class o2em(Runner):
             "option": "controller2",
             "type": "choice",
             "choices": controller_choices,
+            "section": _("Controllers"),
             "label": _("Second controller"),
             "default": "1",
         },
         {
             "option": "fullscreen",
             "type": "bool",
+            "section": _("Graphics"),
             "label": _("Fullscreen"),
             "default": False,
         },
         {
             "option": "scanlines",
             "type": "bool",
+            "section": _("Graphics"),
             "label": _("Scanlines display style"),
             "default": False,
             "help": _("Activates a display filter adding scanlines to imitate "
@@ -93,7 +98,7 @@ class o2em(Runner):
                     return self.platforms[i]
         return ""
 
-    def install(self, version=None, downloader=None, callback=None):
+    def install(self, install_ui_delegate, version=None, callback=None):
 
         def on_runner_installed(*args):
             if not system.path_exists(self.bios_path):
@@ -101,7 +106,7 @@ class o2em(Runner):
             if callback:
                 callback()
 
-        super().install(version, downloader, on_runner_installed)
+        super().install(install_ui_delegate, version, on_runner_installed)
 
     def play(self):
         arguments = ["-biosdir=%s" % self.bios_path]
@@ -118,9 +123,9 @@ class o2em(Runner):
             arguments.append("-s2=%s" % self.runner_config["controller2"])
         rom_path = self.game_config.get("main_file") or ""
         if not system.path_exists(rom_path):
-            return {"error": "FILE_NOT_FOUND", "file": rom_path}
+            raise MissingGameExecutableError(filename=rom_path)
         romdir = os.path.dirname(rom_path)
         romfile = os.path.basename(rom_path)
         arguments.append("-romdir=%s/" % romdir)
         arguments.append(romfile)
-        return {"command": [self.get_executable()] + arguments}
+        return {"command": self.get_command() + arguments}
