@@ -8,14 +8,13 @@ from time import sleep
 
 from lutris import settings
 from lutris.database.games import get_game_by_field
+from lutris.exceptions import MissingGameExecutableError
 from lutris.runners.runner import Runner
 from lutris.util import system
 from lutris.util.downloader import Downloader
 from lutris.util.log import logger
 from lutris.util.strings import split_arguments
 
-DOWNLOAD_URL = "https://github.com/daniel-j/lutris-pico-8-runner/archive/master.tar.gz"
-
 
 class pico8(Runner):
     description = _("Runs PICO-8 fantasy console cartridges")
@@ -77,21 +76,8 @@ class pico8(Runner):
 
     def __init__(self, config=None):
         super().__init__(config)
-
         self.runnable_alone = self.is_native
 
-    def __repr__(self):
-        return _("PICO-8 runner (%s)") % self.config
-
-    def install(self, install_ui_delegate, version=None, callback=None):
-        opts = {}
-        opts["install_ui_delegate"] = install_ui_delegate
-        if callback:
-            opts["callback"] = callback
-        opts["dest"] = settings.RUNNER_DIR + "/pico8"
-        opts["merge_single"] = True
-        self.download_and_extract(DOWNLOAD_URL, **opts)
-
     @property
     def is_native(self):
         return self.runner_config.get("runner_executable", "") != ""
@@ -119,7 +105,7 @@ class pico8(Runner):
     @property
     def launch_args(self):
         if self.is_native:
-            args = [self.get_executable()]
+            args = self.get_command()
             args.append("-windowed")
             args.append("0" if self.runner_config.get("fullscreen") else "1")
             if self.runner_config.get("splore"):
@@ -135,8 +121,7 @@ class pico8(Runner):
             for arg in split_arguments(extra_args):
                 args.append(arg)
         else:
-            args = [
-                self.get_executable(),
+            args = self.get_command() + [
                 os.path.join(settings.RUNNER_DIR, "pico8/web/player.html"),
                 "--window-size",
                 self.runner_config.get("window_size"),
@@ -146,7 +131,7 @@ class pico8(Runner):
     def get_run_data(self):
         return {"command": self.launch_args, "env": self.get_env(os_env=False)}
 
-    def is_installed(self, version=None, fallback=True, min_version=None):
+    def is_installed(self, flatpak_allowed: bool = True) -> bool:
         """Checks if pico8 runner is installed and if the pico8 executable available.
         """
         if self.is_native and system.path_exists(self.runner_config.get("runner_executable")):
@@ -226,7 +211,7 @@ class pico8(Runner):
                 command.append("-run")
             cartPath = self.cart_path
             if not os.path.exists(cartPath):
-                return {"error": "FILE_NOT_FOUND", "file": cartPath}
+                raise MissingGameExecutableError(filename=cartPath)
             command.append(cartPath)
 
         else:
