@@ -1,6 +1,7 @@
 # Standard Library
 from gettext import gettext as _
 
+from lutris.exceptions import MissingGameExecutableError
 # Lutris Modules
 from lutris.runners.runner import Runner
 from lutris.util import system
@@ -12,6 +13,7 @@ class rpcs3(Runner):
     platforms = [_("Sony PlayStation 3")]
     runnable_alone = True
     runner_executable = "rpcs3/rpcs3"
+    flatpak_id = "net.rpcs3.RPCS3"
     game_options = [
         {
             "option": "main_file",
@@ -26,13 +28,13 @@ class rpcs3(Runner):
     system_options_override = [{"option": "disable_runtime", "default": True}]
 
     def play(self):
-        arguments = [self.get_executable()]
+        arguments = self.get_command()
 
         if self.runner_config.get("nogui"):
             arguments.append("--no-gui")
 
         eboot = self.game_config.get("main_file") or ""
         if not system.path_exists(eboot):
-            return {"error": "FILE_NOT_FOUND", "file": eboot}
+            raise MissingGameExecutableError(filename=eboot)
         arguments.append(eboot)
         return {"command": arguments}
