@@ -2,17 +2,17 @@
 import os
 import signal
 from gettext import gettext as _
+from typing import Callable, Dict
 
 from lutris import runtime, settings
+from lutris.api import format_runner_version, get_default_runner_version_info
 from lutris.command import MonitoredCommand
 from lutris.config import LutrisConfig
 from lutris.database.games import get_game_by_field
-from lutris.exceptions import GameConfigError, UnavailableLibrariesError
+from lutris.exceptions import MisconfigurationError, MissingExecutableError, UnavailableLibrariesError
 from lutris.runners import RunnerInstallationError
-from lutris.util import strings, system
-from lutris.util.downloader import Downloader
-from lutris.util.extract import ExtractFailure, extract_archive
-from lutris.util.http import HTTPError, Request
+from lutris.util import flatpak, strings, system
+from lutris.util.extract import ExtractError, extract_archive
 from lutris.util.linux import LINUX_SYSTEM
 from lutris.util.log import logger
 
@@ -33,63 +33,8 @@ class Runner:  # pylint: disable=too-many-public-methods
     entry_point_option = "main_file"
     download_url = None
     arch = None  # If the runner is only available for an architecture that isn't x86_64
-
-    class InstallUIDelegate:
-        """These objects provide UI for a runner as it is installing itself.
-        One of these must be provided to the install() method.
-
-        The default implementation provides no UI and makes default choices for
-        the user, but DialogInstallUIDelegate implements this to show dialogs and
-        ask the user questions. Windows then inherit from DialogLaunchUIDelegate.
-        """
-
-        def check_wine_availability(self):
-            """Called to verify that wine is available, if it is required. May
-            return False to cancel the installation."""
-
-            from lutris.runners.wine import get_wine_version
-
-            if not get_wine_version() and not LINUX_SYSTEM.is_flatpak:
-                logger.warning("WINE is not installed.")
-
-            return True
-
-        def show_install_notice(self, message, secondary=None):
-            """Called to show an informational message to the user.
-
-            The default is to log the message."""
-            logger.info(message)
-
-        def show_install_yesno_inquiry(self, question, title):
-            """Called to ask the user a yes/no question.
-
-            The default is 'yes'."""
-            return True
-
-        def show_install_file_inquiry(self, question, title, message):
-            """Called to ask the user for a file.
-
-            Lutris first asks the user the question given (showing the title);
-            if the user answers 'Yes', it asks for the file using the message.
-
-            Returns None if the user answers 'No' or cancels out. Returns the
-            file path if the user selected one.
-
-            The default is to return None always.
-            """
-            return None
-
-        def download_install_file(self, url, destination):
-            """Downloads a file from a URL to a destination, overwriting any
-            file at that path.
-
-            Returns True if sucessful, and False if the user cancels.
-
-            The default is to download with no UI, and no option to cancel.
-            """
-            downloader = Downloader(url, destination, overwrite=True)
-            downloader.start()
-            return downloader.join()
+    flatpak_id = None
+    has_runner_versions = False
 
     def __init__(self, config=None):
         """Initialize runner."""
@@ -115,6 +60,12 @@ class Runner:  # pylint: disable=too-many-public-methods
         """Leave the ability to override the docstring."""
         self.__doc__ = value  # What the shit
 
+    @property
+    def runner_warning(self):
+        """Returns a message (as markup) that is displayed in the configuration dialog as
+        a warning."""
+        return None
+
     @property
     def name(self):
         return self.__class__.__name__
@@ -162,7 +113,7 @@ class Runner:  # pylint: disable=too-many-public-methods
         """Return the directory where the game is installed."""
         game_path = self.game_data.get("directory")
         if game_path:
-            return game_path
+            return os.path.expanduser(game_path)  # expanduser just in case!
 
         if self.has_explicit_config:
             # Default to the directory where the entry point is located.
@@ -177,11 +128,6 @@ class Runner:  # pylint: disable=too-many-public-methods
         which can find things like /usr/games when applicable."""
         return self.game_path
 
-    @property
-    def library_folders(self):
-        """Return a list of paths where a game might be installed"""
-        return []
-
     @property
     def working_dir(self):
         """Return the working directory to use when running the game."""
@@ -221,22 +167,53 @@ class Runner:  # pylint: disable=too-many-public-methods
                     "advanced": True,
                 }
             )
+
+        runner_options.append(
+            {
+                "section": _("Side Panel"),
+                "option": "visible_in_side_panel",
+                "type": "bool",
+                "label": _("Visible in Side Panel"),
+                "default": True,
+                "advanced": True,
+                "scope": ["runner"],
+                "help": _("Show this runner in the side panel if it is installed or available through Flatpak.")
+            }
+        )
         return runner_options
 
-    def get_executable(self):
+    def get_executable(self) -> str:
         if "runner_executable" in self.runner_config:
             runner_executable = self.runner_config["runner_executable"]
             if os.path.isfile(runner_executable):
                 return runner_executable
         if not self.runner_executable:
-            raise ValueError("runner_executable not set for {}".format(self.name))
-        return os.path.join(settings.RUNNER_DIR, self.runner_executable)
+            raise MisconfigurationError("runner_executable not set for {}".format(self.name))
+
+        exe = os.path.join(settings.RUNNER_DIR, self.runner_executable)
+        if not os.path.isfile(exe):
+            raise MissingExecutableError(_("The executable '%s' could not be found.") % self.runner_executable)
+        return exe
+
+    def get_command(self):
+        """Returns the command line to run the runner itself; generally a game
+        will be appended to this by play()."""
+        try:
+            exe = self.get_executable()
+            if not system.path_exists(exe):
+                raise MissingExecutableError(_("The executable '%s' could not be found.") % exe)
+            return [exe]
+        except MisconfigurationError:
+            if flatpak.is_app_installed(self.flatpak_id):
+                return flatpak.get_run_command(self.flatpak_id)
+
+            raise
 
     def get_env(self, os_env=False, disable_runtime=False):
         """Return environment variables used for a game."""
         env = {}
         if os_env:
-            env.update(os.environ.copy())
+            env = system.get_environment()
 
         # By default we'll set NVidia's shader disk cache to be
         # per-game, so it overflows less readily.
@@ -326,13 +303,8 @@ class Runner:  # pylint: disable=too-many-public-methods
 
         if "command" in launch_config:
             command = strings.split_arguments(launch_config["command"])
-        elif "command" in gameplay_info:
-            command = [gameplay_info["command"][0]]
         else:
-            logger.debug("No command in %s", gameplay_info)
-            logger.debug(launch_config)
-            # The 'file' sort of gameplay_info cannot be made to use a configuration
-            raise GameConfigError(_("The runner could not find a command to apply the configuration to."))
+            command = self.get_command()
 
         exe = self.get_launch_config_exe(launch_config)
         if exe:
@@ -351,14 +323,17 @@ class Runner:  # pylint: disable=too-many-public-methods
         exe = launch_config.get("exe")
         config_working_dir = self.get_launch_config_working_dir(launch_config)
 
-        if exe and config_working_dir and not os.path.isabs(exe):
-            exe_from_config = self.resolve_config_path(exe, config_working_dir)
-            exe_from_game = self.resolve_config_path(exe)
+        if exe:
+            exe = os.path.expanduser(exe)  # just in case!
 
-            if os.path.exists(exe_from_game) and not os.path.exists(exe_from_config):
-                relative = os.path.relpath(exe_from_game, start=config_working_dir)
-                if not relative.startswith("../"):
-                    return relative
+            if config_working_dir and not os.path.isabs(exe):
+                exe_from_config = self.resolve_config_path(exe, config_working_dir)
+                exe_from_game = self.resolve_config_path(exe)
+
+                if os.path.exists(exe_from_game) and not os.path.exists(exe_from_config):
+                    relative = os.path.relpath(exe_from_game, start=config_working_dir)
+                    if not relative.startswith("../"):
+                        return relative
 
         return exe
 
@@ -378,11 +353,14 @@ class Runner:  # pylint: disable=too-many-public-methods
 
     def resolve_config_path(self, path, relative_to=None):
         """Interpret a path taken from the launch_config relative to
-        a working directory, using the game's working_dir if that is omitted.
+        a working directory, using the game's working_dir if that is omitted,
+        and expanding the '~' if we get one.
 
         This is provided as a method so the WINE runner can try to convert
         Windows-style paths to usable paths.
         """
+        path = os.path.expanduser(path)
+
         if not os.path.isabs(path):
             if not relative_to:
                 relative_to = self.working_dir
@@ -412,7 +390,7 @@ class Runner:  # pylint: disable=too-many-public-methods
         """Return dict with command (exe & args list) and env vars (dict).
 
         Reimplement in derived runner if need be."""
-        return {"command": [self.get_executable()], "env": self.get_env()}
+        return {"command": self.get_command(), "env": self.get_env()}
 
     def run(self, ui_delegate):
         """Run the runner alone."""
@@ -461,64 +439,30 @@ class Runner:  # pylint: disable=too-many-public-methods
             return self.is_installed()
         return False
 
-    def is_installed(self):
+    def is_installed(self, flatpak_allowed: bool = True) -> bool:
         """Return whether the runner is installed"""
-        return system.path_exists(self.get_executable())
-
-    def get_runner_version(self, version=None, lutris_only=False):
-        """Get the appropriate version for a runner
-
-        Params:
-            version (str): Optional version to lookup, will return this one if found
-            lutris_only (bool): True to reject 'system' and such pseudo-versions (for WINE)
-
-        Returns:
-            dict: Dict containing version, architecture and url for the runner, None
-            if the data can't be retrieved. If a pseudo-version is accepted, may be
-            a dict containing only the version itself.
-        """
-        logger.info(
-            "Getting runner information for %s%s",
-            self.name,
-            " (version: %s)" % version if version else "",
-        )
-
         try:
-            request = Request("{}/api/runners/{}".format(settings.SITE_URL, self.name))
-            runner_info = request.get().json
+            # Don't care where the exe is, only if we can find it.
+            exe = self.get_executable()
+            if system.path_exists(exe):
+                return True
+        except MisconfigurationError:
+            pass  # We can still try flatpak even if 'which' fails us!
 
-            if not runner_info:
-                logger.error("Failed to get runner information")
-        except HTTPError as ex:
-            logger.error("Unable to get runner information: %s", ex)
-            runner_info = None
+        return bool(flatpak_allowed and self.flatpak_id and flatpak.is_app_installed(self.flatpak_id))
 
-        if not runner_info:
-            return
+    def is_installed_for(self, interpreter):
+        """Returns whether the runner is installed. Specific runners can extract additional
+        script settings, to determine more precisely what must be installed."""
+        return self.is_installed()
 
-        versions = runner_info.get("versions") or []
-        arch = LINUX_SYSTEM.arch
-        if version:
-            if version.endswith("-i386") or version.endswith("-x86_64"):
-                version, arch = version.rsplit("-", 1)
-            versions = [v for v in versions if v["version"] == version]
-        versions_for_arch = [v for v in versions if v["architecture"] == arch]
-        if len(versions_for_arch) == 1:
-            return versions_for_arch[0]
+    def get_installer_runner_version(self, installer, use_runner_config: bool = True) -> str:
+        raise RuntimeError("The '%s' runner does not support versions" % self.name)
 
-        if len(versions_for_arch) > 1:
-            default_version = [v for v in versions_for_arch if v["default"] is True]
-            if default_version:
-                return default_version[0]
-        elif len(versions) == 1 and LINUX_SYSTEM.is_64_bit:
-            return versions[0]
-        elif len(versions) > 1 and LINUX_SYSTEM.is_64_bit:
-            default_version = [v for v in versions if v["default"] is True]
-            if default_version:
-                return default_version[0]
-        # If we didn't find a proper version yet, return the first available.
-        if len(versions_for_arch) >= 1:
-            return versions_for_arch[0]
+    def get_runner_version(self, version: str = None) -> Dict[str, str]:
+        """Get the appropriate version for a runner, as with get_default_runner_version(),
+        but this method allows the runner to apply its configuration."""
+        return get_default_runner_version_info(self.name, version)
 
     def install(self, install_ui_delegate, version=None, callback=None):
         """Install runner using package management systems."""
@@ -532,20 +476,20 @@ class Runner:  # pylint: disable=too-many-public-methods
         if self.download_url:
             opts["dest"] = self.directory
             return self.download_and_extract(self.download_url, **opts)
-        runner = self.get_runner_version(version, lutris_only=True)
-        if not runner:
+        runner_version_info = self.get_runner_version(version)
+        if not runner_version_info:
             raise RunnerInstallationError(_("Failed to retrieve {} ({}) information").format(self.name, version))
 
         if "wine" in self.name:
             opts["merge_single"] = True
             opts["dest"] = os.path.join(
-                self.directory, "{}-{}".format(runner["version"], runner["architecture"])
+                self.directory, format_runner_version(runner_version_info)
             )
 
         if self.name == "libretro" and version:
             opts["merge_single"] = False
             opts["dest"] = os.path.join(settings.RUNNER_DIR, "retroarch/cores")
-        self.download_and_extract(runner["url"], **opts)
+        self.download_and_extract(runner_version_info["url"], **opts)
 
     def download_and_extract(self, url, dest=None, **opts):
         install_ui_delegate = opts["install_ui_delegate"]
@@ -560,19 +504,19 @@ class Runner:  # pylint: disable=too-many-public-methods
         self.extract(archive=runner_archive, dest=dest, merge_single=merge_single, callback=callback)
 
     def extract(self, archive=None, dest=None, merge_single=None, callback=None):
-        if not system.path_exists(archive):
+        if not system.path_exists(archive, exclude_empty=True):
             raise RunnerInstallationError(_("Failed to extract {}").format(archive))
         try:
             extract_archive(archive, dest, merge_single=merge_single)
-        except ExtractFailure as ex:
+        except ExtractError as ex:
             logger.error("Failed to extract the archive %s file may be corrupt", archive)
             raise RunnerInstallationError(_("Failed to extract {}: {}").format(archive, ex)) from ex
         os.remove(archive)
 
         if self.name == "wine":
             logger.debug("Clearing wine version cache")
-            from lutris.util.wine.wine import get_wine_versions
-            get_wine_versions.cache_clear()
+            from lutris.util.wine.wine import get_installed_wine_versions
+            get_installed_wine_versions.cache_clear()
 
         if self.runner_executable:
             runner_executable = os.path.join(settings.RUNNER_DIR, self.runner_executable)
@@ -588,10 +532,12 @@ class Runner:  # pylint: disable=too-many-public-methods
     def can_uninstall(self):
         return os.path.isdir(self.directory)
 
-    def uninstall(self):
+    def uninstall(self, uninstall_callback: Callable[[], None]) -> None:
         runner_path = self.directory
         if os.path.isdir(runner_path):
-            system.remove_folder(runner_path)
+            system.remove_folder(runner_path, completion_function=uninstall_callback)
+        else:
+            uninstall_callback()
 
     def find_option(self, options_group, option_name):
         """Retrieve an option dict if it exists in the group"""
