@@ -6,6 +6,7 @@ from urllib.parse import urlparse
 
 from lutris import settings
 from lutris.database.games import get_game_by_field
+from lutris.exceptions import GameConfigError
 from lutris.runners.runner import Runner
 from lutris.util import datapath, linux, resources, system
 from lutris.util.strings import split_arguments
@@ -93,14 +94,10 @@ class web(Runner):
                       "when hovering above the window."),
         },
         {
-            "option":
-            "open_links",
-            "label":
-            _("Open links in game window"),
-            "type":
-            "bool",
-            "default":
-            False,
+            "option": "open_links",
+            "label": _("Open links in game window"),
+            "type": "bool",
+            "default": False,
             "help": _(
                 "Enable this option if you want clicked links to open inside the "
                 "game window. By default all links open in your default web browser."
@@ -145,26 +142,19 @@ class web(Runner):
             "help": _("Launch the game in a web browser."),
         },
         {
-            "option":
-            "custom_browser_executable",
-            "label":
-            _("Custom web browser executable"),
-            "type":
-            "file",
+            "option": "custom_browser_executable",
+            "label": _("Custom web browser executable"),
+            "type": "file",
             "help": _(
                 "Select the executable of a browser on your system.\n"
                 "If left blank, Lutris will launch your default browser (xdg-open)."
             ),
         },
         {
-            "option":
-            "custom_browser_args",
-            "label":
-            _("Web browser arguments"),
-            "type":
-            "string",
-            "default":
-            '"$GAME"',
+            "option": "custom_browser_args",
+            "label": _("Web browser arguments"),
+            "type": "string",
+            "default": '"$GAME"',
             "help": _(
                 "Command line arguments to pass to the executable.\n"
                 "$GAME or $URL inserts the game url.\n\n"
@@ -172,7 +162,6 @@ class web(Runner):
             ),
         },
     ]
-    system_options_override = [{"option": "disable_runtime", "default": True}]
     runner_executable = "web/electron/electron"
 
     def get_env(self, os_env=True, disable_runtime=False):
@@ -186,22 +175,16 @@ class web(Runner):
     def play(self):
         url = self.game_config.get("main_file")
         if not url:
-            return {
-                "error": "CUSTOM",
-                "text": _("The web address is empty, \n"
-                          "verify the game's configuration."),
-            }
+            raise GameConfigError(_("The web address is empty, \n"
+                                    "verify the game's configuration."))
 
         # check if it's an url or a file
         is_url = urlparse(url).scheme != ""
 
         if not is_url:
             if not system.path_exists(url):
-                return {
-                    "error": "CUSTOM",
-                    "text": _("The file %s does not exist, \n"
-                              "verify the game's configuration.") % url,
-                }
+                raise GameConfigError(_("The file %s does not exist, \n"
+                                        "verify the game's configuration.") % url)
             url = "file://" + url
 
         game_data = get_game_by_field(self.config.game_config_id, "configpath")
@@ -258,7 +241,7 @@ class web(Runner):
         if self.runner_config.get("user_agent"):
             command.append("--user-agent")
             command.append(self.runner_config.get("user_agent"))
-        if linux.LINUX_SYSTEM.is_flatpak:
+        if linux.LINUX_SYSTEM.is_flatpak():
             command.append("--no-sandbox")
 
         return {"command": command, "env": self.get_env(False)}
