@@ -3,8 +3,17 @@
 import os
 import shlex
 from gettext import gettext as _
+from typing import Dict, Tuple
 
 from lutris import runtime, settings
+from lutris.api import format_runner_version
+from lutris.config import LutrisConfig
+from lutris.database.games import get_game_by_field
+from lutris.exceptions import (
+    EsyncLimitError, FsyncUnsupportedError, MisconfigurationError, MissingExecutableError, MissingGameExecutableError,
+    UnspecifiedVersionError
+)
+from lutris.game import Game
 from lutris.gui.dialogs import FileDialog
 from lutris.runners.commands.wine import (  # noqa: F401 pylint: disable=unused-import
     create_prefix, delete_registry_key, eject_disc, install_cab_component, open_wine_terminal, set_regedit,
@@ -13,10 +22,11 @@ from lutris.runners.commands.wine import (  # noqa: F401 pylint: disable=unused-
 from lutris.runners.runner import Runner
 from lutris.util import system
 from lutris.util.display import DISPLAY_MANAGER, get_default_dpi
-from lutris.util.graphics import vkquery
+from lutris.util.graphics import drivers, vkquery
+from lutris.util.linux import LINUX_SYSTEM
 from lutris.util.log import logger
 from lutris.util.steam.config import get_steam_dir
-from lutris.util.strings import parse_version, split_arguments
+from lutris.util.strings import split_arguments
 from lutris.util.wine.d3d_extras import D3DExtrasManager
 from lutris.util.wine.dgvoodoo2 import dgvoodoo2Manager
 from lutris.util.wine.dxvk import REQUIRED_VULKAN_API_VERSION, DXVKManager
@@ -25,119 +35,97 @@ from lutris.util.wine.extract_icon import PEFILE_AVAILABLE, ExtractIcon
 from lutris.util.wine.prefix import DEFAULT_DLL_OVERRIDES, WinePrefixManager, find_prefix
 from lutris.util.wine.vkd3d import VKD3DManager
 from lutris.util.wine.wine import (
-    POL_PATH, WINE_DIR, WINE_PATHS, detect_arch, display_vulkan_error, esync_display_limit_warning,
-    esync_display_version_warning, fsync_display_support_warning, fsync_display_version_warning, get_default_version,
-    get_overrides_env, get_proton_paths, get_real_executable, get_wine_version, get_wine_versions, is_esync_limit_set,
-    is_fsync_supported, is_gstreamer_build, is_version_esync, is_version_fsync
+    WINE_DEFAULT_ARCH, WINE_DIR, WINE_PATHS, detect_arch, get_default_wine_runner_version_info,
+    get_default_wine_version, get_installed_wine_versions, get_overrides_env, get_proton_paths, get_real_executable,
+    get_system_wine_version, get_wine_path_for_version, is_esync_limit_set, is_fsync_supported, is_gstreamer_build
 )
 
-DEFAULT_WINE_PREFIX = "~/.wine"
-MIN_SAFE_VERSION = "7.0"  # Wine installers must run with at least this version
+
+def _get_prefix_warning(config, _option_key):
+    if config.get("prefix"):
+        return None
+
+    exe = config.get("exe")
+    if exe and find_prefix(exe):
+        return None
+
+    return _("<b>Warning</b> Some Wine configuration options cannot be applied, if no prefix can be found.")
 
 
-def _get_prefix_warning(config):
-    if not config.get("prefix"):
-        return _("Some Wine configuration options cannot be applied without an explicit prefix.")
+def _get_dxvk_warning(config, option_key):
+    if drivers.is_outdated():
+        driver_info = drivers.get_nvidia_driver_info()
+        return _("<b>Warning</b> Your NVIDIA driver is outdated.\n"
+                 "You are currently running driver %s which does not "
+                 "fully support all features for Vulkan and DXVK games."
+                 ) % (
+            driver_info["nvrm"]["version"],
+        )
 
     return None
 
 
-def _get_dxvk_warning(config):
-    if config.get("dxvk") and not vkquery.is_vulkan_supported():
-        return _("Vulkan is not installed or is not supported by your system")
-
+def _get_simple_vulkan_support_error(config, option_key, feature):
+    if config.get(option_key) and not LINUX_SYSTEM.is_vulkan_supported():
+        return _("<b>Error</b> Vulkan is not installed or is not supported by your system, "
+                 "%s is not available.") % feature
     return None
 
 
-def _get_dxvk_version_warning(config):
-    if config.get("dxvk") and vkquery.is_vulkan_supported():
+def _get_dxvk_version_warning(config, _option_key):
+    if config.get("dxvk") and LINUX_SYSTEM.is_vulkan_supported():
         version = config.get("dxvk_version")
         if version and not version.startswith("v1."):
-            required_api_version = REQUIRED_VULKAN_API_VERSION
             library_api_version = vkquery.get_vulkan_api_version()
-            if library_api_version and library_api_version < required_api_version:
+            if library_api_version and library_api_version < REQUIRED_VULKAN_API_VERSION:
                 return _("<b>Warning</b> Lutris has detected that Vulkan API version %s is installed, "
                          "but to use the latest DXVK version, %s is required."
                          ) % (
                     vkquery.format_version(library_api_version),
-                    vkquery.format_version(required_api_version)
+                    vkquery.format_version(REQUIRED_VULKAN_API_VERSION)
                 )
 
             devices = vkquery.get_device_info()
 
-            if devices and devices[0].api_version < required_api_version:
+            if devices and devices[0].api_version < REQUIRED_VULKAN_API_VERSION:
                 return _(
                     "<b>Warning</b> Lutris has detected that the best device available ('%s') supports Vulkan API %s, "
                     "but to use the latest DXVK version, %s is required."
                 ) % (
                     devices[0].name,
                     vkquery.format_version(devices[0].api_version),
-                    vkquery.format_version(required_api_version)
+                    vkquery.format_version(REQUIRED_VULKAN_API_VERSION)
                 )
 
     return None
 
 
-def _get_vkd3d_warning(config):
-    if config.get("vkd3d"):
-        if not vkquery.is_vulkan_supported():
-            return _("<b>Warning</b> Vulkan is not installed or is not supported by your system")
-
-    return None
-
-
-def _get_path_for_version(config, version=None):
-    """Return the absolute path of a wine executable for a given version,
-    or the configured version if you don't ask for a version."""
-    if not version:
-        version = config["version"]
-
-    if version in WINE_PATHS:
-        return system.find_executable(WINE_PATHS[version])
-    if "Proton" in version:
-        for proton_path in get_proton_paths():
-            if os.path.isfile(os.path.join(proton_path, version, "dist/bin/wine")):
-                return os.path.join(proton_path, version, "dist/bin/wine")
-            if os.path.isfile(os.path.join(proton_path, version, "files/bin/wine")):
-                return os.path.join(proton_path, version, "files/bin/wine")
-    if version.startswith("PlayOnLinux"):
-        version, arch = version.split()[1].rsplit("-", 1)
-        return os.path.join(POL_PATH, "wine", "linux-" + arch, version, "bin/wine")
-    if version == "custom":
-        return config.get("custom_wine_path", "")
-    return os.path.join(WINE_DIR, version, "bin/wine")
-
-
-def _get_esync_warning(config):
+def _get_esync_warning(config, _option_key):
     if config.get("esync"):
         limits_set = is_esync_limit_set()
-        wine_path = _get_path_for_version(config)
-        wine_ver = is_version_esync(wine_path)
-
-        if not wine_ver:
-            return _("<b>Warning</b> The Wine build you have selected does not support Esync")
-
         if not limits_set:
             return _("<b>Warning</b> Your limits are not set correctly. Please increase them as described here:\n"
                      "<a href='https://github.com/lutris/docs/blob/master/HowToEsync.md'>"
                      "How-to-Esync (https://github.com/lutris/docs/blob/master/HowToEsync.md)</a>")
-
-    return None
+    return ""
 
 
-def _get_fsync_warning(config):
+def _get_fsync_warning(config, _option_key):
     if config.get("fsync"):
         fsync_supported = is_fsync_supported()
-        wine_path = _get_path_for_version(config)
-        wine_ver = is_version_fsync(wine_path)
-
-        if not wine_ver:
-            return _("<b>Warning</b> The Wine build you have selected does not support Fsync.")
-
         if not fsync_supported:
             return _("<b>Warning</b> Your kernel is not patched for fsync.")
+        return ""
 
-        return None
+
+def _get_virtual_desktop_warning(config, _option_key):
+    message = _("Wine virtual desktop is no longer supported")
+    if config.get("Desktop"):
+        version = str(config.get("version")).casefold()
+        if "-ge-" in version or "proton" in version:
+            message += "\n"
+            message += _("Virtual desktops cannot be enabled in Proton or GE Wine versions.")
+    return message
 
 
 class wine(Runner):
@@ -226,10 +214,10 @@ class wine(Runner):
                 "wine-development": _("Wine Development ({})"),
                 "system": _("System ({})"),
             }
-            versions = get_wine_versions()
+            versions = get_installed_wine_versions()
             for version in versions:
                 if version in labels:
-                    version_number = get_wine_version(WINE_PATHS[version])
+                    version_number = get_system_wine_version(WINE_PATHS[version])
                     label = labels[version].format(version_number)
                 else:
                     label = version
@@ -242,7 +230,7 @@ class wine(Runner):
                 "label": _("Wine version"),
                 "type": "choice",
                 "choices": get_wine_version_choices,
-                "default": get_default_version(),
+                "default": get_default_wine_version,
                 "help": _(
                     "The version of Wine used to launch the game.\n"
                     "Using the last version is generally recommended, "
@@ -272,6 +260,7 @@ class wine(Runner):
                 "type": "bool",
                 "default": True,
                 "warning": _get_dxvk_warning,
+                "error": lambda c, k: _get_simple_vulkan_support_error(c, k, _("DXVK")),
                 "active": True,
                 "help": _(
                     "Use DXVK to "
@@ -284,9 +273,10 @@ class wine(Runner):
                 "label": _("DXVK version"),
                 "advanced": True,
                 "type": "choice_with_entry",
+                "condition": LINUX_SYSTEM.is_vulkan_supported(),
                 "choices": DXVKManager().version_choices,
                 "default": DXVKManager().version,
-                "warning": _get_dxvk_version_warning
+                "warning": _get_dxvk_version_warning,
             },
 
             {
@@ -294,7 +284,7 @@ class wine(Runner):
                 "section": _("Graphics"),
                 "label": _("Enable VKD3D"),
                 "type": "bool",
-                "warning": _get_vkd3d_warning,
+                "error": lambda c, k: _get_simple_vulkan_support_error(c, k, _("VKD3D")),
                 "default": True,
                 "active": True,
                 "help": _(
@@ -307,6 +297,7 @@ class wine(Runner):
                 "label": _("VKD3D version"),
                 "advanced": True,
                 "type": "choice_with_entry",
+                "condition": LINUX_SYSTEM.is_vulkan_supported(),
                 "choices": VKD3DManager().version_choices,
                 "default": VKD3DManager().version,
             },
@@ -336,6 +327,7 @@ class wine(Runner):
                 "section": _("Graphics"),
                 "label": _("Enable DXVK-NVAPI / DLSS"),
                 "type": "bool",
+                "error": lambda c, k: _get_simple_vulkan_support_error(c, k, _("DXVK-NVAPI / DLSS")),
                 "default": True,
                 "advanced": True,
                 "help": _(
@@ -436,6 +428,8 @@ class wine(Runner):
                 "section": _("Virtual Desktop"),
                 "label": _("Windowed (virtual desktop)"),
                 "type": "bool",
+                "advanced": True,
+                "warning": _get_virtual_desktop_warning,
                 "default": False,
                 "help": _(
                     "Run the whole Windows desktop in a window.\n"
@@ -448,6 +442,7 @@ class wine(Runner):
                 "section": _("Virtual Desktop"),
                 "label": _("Virtual desktop resolution"),
                 "type": "choice_with_entry",
+                "advanced": True,
                 "choices": DISPLAY_MANAGER.get_resolutions,
                 "help": _("The size of the virtual desktop in pixels."),
             },
@@ -456,6 +451,7 @@ class wine(Runner):
                 "section": _("DPI"),
                 "label": _("Enable DPI Scaling"),
                 "type": "bool",
+                "advanced": True,
                 "default": False,
                 "help": _(
                     "Enables the Windows application's DPI scaling.\n"
@@ -467,6 +463,7 @@ class wine(Runner):
                 "section": _("DPI"),
                 "label": _("DPI"),
                 "type": "string",
+                "advanced": True,
                 "help": _(
                     "The DPI to be used if 'Enable DPI Scaling' is turned on.\n"
                     "If blank or 'auto', Lutris will auto-detect this."
@@ -565,45 +562,49 @@ class wine(Runner):
                 "type": "directory_chooser",
                 "section": _("Sandbox"),
                 "label": _("Sandbox directory"),
+                "warn_if_non_writable_parent": True,
                 "help": _("Custom directory for desktop integration folders."),
                 "advanced": True,
             },
         ]
 
+    @property
+    def runner_warning(self):
+        if not get_system_wine_version():
+            return _(
+                "<b>Warning</b> Wine is not installed on your system\n\n"
+                "Having Wine installed on your system guarantees that "
+                "Wine builds from Lutris will have all required dependencies.\nPlease "
+                "follow the instructions given in the <a "
+                "href='https://github.com/lutris/docs/blob/master/WineDependencies.md'>Lutris Wiki</a> to "
+                "install Wine."
+            )
+
     @property
     def context_menu_entries(self):
         """Return the contexual menu entries for wine"""
         return [
             ("wineexec", _("Run EXE inside Wine prefix"), self.run_wineexec),
-            ("winecfg", _("Wine configuration"), self.run_winecfg),
             ("wineshell", _("Open Bash terminal"), self.run_wine_terminal),
             ("wineconsole", _("Open Wine console"), self.run_wineconsole),
+            (None, "-", None),
+            ("winecfg", _("Wine configuration"), self.run_winecfg),
             ("wine-regedit", _("Wine registry"), self.run_regedit),
-            ("winetricks", _("Winetricks"), self.run_winetricks),
             ("winecpl", _("Wine Control Panel"), self.run_winecpl),
+            ("winetaskmgr", _("Wine Task Manager"), self.run_taskmgr),
+            (None, "-", None),
+            ("winetricks", _("Winetricks"), self.run_winetricks),
         ]
 
     @property
     def prefix_path(self):
         """Return the absolute path of the Wine prefix. Falls back to default WINE prefix."""
-        _prefix_path = self._get_raw_prefix_path()
-        if not _prefix_path:
-            logger.warning("No WINE prefix provided, falling back to system default WINE prefix.")
-            _prefix_path = DEFAULT_WINE_PREFIX
-        return os.path.expanduser(_prefix_path)
-
-    @property
-    def prefix_path_if_provided(self):
-        """Return the absolute path of the Wine prefix, if known. None if not."""
-        _prefix_path = self._get_raw_prefix_path()
-        if _prefix_path:
-            return os.path.expanduser(_prefix_path)
-
-    def _get_raw_prefix_path(self):
         _prefix_path = self._prefix or self.game_config.get("prefix") or os.environ.get("WINEPREFIX")
         if not _prefix_path and self.game_config.get("exe"):
             # Find prefix from game if we have one
             _prefix_path = find_prefix(self.game_exe)
+        if _prefix_path:
+            _prefix_path = os.path.expanduser(_prefix_path)  # just in case!
         return _prefix_path
 
     @property
@@ -614,6 +615,7 @@ class wine(Runner):
         if not exe:
             logger.error("The game doesn't have an executable")
             return None
+        exe = os.path.expanduser(exe)  # just in case!
         if os.path.isabs(exe):
             return system.fix_path_case(exe)
         if not self.game_path:
@@ -626,7 +628,7 @@ class wine(Runner):
         """Return the working directory to use when running the game."""
         _working_dir = self._working_dir or self.game_config.get("working_dir")
         if _working_dir:
-            return _working_dir
+            return os.path.expanduser(_working_dir)
         if self.game_exe:
             game_dir = os.path.dirname(self.game_exe)
             if os.path.isdir(game_dir):
@@ -645,30 +647,46 @@ class wine(Runner):
         Get it from the config or detect it from the prefix"""
         arch = self._wine_arch or self.game_config.get("arch") or "auto"
         if arch not in ("win32", "win64"):
-            arch = detect_arch(self.prefix_path, self.get_executable())
+            prefix_path = self.prefix_path
+            if prefix_path:
+                arch = detect_arch(prefix_path, self.get_executable())
+            else:
+                arch = WINE_DEFAULT_ARCH
         return arch
 
-    def get_runner_version(self, version=None, lutris_only=False):
+    def get_runner_version(self, version: str = None) -> Dict[str, str]:
         if not version:
-            version = self.get_version()
+            default_version_info = get_default_wine_runner_version_info()
+            default_version = format_runner_version(default_version_info) if default_version_info else None
+            version = self.read_version_from_config(default=default_version)
 
-        if version and not lutris_only and version in WINE_PATHS:
+        if version in WINE_PATHS:
             return {"version": version}
 
         return super().get_runner_version(version)
 
-    def get_version(self, use_default=True):
+    def read_version_from_config(self, default: str = None) -> str:
         """Return the Wine version to use. use_default can be set to false to
-        force the installation of a specific wine version"""
-        runner_version = self.runner_config.get("version")
-        if runner_version:
-            return runner_version
-        if use_default:
-            return get_default_version()
+        force the installation of a specific wine version. If no version is configured,
+        we return the default supplied, or the4 global Wine default if none is."""
 
-    def get_path_for_version(self, version):
+        # We must use the config levels to avoid getting a default if the setting
+        # is not set; we'll fall back to get_default_version()
+
+        for level in [self.config.game_level, self.config.runner_level]:
+            if "wine" in level:
+                runner_version = level["wine"].get("version")
+                if runner_version:
+                    return runner_version
+
+        if default:
+            return default
+
+        return get_default_wine_version()
+
+    def get_path_for_version(self, version: str) -> str:
         """Return the absolute path of a wine executable for a given version"""
-        return _get_path_for_version(self.runner_config, version)
+        return get_wine_path_for_version(version, config=self.runner_config)
 
     def resolve_config_path(self, path, relative_to=None):
         # Resolve paths with tolerance for Windows-isms;
@@ -683,55 +701,105 @@ class wine(Runner):
             fixed = path.replace('\\', '/')
             fixed_resolved = super().resolve_config_path(fixed, relative_to)
             fixed_resolved = system.fix_path_case(fixed_resolved)
-            if fixed_resolved:
-                return fixed_resolved
+            return fixed_resolved
 
         return resolved
 
-    def get_executable(self, version=None, fallback=True):
+    def get_executable(self, version: str = None, fallback: bool = True) -> str:
         """Return the path to the Wine executable.
         A specific version can be specified if needed.
         """
         if version is None:
-            version = self.get_version()
-        if not version:
-            return
+            version = self.read_version_from_config()
 
         wine_path = self.get_path_for_version(version)
         if system.path_exists(wine_path):
             return wine_path
 
-        if fallback:
-            # Fallback to default version
-            default_version = get_default_version()
-            wine_path = self.get_path_for_version(default_version)
-            if wine_path:
-                # Update the version in the config
-                if version == self.runner_config.get("version"):
-                    self.runner_config["version"] = default_version
-                    # TODO: runner_config is a dict so we have to instanciate a
-                    # LutrisConfig object to save it.
-                    # XXX: The version key could be either in the game specific
-                    # config or the runner specific config. We need to know
-                    # which one to get the correct LutrisConfig object.
-            return wine_path
+        if not fallback:
+            raise MissingExecutableError(_("The Wine executable at '%s' is missing.") % wine_path)
 
-    def is_installed(self, version=None, fallback=True, min_version=None):
+        # Fallback to default version
+        default_version = get_default_wine_version()
+        wine_path = self.get_path_for_version(default_version)
+        if not system.path_exists(wine_path):
+            raise MissingExecutableError(_("The Wine executable at '%s' is missing.") % wine_path)
+
+        # Update the version in the config
+        if version == self.runner_config.get("version"):
+            self.runner_config["version"] = default_version
+            # TODO: runner_config is a dict so we have to instanciate a
+            # LutrisConfig object to save it.
+            # XXX: The version key could be either in the game specific
+            # config or the runner specific config. We need to know
+            # which one to get the correct LutrisConfig object.
+        return wine_path
+
+    def is_installed(self, flatpak_allowed: bool = True, version: str = None, fallback: bool = True) -> bool:
         """Check if Wine is installed.
         If no version is passed, checks if any version of wine is available
         """
-        if version:
-            return system.path_exists(self.get_executable(version, fallback))
+        try:
+            if version:
+                # We don't care where Wine is, but only if it was found at all.
+                self.get_executable(version, fallback)
+                return True
 
-        wine_versions = get_wine_versions()
-        if min_version:
-            min_version_list, _, _ = parse_version(min_version)
-            for wine_version in wine_versions:
-                version_list, _, _ = parse_version(wine_version)
-                if version_list > min_version_list:
-                    return True
-            logger.warning("Wine %s or higher not found", min_version)
-        return bool(wine_versions)
+            return bool(get_installed_wine_versions())
+        except MisconfigurationError:
+            return False
+
+    def is_installed_for(self, interpreter):
+        try:
+            version = self.get_installer_runner_version(interpreter.installer, use_api=True)
+            return self.is_installed(version=version, fallback=False)
+        except MisconfigurationError:
+            return False
+
+    def get_installer_runner_version(self, installer, use_runner_config: bool = True, use_api: bool = False) -> str:
+        # If a version is specified in the script choose this one
+        version = None
+        if installer.script.get(installer.runner):
+            version = installer.script[installer.runner].get("version")
+        # If the installer is an extension, use the wine version from the base game
+        elif installer.requires:
+            db_game = get_game_by_field(installer.requires, field="installer_slug")
+            if not db_game:
+                db_game = get_game_by_field(installer.requires, field="slug")
+            if not db_game:
+                raise MisconfigurationError(
+                    _("The required game '%s' could not be found.") % installer.requires)
+            game = Game(db_game["id"])
+            version = game.config.runner_config["version"]
+
+        if not version and use_runner_config:
+            # Try to read the version from the saved runner config for Wine.
+            try:
+                return wine.get_runner_version_and_config()[0]
+            except UnspecifiedVersionError:
+                pass  # prefer the error message below for this
+
+        if not version and use_api:
+            # Try to obtain the default wine version from the Lutris API.
+            default_version_info = self.get_runner_version()
+            if "version" in default_version_info:
+                logger.debug("Default wine version is %s", default_version_info["version"])
+                version = format_runner_version(default_version_info)
+
+        if not version:
+            raise UnspecifiedVersionError(_("The installer does not specify a Wine version."))
+
+        return version
+
+    @classmethod
+    def get_runner_version_and_config(cls) -> Tuple[str, LutrisConfig]:
+        runner_config = LutrisConfig(runner_slug="wine")
+        if "wine" in runner_config.runner_level:
+            config_version = runner_config.runner_level["wine"].get("version")
+            if config_version:
+                return config_version, runner_config
+
+        raise UnspecifiedVersionError(_("The runner configuration does not specify a Wine version."))
 
     @classmethod
     def msi_exec(
@@ -834,6 +902,11 @@ class wine(Runner):
         self.prelaunch()
         self._run_executable("control")
 
+    def run_taskmgr(self, *args):
+        """Execute Wine task manager"""
+        self.prelaunch()
+        self._run_executable("taskmgr")
+
     def run_winekill(self, *args):
         """Runs wineserver -k."""
         winekill(
@@ -847,37 +920,40 @@ class wine(Runner):
 
     def set_regedit_keys(self):
         """Reset regedit keys according to config."""
-        prefix = self.prefix_path_if_provided
-        if prefix:
-            prefix_manager = WinePrefixManager(prefix)
-            # Those options are directly changed with the prefix manager and skip
-            # any calls to regedit.
-            managed_keys = {
-                "ShowCrashDialog": prefix_manager.set_crash_dialogs,
-                "Desktop": prefix_manager.set_virtual_desktop,
-                "WineDesktop": prefix_manager.set_desktop_size,
-            }
+        prefix_manager = WinePrefixManager(self.prefix_path)
+        # Those options are directly changed with the prefix manager and skip
+        # any calls to regedit.
+        managed_keys = {
+            "ShowCrashDialog": prefix_manager.set_crash_dialogs,
+            "Desktop": prefix_manager.set_virtual_desktop,
+            "WineDesktop": prefix_manager.set_desktop_size,
+        }
+        for key, path in self.reg_keys.items():
+            value = self.runner_config.get(key) or "auto"
+            if not value or value == "auto" and key not in managed_keys:
+                prefix_manager.clear_registry_subkeys(path, key)
+            elif key in self.runner_config:
+                if key in managed_keys:
+                    # Do not pass fallback 'auto' value to managed keys
+                    if value == "auto":
+                        value = None
+                    if value and key in ("Desktop", "WineDesktop") and (
+                        "wine-ge" in self.get_executable().lower()
+                        or "proton" in self.get_executable().lower()
+                    ):
+                        logger.warning("Wine Virtual Desktop can't be used with Wine-GE and Proton")
+                        value = None
+                    managed_keys[key](value)
+                    continue
+                # Convert numeric strings to integers so they are saved as dword
+                if value.isdigit():
+                    value = int(value)
 
-            for key, path in self.reg_keys.items():
-                value = self.runner_config.get(key) or "auto"
-                if not value or value == "auto" and key not in managed_keys:
-                    prefix_manager.clear_registry_subkeys(path, key)
-                elif key in self.runner_config:
-                    if key in managed_keys:
-                        # Do not pass fallback 'auto' value to managed keys
-                        if value == "auto":
-                            value = None
-                        managed_keys[key](value)
-                        continue
-                    # Convert numeric strings to integers so they are saved as dword
-                    if value.isdigit():
-                        value = int(value)
+                prefix_manager.set_registry_key(path, key, value)
 
-                    prefix_manager.set_registry_key(path, key, value)
-
-            # We always configure the DPI, because if the user turns off DPI scaling, but it
-            # had been on the only way to implement that is to save 96 DPI into the registry.
-            prefix_manager.set_dpi(self.get_dpi())
+        # We always configure the DPI, because if the user turns off DPI scaling, but it
+        # had been on the only way to implement that is to save 96 DPI into the registry.
+        prefix_manager.set_dpi(self.get_dpi())
 
     def get_dpi(self):
         """Return the DPI to be used by Wine; returns None to allow Wine's own
@@ -896,13 +972,16 @@ class wine(Runner):
         return None
 
     def prelaunch(self):
-        if not system.path_exists(os.path.join(self.prefix_path, "user.reg")):
-            logger.warning("No valid prefix detected in %s, creating one...", self.prefix_path)
-            create_prefix(self.prefix_path, wine_path=self.get_executable(), arch=self.wine_arch, runner=self)
+        if not get_system_wine_version():
+            logger.warning("Wine is not installed on your system; required dependencies may be missing.")
 
-        prefix = self.prefix_path_if_provided
-        if prefix:
-            prefix_manager = WinePrefixManager(prefix)
+        prefix_path = self.prefix_path
+        if prefix_path:
+            if not system.path_exists(os.path.join(prefix_path, "user.reg")):
+                logger.warning("No valid prefix detected in %s, creating one...", prefix_path)
+                create_prefix(prefix_path, wine_path=self.get_executable(), arch=self.wine_arch, runner=self)
+
+            prefix_manager = WinePrefixManager(prefix_path)
             if self.runner_config.get("autoconf_joypad", False):
                 prefix_manager.configure_joypads()
             prefix_manager.create_user_symlinks()
@@ -925,19 +1004,22 @@ class wine(Runner):
         ]
 
         managers = {}
-        prefix = self.prefix_path_if_provided
+        for manager_class, enabled_option, version_option in manager_classes:
+            enabled = bool(self.runner_config.get(enabled_option))
+            version = self.runner_config.get(version_option)
+            if enabled or not enabled_only:
+                manager = manager_class(
+                    self.prefix_path,
+                    arch=self.wine_arch,
+                    version=version
+                )
 
-        if prefix:
-            for manager_class, enabled_option, version_option in manager_classes:
-                enabled = bool(self.runner_config.get(enabled_option))
-                version = self.runner_config.get(version_option)
-                if enabled or not enabled_only:
-                    manager = manager_class(
-                        prefix,
-                        arch=self.wine_arch,
-                        version=version
-                    )
-                    managers[manager] = enabled
+                if not manager.can_enable():
+                    enabled = False
+                    if enabled_only:
+                        continue
+
+                managers[manager] = enabled
 
         return managers
 
@@ -964,12 +1046,14 @@ class wine(Runner):
         if show_debug == "-all":
             env["DXVK_LOG_LEVEL"] = "none"
         env["WINEARCH"] = self.wine_arch
-        env["WINE"] = self.get_executable()
-        env["WINE_MONO_CACHE_DIR"] = os.path.join(WINE_DIR, self.get_version(), "mono")
-        env["WINE_GECKO_CACHE_DIR"] = os.path.join(WINE_DIR, self.get_version(), "gecko")
-        if is_gstreamer_build(self.get_executable()):
-            path_64 = os.path.join(WINE_DIR, self.get_version(), "lib64/gstreamer-1.0/")
-            path_32 = os.path.join(WINE_DIR, self.get_version(), "lib/gstreamer-1.0/")
+        wine_exe = self.get_executable()
+        wine_config_version = self.read_version_from_config()
+        env["WINE"] = wine_exe
+        env["WINE_MONO_CACHE_DIR"] = os.path.join(WINE_DIR, wine_config_version, "mono")
+        env["WINE_GECKO_CACHE_DIR"] = os.path.join(WINE_DIR, wine_config_version, "gecko")
+        if is_gstreamer_build(wine_exe):
+            path_64 = os.path.join(WINE_DIR, wine_config_version, "lib64/gstreamer-1.0/")
+            path_32 = os.path.join(WINE_DIR, wine_config_version, "lib/gstreamer-1.0/")
             if os.path.exists(path_64) or os.path.exists(path_32):
                 env["GST_PLUGIN_SYSTEM_PATH_1_0"] = path_64 + ":" + path_32
         if self.prefix_path:
@@ -1004,11 +1088,15 @@ class wine(Runner):
         env["WINEDLLOVERRIDES"] = get_overrides_env(self.dll_overrides)
 
         # Proton support
-        if "Proton" in self.get_version():
+        if wine_config_version and "Proton" in wine_config_version:
             steam_dir = get_steam_dir()
             if steam_dir:  # May be None for example if Proton-GE is used but Steam is not installed
                 env["STEAM_COMPAT_CLIENT_INSTALL_PATH"] = steam_dir
-            env["STEAM_COMPAT_DATA_PATH"] = self.prefix_path
+
+            prefix_path = self.prefix_path
+            if prefix_path:
+                env["STEAM_COMPAT_DATA_PATH"] = prefix_path
+
             env["STEAM_COMPAT_APP_ID"] = '0'
             env["SteamAppId"] = '0'
             if "SteamGameId" not in env:
@@ -1017,25 +1105,30 @@ class wine(Runner):
 
     def get_runtime_env(self):
         """Return runtime environment variables with path to wine for Lutris builds"""
-        wine_path = self.get_executable()
-        wine_root = None
-        if WINE_DIR:
-            wine_root = os.path.dirname(os.path.dirname(wine_path))
-        for proton_path in get_proton_paths():
-            if proton_path in wine_path:
-                wine_root = os.path.dirname(os.path.dirname(wine_path))
+        wine_path = None
+        try:
+            exe = self.get_executable()
+            if WINE_DIR:
+                wine_path = os.path.dirname(os.path.dirname(exe))
+            for proton_path in get_proton_paths():
+                if proton_path in exe:
+                    wine_path = os.path.dirname(os.path.dirname(exe))
+        except MisconfigurationError:
+            wine_path = None
+
         return runtime.get_env(
             version="Ubuntu-18.04",
             prefer_system_libs=self.system_config.get("prefer_system_libs", True),
-            wine_path=wine_root,
+            wine_path=wine_path,
         )
 
     def get_pids(self, wine_path=None):
         """Return a list of pids of processes using the current wine exe."""
-        if wine_path:
-            exe = wine_path
-        else:
-            exe = self.get_executable()
+        try:
+            exe = wine_path or self.get_executable()
+        except MisconfigurationError:
+            return set()
+
         if not exe.startswith("/"):
             exe = system.find_executable(exe)
         pids = system.get_pids_using_file(exe)
@@ -1049,10 +1142,13 @@ class wine(Runner):
         return pids
 
     def sandbox(self, wine_prefix):
-        if self.runner_config.get("sandbox", True):
-            wine_prefix.enable_desktop_integration_sandbox(desktop_dir=self.runner_config.get("sandbox_dir"))
-        else:
-            wine_prefix.restore_desktop_integration()
+        try:
+            if self.runner_config.get("sandbox", True):
+                wine_prefix.enable_desktop_integration_sandbox(desktop_dir=self.runner_config.get("sandbox_dir"))
+            else:
+                wine_prefix.restore_desktop_integration()
+        except Exception as ex:
+            logger.exception("Failed to setup desktop integration, the prefix may not be valid: %s", ex)
 
     def play(self):  # pylint: disable=too-many-return-statements # noqa: C901
         game_exe = self.game_exe
@@ -1063,44 +1159,23 @@ class wine(Runner):
         if using_dxvk:
             # Set this to 1 to enable access to more RAM for 32bit applications
             launch_info["env"]["WINE_LARGE_ADDRESS_AWARE"] = "1"
-            if not vkquery.is_vulkan_supported():
-                if not display_vulkan_error(on_launch=True):
-                    return {"error": "VULKAN_NOT_FOUND"}
 
         if not game_exe or not system.path_exists(game_exe):
-            return {"error": "FILE_NOT_FOUND", "file": game_exe}
+            raise MissingGameExecutableError(filename=game_exe)
 
         if launch_info["env"].get("WINEESYNC") == "1":
             limit_set = is_esync_limit_set()
-            wine_ver = is_version_esync(self.get_executable())
 
-            if not limit_set and not wine_ver:
-                esync_display_version_warning(True)
-                esync_display_limit_warning()
-                return {"error": "ESYNC_LIMIT_NOT_SET"}
-            if not is_esync_limit_set():
-                esync_display_limit_warning()
-                return {"error": "ESYNC_LIMIT_NOT_SET"}
-            if not wine_ver:
-                if not esync_display_version_warning(True):
-                    return {"error": "NON_ESYNC_WINE_VERSION"}
+            if not limit_set:
+                raise EsyncLimitError(_("Your ESYNC limits are not set correctly."))
 
         if launch_info["env"].get("WINEFSYNC") == "1":
             fsync_supported = is_fsync_supported()
-            wine_ver = is_version_fsync(self.get_executable())
 
-            if not fsync_supported and not wine_ver:
-                fsync_display_version_warning(True)
-                fsync_display_support_warning()
-                return {"error": "FSYNC_NOT_SUPPORTED"}
             if not fsync_supported:
-                fsync_display_support_warning()
-                return {"error": "FSYNC_NOT_SUPPORTED"}
-            if not wine_ver:
-                if not fsync_display_version_warning(True):
-                    return {"error": "NON_FSYNC_WINE_VERSION"}
+                raise FsyncUnsupportedError(_("Your kernel is not patched for fsync."))
 
-        command = [self.get_executable()]
+        command = self.get_command()
 
         game_exe, args, _working_dir = get_real_executable(game_exe, self.working_dir)
         command.append(game_exe)
