@@ -2,6 +2,7 @@
 import os
 import shutil
 from gettext import gettext as _
+from typing import List
 
 from gi.repository import Gio, GObject
 
@@ -16,18 +17,19 @@ from lutris.gui.dialogs import NoticeDialog
 from lutris.gui.dialogs.webconnect_dialog import DEFAULT_USER_AGENT, WebConnectDialog
 from lutris.gui.views.media_loader import download_media
 from lutris.gui.widgets.utils import BANNER_SIZE, ICON_SIZE
-from lutris.installer import get_installers
 from lutris.services.service_media import ServiceMedia
 from lutris.util import system
 from lutris.util.cookies import WebkitCookieJar
 from lutris.util.jobs import AsyncCall
 from lutris.util.log import logger
+from lutris.util.strings import slugify
 
 PGA_DB = settings.PGA_DB
 
 
-class AuthTokenExpired(Exception):
-    """Exception raised when a token is no longer valid"""
+class AuthTokenExpiredError(Exception):
+    """Exception raised when a token is no longer valid; the sidebar will
+    log-out and log-in again in response to this rather than reporting it."""
 
 
 class LutrisBanner(ServiceMedia):
@@ -50,7 +52,7 @@ class LutrisIcon(LutrisBanner):
     def custom_media_storage_size(self):
         return (128, 128)
 
-    def update_desktop(self):
+    def run_system_update_desktop_icons(self):
         system.update_desktop_icons()
 
 
@@ -77,6 +79,7 @@ class BaseService(GObject.Object):
     _matcher = None
     has_extras = False
     name = NotImplemented
+    description = ""
     icon = NotImplemented
     online = False
     local = False
@@ -129,6 +132,7 @@ class BaseService(GObject.Object):
         """Refresh the service's games, asynchronously. This raises signals, but
         does so on the main thread- and runs the reload on a worker thread. It calls
         reloaded_callback when done, passing any error (or None on success)"""
+
         def do_reload():
             if self.is_loading:
                 logger.warning("'%s' games are already loading", self.name)
@@ -181,19 +185,43 @@ class BaseService(GObject.Object):
         """Used to generate an installer from the data returned from the services"""
         return {}
 
-    def match_game(self, service_game, api_game):
+    def install_from_api(self, db_game, appid=None):
+        """Install a game, using the API or generate_installer() to obtain the installer."""
+        if not appid:
+            appid = db_game["appid"]
+
+        def on_installers_ready(service_installers, error):
+            if error:
+                raise error  # bounce any error off the backstop
+
+            if not service_installers:
+                service_installers = [self.generate_installer(db_game)]
+            application = Gio.Application.get_default()
+            application.show_installer_window(service_installers, service=self, appid=appid)
+
+        AsyncCall(self.get_installers_from_api, on_installers_ready, appid)
+
+    def get_installer_files(self, installer, installer_file_id, selected_extras):
+        """Used to obtains the content files from the service, when an 'N/A' file is left in
+        the installer. This handles 'extras', and must return a tuple; first a list of
+        InstallerFile or InstallerFileCollection objects that are for the files themselves,
+        and then a list of such objects for the extras. This separation allows us to generate
+        extra installer script steps to move the extras in."""
+        return [], []
+
+    def match_game(self, service_game, lutris_game):
         """Match a service game to a lutris game referenced by its slug"""
         if not service_game:
             return
         sql.db_update(
             PGA_DB,
             "service_games",
-            {"lutris_slug": api_game["slug"]},
+            {"lutris_slug": lutris_game["slug"]},
             conditions={"appid": service_game["appid"], "service": self.id}
         )
         unmatched_lutris_games = get_games(
             searches={"installer_slug": self.matcher},
-            filters={"slug": api_game["slug"]},
+            filters={"slug": lutris_game["slug"]},
             excludes={"service": self.id}
         )
         for game in unmatched_lutris_games:
@@ -223,14 +251,14 @@ class BaseService(GObject.Object):
                     continue
                 self.match_game(service_games.get(provider_game["slug"]), lutris_game)
 
-    def match_existing_game(self, db_games, appid):
+    def match_existing_game(self, db_games, appid, no_signal=False):
         """Checks if a game is already installed and populates the service info"""
         for _game in db_games:
             logger.debug("Matching %s with existing install: %s", appid, _game)
             game = Game(_game["id"])
             game.appid = appid
             game.service = self.id
-            game.save()
+            game.save(no_signal=no_signal)
             service_game = ServiceGameCollection.get_game(self.id, appid)
             sql.db_update(PGA_DB, "service_games", {"lutris_slug": game.slug}, {"id": service_game["id"]})
             return game
@@ -247,6 +275,43 @@ class BaseService(GObject.Object):
                     service_installers.append(installer)
         return service_installers
 
+    def get_installed_slug(self, db_game):
+        """Returns the slug the game will have after installation, by default. This
+        is Lutris's slug, not the one for the service. By default, we derive it from
+        the Game's name."""
+        return slugify(db_game["name"])
+
+    def get_installed_runner_name(self, db_game):
+        """Returns the name of the runner this game will have after installation, or
+        blank if this is not known."""
+        return ""
+
+    def get_service_installers(self, db_game, update):
+        appid = db_game["appid"]
+        if update:
+            service_installers = self.get_update_installers(db_game)
+        else:
+            service_installers = self.get_installers_from_api(appid)
+        # Check if the game is not already installed
+        for service_installer in service_installers:
+            existing_game = self.match_existing_game(
+                get_games(filters={"installer_slug": service_installer["slug"], "installed": "1"}),
+                appid,
+                no_signal=True  # we're on a thread here, signals can crash us!
+            )
+            if existing_game:
+                logger.debug("Found existing game, aborting install")
+                return None, None, existing_game
+        installer = self.generate_installer(db_game) if not update else None
+        if installer:
+            if service_installers:
+                installer["version"] = installer["version"] + " (auto-generated)"
+            service_installers.append(installer)
+        if not service_installers:
+            logger.error("No installer found for %s", db_game)
+            return
+        return service_installers, db_game, None
+
     def install(self, db_game, update=False):
         """Install a service game, or starts the installer of the game.
 
@@ -259,40 +324,35 @@ class BaseService(GObject.Object):
                 run now. Many installers start from here, but continue running after this returns;
                 they return None.
         """
-        appid = db_game["appid"]
-        logger.debug("Installing %s from service %s", appid, self.id)
-
+        logger.debug("Installing %s from service %s", db_game["appid"], self.id)
         # Local services (aka game libraries that don't require any type of online interaction) can
         # be added without going through an install dialog.
         if self.local:
             return self.simple_install(db_game)
-        if update:
-            service_installers = self.get_update_installers(db_game)
-        else:
-            service_installers = self.get_installers_from_api(appid)
-        # Check if the game is not already installed
-        for service_installer in service_installers:
-            existing_game = self.match_existing_game(
-                get_games(filters={"installer_slug": service_installer["slug"], "installed": "1"}),
-                appid
-            )
-            if existing_game:
-                logger.debug("Found existing game, aborting install")
-                return
-        if update:
-            installer = None
-        else:
-            installer = self.generate_installer(db_game)
-        if installer:
-            if service_installers:
-                installer["version"] = installer["version"] + " (auto-generated)"
-            service_installers.append(installer)
-        if not service_installers:
-            logger.error("No installer found for %s", db_game)
-            return
+        AsyncCall(self.get_service_installers, self.on_service_installers_loaded, db_game, update)
 
-        application = Gio.Application.get_default()
-        application.show_installer_window(service_installers, service=self, appid=appid)
+    def install_by_id(self, appid):
+        """Installs a game given the appid for the game on this service."""
+        db_game = ServiceGameCollection.get_game(self.id, appid)
+        if not db_game:
+            logger.error("No game %s found for %s", appid, self.id)
+            return None
+        return self.install(db_game)
+
+    def on_service_installers_loaded(self, result, error):
+        if error:
+            raise error  # bounce this error off the backstop for default handling
+
+        service_installers, db_game, existing_game = result
+
+        # If an existing game was found, it may have been updated,
+        # and it's not safe to fire this until we get here.
+        if existing_game:
+            existing_game.emit("game-updated")
+
+        if service_installers and db_game:
+            application = Gio.Application.get_default()
+            application.show_installer_window(service_installers, service=self, appid=db_game["appid"])
 
     def simple_install(self, db_game):
         """A simplified version of the install method, used when a game doesn't need any setup"""
@@ -323,9 +383,16 @@ class BaseService(GObject.Object):
         """Specific services should implement this"""
         return ""
 
-    def get_game_platforms(self, db_game):
+    def get_game_platforms(self, db_game: dict) -> List[str]:
         """Interprets the database record for this game from this service
-        to extract its platform, or returns None if this is not available."""
+        to extract its platform, or returns an empty list if this is not available."""
+        return []
+
+    def get_service_db_game(self, game: Game):
+        """Returns the row dictionary for the service-game corresponding to the
+        PGA game given, if any, or None."""
+        if game.service == self.id and game.appid:
+            return ServiceGameCollection.get_game(self.id, game.appid)
         return None
 
 
@@ -352,8 +419,7 @@ class OnlineService(BaseService):
                 _("This service requires a game launcher. The following steps will install it.\n"
                   "Once the client is installed, you can login to %s.") % self.name)
             application = Gio.Application.get_default()
-            installers = get_installers(game_slug=self.client_installer)
-            application.show_installer_window(installers)
+            application.show_lutris_installer_window(game_slug=self.client_installer)
             return
         logger.debug("Connecting to %s", self.name)
         dialog = WebConnectDialog(self, parent)
