@@ -4,8 +4,11 @@ import json
 import os
 from gettext import gettext as _
 
+from gi.repository import Gtk
+
 from lutris import settings
-from lutris.exceptions import UnavailableGame
+from lutris.exceptions import UnavailableGameError
+from lutris.gui.dialogs import HumbleBundleCookiesDialog, QuestionDialog
 from lutris.installer import AUTO_ELF_EXE, AUTO_WIN32_EXE
 from lutris.installer.installer_file import InstallerFile
 from lutris.services.base import OnlineService
@@ -14,7 +17,6 @@ from lutris.services.service_media import ServiceMedia
 from lutris.util import linux
 from lutris.util.http import HTTPError, Request
 from lutris.util.log import logger
-from lutris.util.strings import slugify
 
 
 class HumbleBundleIcon(ServiceMedia):
@@ -23,6 +25,7 @@ class HumbleBundleIcon(ServiceMedia):
     size = (70, 70)
     dest_path = os.path.join(settings.CACHE_DIR, "humblebundle/icons")
     file_pattern = "%s.png"
+    file_format = "png"
     api_field = "icon"
 
 
@@ -74,7 +77,26 @@ class HumbleBundleService(OnlineService):
     cache_path = os.path.join(settings.CACHE_DIR, "humblebundle/library/")
 
     supported_platforms = ("linux", "windows")
-    is_loading = False
+
+    def login(self, parent=None):
+        dialog = QuestionDialog({
+            "title": _("Workaround for Humble Bundle authentication"),
+            "question": _("Humble Bundle is restricting API calls from software like Lutris and GameHub.\n"
+                          "Authentication to the service will likely fail.\n"
+                          "There is a workaround involving copying cookies "
+                          "from Firefox, do you want to do this instead?"),
+            "parent": parent
+        })
+        if dialog.result == Gtk.ResponseType.YES:
+            dialog = HumbleBundleCookiesDialog()
+            if dialog.cookies_content:
+                with open(self.cookies_path, "w", encoding="utf-8") as cookies_file:
+                    cookies_file.write(dialog.cookies_content)
+                self.login_callback(None)
+            else:
+                self.logout()
+        else:
+            return super().login(parent=parent)
 
     def login_callback(self, url):
         """Called after the user has logged in successfully"""
@@ -88,16 +110,10 @@ class HumbleBundleService(OnlineService):
 
     def load(self):
         """Load the user's Humble Bundle library"""
-        if self.is_loading:
-            logger.warning("Humble bundle games are already loading")
-            return
-
-        self.is_loading = True
         try:
             library = self.get_library()
-        except ValueError:
-            logger.error("Failed to get Humble Bundle library. Try logging out and back-in.")
-            return
+        except ValueError as ex:
+            raise RuntimeError("Failed to get Humble Bundle library. Try logging out and back-in.") from ex
         humble_games = []
         seen = set()
         for game in library:
@@ -107,7 +123,6 @@ class HumbleBundleService(OnlineService):
             seen.add(game["human_name"])
         for game in humble_games:
             game.save()
-        self.is_loading = False
         return humble_games
 
     def make_api_request(self, url):
@@ -116,10 +131,7 @@ class HumbleBundleService(OnlineService):
         try:
             request.get()
         except HTTPError:
-            logger.error(
-                "Failed to request %s, check your Humble Bundle credentials and internet connectivity",
-                url,
-            )
+            logger.error("Failed to request %s", url)
             return
         return request.json
 
@@ -214,22 +226,21 @@ class HumbleBundleService(OnlineService):
                 download_links.append(download)
         return download_links
 
-    def get_installer_files(self, installer, installer_file_id):
+    def get_installer_files(self, installer, installer_file_id, _selected_extras):
         """Replace the user provided file with download links from Humble Bundle"""
         try:
             link = get_humble_download_link(installer.service_appid, installer.runner)
         except Exception as ex:
             logger.exception("Failed to get Humble Bundle game: %s", ex)
-            raise UnavailableGame("The download URL for the game could not be determined.") from ex
+            raise UnavailableGameError(_("The download URL for the game could not be determined.")) from ex
         if not link:
-            raise UnavailableGame("No game found on Humble Bundle")
+            raise UnavailableGameError(_("No game found on Humble Bundle"))
         filename = link.split("?")[0].split("/")[-1]
-        return [
-            InstallerFile(installer.game_slug, installer_file_id, {
-                "url": link,
-                "filename": filename
-            })
-        ]
+        file = InstallerFile(installer.game_slug, installer_file_id, {
+            "url": link,
+            "filename": filename
+        })
+        return [file], []
 
     @staticmethod
     def get_filename_for_platform(downloads, platform):
@@ -298,7 +309,7 @@ class HumbleBundleService(OnlineService):
             "name": db_game["name"],
             "version": "Humble Bundle",
             "slug": details["machine_name"],
-            "game_slug": slugify(db_game["name"]),
+            "game_slug": self.get_installed_slug(db_game),
             "runner": runner,
             "humbleid": db_game["appid"],
             "script": {
@@ -311,6 +322,18 @@ class HumbleBundleService(OnlineService):
             }
         }
 
+    def get_installed_runner_name(self, db_game):
+        details = json.loads(db_game["details"])
+        platforms = [download["platform"] for download in details["downloads"]]
+
+        if "linux" in platforms and self.platform_has_downloads(details["downloads"], "linux"):
+            return "linux"
+
+        if "windows" in platforms:
+            return "wine"
+
+        return ""
+
 
 def pick_download_url_from_download_info(download_info):
     """From a list of downloads in Humble Bundle, pick the most appropriate one
