@@ -3,10 +3,12 @@ import datetime
 import json
 import os
 from gettext import gettext as _
+from typing import List
 from urllib.parse import quote_plus, urlencode
 
 from lutris import settings
 from lutris.database import games as games_db
+from lutris.exceptions import UnavailableGameError
 from lutris.installer import AUTO_ELF_EXE, AUTO_WIN32_EXE
 from lutris.installer.installer_file import InstallerFile
 from lutris.services.base import OnlineService
@@ -41,6 +43,16 @@ class ItchIoCover(ServiceMedia):
         return
 
 
+class ItchIoCoverMedium(ItchIoCover):
+    """itch.io game cover, at 60% size"""
+    size = (189, 150)
+
+
+class ItchIoCoverSmall(ItchIoCover):
+    """itch.io game cover, at 30% size"""
+    size = (95, 75)
+
+
 class ItchIoGame(ServiceGame):
     """itch.io Game"""
     service = "itchio"
@@ -81,14 +93,15 @@ class ItchIoService(OnlineService):
     drm_free = True
     has_extras = True
     medias = {
+        "banner_small": ItchIoCoverSmall,
+        "banner_med": ItchIoCoverMedium,
         "banner": ItchIoCover,
     }
     default_format = "banner"
 
     api_url = "https://api.itch.io"
     login_url = "https://itch.io/login"
-    redirect_uri = "https://itch.io/dashboard"
-
+    redirect_uri = "https://itch.io/my-feed"
     cookies_path = os.path.join(settings.CACHE_DIR, ".itchio.auth")
     cache_path = os.path.join(settings.CACHE_DIR, "itchio/api/")
 
@@ -256,6 +269,8 @@ class ItchIoService(OnlineService):
 
     def get_key(self, appid):
         """Retrieve cache information on a key"""
+        if not appid:
+            raise ValueError("Missing Itch.io app ID")
         game_filename = self.get_game_cache(appid)
         game = {}
 
@@ -263,8 +278,11 @@ class ItchIoService(OnlineService):
             with open(game_filename, "r", encoding="utf-8") as game_file:
                 game = json.load(game_file)
         else:
-            game = self.fetch_game(appid).get("game", {})
-            self._cache_games([game])
+            try:
+                game = self.fetch_game(appid).get("game", {})
+                self._cache_games([game])
+            except HTTPError:
+                return
 
         traits = game.get("traits", [])
         if "can_be_bought" not in traits:
@@ -288,7 +306,10 @@ class ItchIoService(OnlineService):
     def get_extras(self, appid):
         """Return a list of bonus content for itch.io game."""
         key = self.get_key(appid)
-        uploads = self.fetch_uploads(appid, key)
+        try:
+            uploads = self.fetch_uploads(appid, key)
+        except HTTPError:
+            return []
         all_extras = {}
         extras = []
         for upload in uploads["uploads"]:
@@ -302,10 +323,13 @@ class ItchIoService(OnlineService):
                     "id": str(upload["id"]),
                 }
             )
-        if len(extras) > 0:
+        if extras:
             all_extras["Bonus Content"] = extras
         return all_extras
 
+    def get_installed_slug(self, db_game):
+        return db_game["slug"]
+
     def generate_installer(self, db_game):
         """Auto generate installer for itch.io game"""
         details = json.loads(db_game["details"])
@@ -332,7 +356,7 @@ class ItchIoService(OnlineService):
             "name": db_game["name"],
             "version": "itch.io",
             "slug": db_game["slug"],
-            "game_slug": db_game["slug"],
+            "game_slug": self.get_installed_slug(db_game),
             "runner": runner,
             "itchid": db_game["appid"],
             "script": {
@@ -344,6 +368,28 @@ class ItchIoService(OnlineService):
             }
         }
 
+    def get_installed_runner_name(self, db_game):
+        details = json.loads(db_game["details"])
+
+        if "p_linux" in details["traits"]:
+            return "linux"
+        if "p_windows" in details["traits"]:
+            return "wine"
+
+        return ""
+
+    def get_game_platforms(self, db_game: dict) -> List[str]:
+        platforms = []
+        details = json.loads(db_game["details"])
+
+        if "p_linux" in details["traits"]:
+            platforms.append("Linux")
+
+        if "p_windows" in details["traits"]:
+            platforms.append("Windows")
+
+        return platforms
+
     def _check_update_with_db(self, db_game, key, upload=None):
         stamp = 0
         if upload:
@@ -420,7 +466,7 @@ class ItchIoService(OnlineService):
                 "version": "itch.io",
                 "name": db_game["name"],
                 "slug": db_game["installer_slug"],
-                "game_slug": db_game["slug"],
+                "game_slug": self.get_installed_slug(db_game),
                 "runner": db_game["runner"],
                 "script": {
                     "extends": db_game["installer_slug"],
@@ -473,12 +519,17 @@ class ItchIoService(OnlineService):
         """Replace the user provided file with download links from itch.io"""
 
         key = self.get_key(installer.service_appid)
-        uploads = self.fetch_uploads(installer.service_appid, key)
+        try:
+            uploads = self.fetch_uploads(installer.service_appid, key)
+        except HTTPError as ex:
+            raise UnavailableGameError from ex
         filtered = []
         extras = []
         files = []
+        extra_files = []
         link = None
         filename = "setup.zip"
+        selected_extras_ids = set(x["id"] for x in selected_extras or [])
 
         file = next(_file.copy() for _file in installer.script_files if _file.id == installer_file_id)
         if not file.url.startswith("N/A"):
@@ -492,9 +543,9 @@ class ItchIoService(OnlineService):
             "date": int(datetime.datetime.now().timestamp())
         }
 
-        if not link or len(selected_extras) > 0:
+        if not link or len(selected_extras_ids) > 0:
             for upload in uploads["uploads"]:
-                if selected_extras and (upload["type"] in self.extra_types):
+                if selected_extras_ids and (upload["type"] in self.extra_types):
                     extras.append(upload)
                     continue
                 # default =  games/tools ("executables")
@@ -544,10 +595,10 @@ class ItchIoService(OnlineService):
             )
 
         for extra in extras:
-            if str(extra["id"]) not in selected_extras:
+            if str(extra["id"]) not in selected_extras_ids:
                 continue
             link = self.get_download_link(extra["id"], key)
-            files.append(
+            extra_files.append(
                 InstallerFile(installer.game_slug, str(extra["id"]), {
                     "url": link,
                     "filename": extra["filename"],
@@ -555,7 +606,7 @@ class ItchIoService(OnlineService):
                 })
             )
 
-        return files
+        return files, extra_files
 
     def get_patch_files(self, installer, installer_file_id):
         """Similar to get_installer_files but for patches"""
