@@ -1,18 +1,13 @@
 """Parser for the glxinfo utility"""
-# Standard Library
-import subprocess
-
-# Lutris Modules
 from lutris.util.log import logger
+from lutris.util.system import read_process_output
 
 
 class Container:  # pylint: disable=too-few-public-methods
-
     """A dummy container for data"""
 
 
 class GlxInfo:
-
     """Give access to the glxinfo information"""
 
     def __init__(self, output=None):
@@ -30,11 +25,7 @@ class GlxInfo:
     @staticmethod
     def get_glxinfo_output():
         """Return the glxinfo -B output"""
-        try:
-            return subprocess.check_output(["glxinfo", "-B"]).decode()
-        except subprocess.CalledProcessError as ex:
-            logger.error("glxinfo call failed: %s", ex)
-            return ""
+        return read_process_output(["glxinfo", "-B"])
 
     def as_dict(self):
         """Return the attributes as a dict"""
@@ -49,7 +40,8 @@ class GlxInfo:
         # combining display and screen)
         output = self._output.replace("  screen", "\nscreen")
         for line in output.split("\n"):
-            if not line.strip():
+            # Skip blank lines, and error lines that may contain no ':'
+            if ":" not in line:
                 continue
 
             key, value = line.split(":", 1)
