@@ -1,11 +1,12 @@
 """XrandR based display management"""
-# Standard Library
 import re
 import subprocess
 from collections import namedtuple
 
-# Lutris Modules
-from lutris.util.log import logger
+from lutris.settings import DEFAULT_RESOLUTION_HEIGHT, DEFAULT_RESOLUTION_WIDTH
+from lutris.util.linux import LINUX_SYSTEM
+from lutris.util.log import logger, logging
+from lutris.util.system import read_process_output
 
 Output = namedtuple("Output", ("name", "mode", "position", "rotation", "primary", "rate"))
 
@@ -13,13 +14,14 @@ Output = namedtuple("Output", ("name", "mode", "position", "rotation", "primary"
 def _get_vidmodes():
     """Return video modes from XrandR"""
     logger.debug("Retrieving video modes from XrandR")
-    try:
-        xrandr_output = subprocess.check_output(["xrandr"])
-    except subprocess.CalledProcessError as ex:
-        logger.error("Unable to read xrandr: %s", ex)
-        return ""
+    return read_process_output([LINUX_SYSTEM.get("xrandr")]).split("\n")
 
-    return xrandr_output.decode().split("\n")
+
+def _log_vidmodes(message):
+    """Write the xrandr output to the log for debugging purposes"""
+    if logger.isEnabledFor(logging.DEBUG):
+        xrandr_output = read_process_output([LINUX_SYSTEM.get("xrandr")])
+        logger.debug("%s\n%s", message, xrandr_output)
 
 
 def get_outputs():  # pylint: disable=too-many-locals
@@ -35,29 +37,30 @@ def get_outputs():  # pylint: disable=too-many-locals
         logger.error("xrandr didn't return anything")
         return []
     for line in vid_modes:
-        if "connected" in line:
-            if "disconnected" in line:
-                continue
-            primary = "primary" in line
+        fields = line.split()
+        if "connected" in fields[1:] and len(fields) >= 4:
             try:
-                if primary:
-                    name, _, _, geometry, rotate, *_ = line.split()
-                else:
-                    name, _, geometry, rotate, *_ = line.split()
+                connected_index = fields.index('connected', 1)
+                name_fields = fields[:connected_index]
+                name = " ".join(name_fields)
+                data_fields = fields[connected_index + 1:]
+                if data_fields[0] == "primary":
+                    data_fields = data_fields[1:]
+                geometry, rotate, *_ = data_fields
+                if geometry.startswith("("):  # Screen turned off, no geometry
+                    continue
+                if rotate.startswith("("):  # Screen not rotated, no need to include
+                    rotate = "normal"
+                _, x_pos, y_pos = geometry.split("+")
+                position = "{x_pos}x{y_pos}".format(x_pos=x_pos, y_pos=y_pos)
             except ValueError as ex:
                 logger.error(
                     "Unhandled xrandr line %s, error: %s. "
                     "Please send your xrandr output to the dev team", line, ex
                 )
                 continue
-            if geometry.startswith("("):  # Screen turned off, no geometry
-                continue
-            if rotate.startswith("("):  # Screen not rotated, no need to include
-                rotate = "normal"
-            _, x_pos, y_pos = geometry.split("+")
-            position = "{x_pos}x{y_pos}".format(x_pos=x_pos, y_pos=y_pos)
         elif "*" in line:
-            mode, *framerates = line.split()
+            mode, *framerates = fields
             for number in framerates:
                 if "*" in number:
                     hertz = number[:-2]
@@ -83,7 +86,8 @@ def turn_off_except(display):
     for output in get_outputs():
         if output.name != display:
             logger.info("Turning off %s", output[0])
-            subprocess.Popen(["xrandr", "--output", output.name, "--off"])
+            with subprocess.Popen([LINUX_SYSTEM.get("xrandr"), "--output", output.name, "--off"]) as xrandr:
+                xrandr.communicate()
 
 
 def get_resolutions():
@@ -94,12 +98,10 @@ def get_resolutions():
             resolution_match = re.match(r".*?(\d+x\d+).*", line)
             if resolution_match:
                 resolution_list.append(resolution_match.groups()[0])
-    return resolution_list
-
-
-def get_unique_resolutions():
-    """Return available resolutions, without duplicates and ordered with highest resolution first"""
-    return sorted(set(get_resolutions()), key=lambda x: int(x.split("x")[0]), reverse=True)
+    if not resolution_list:
+        _log_vidmodes("Unable to generate resolution list from xrandr output")
+        return ['%sx%s' % (DEFAULT_RESOLUTION_WIDTH, DEFAULT_RESOLUTION_HEIGHT)]
+    return sorted(set(resolution_list), key=lambda x: int(x.split("x")[0]), reverse=True)
 
 
 def change_resolution(resolution):
@@ -118,7 +120,9 @@ def change_resolution(resolution):
             logger.warning("Resolution %s doesn't exist.", resolution)
         else:
             logger.info("Changing resolution to %s", resolution)
-            subprocess.Popen(["xrandr", "-s", resolution])
+            with subprocess.Popen([LINUX_SYSTEM.get("xrandr"), "-s", resolution]) as xrandr:
+                xrandr.communicate()
+
     else:
         for display in resolution:
             logger.debug("Switching to %s on %s", display.mode, display.name)
@@ -133,9 +137,9 @@ def change_resolution(resolution):
             else:
                 rotation = "normal"
             logger.info("Switching resolution of %s to %s", display.name, display.mode)
-            subprocess.Popen(
+            with subprocess.Popen(
                 [
-                    "xrandr",
+                    LINUX_SYSTEM.get("xrandr"),
                     "--output",
                     display.name,
                     "--mode",
@@ -147,7 +151,8 @@ def change_resolution(resolution):
                     "--rate",
                     display.rate,
                 ]
-            ).communicate()
+            ) as xrandr:
+                xrandr.communicate()
 
 
 class LegacyDisplayManager:  # pylint: disable=too-few-public-methods
@@ -174,7 +179,8 @@ class LegacyDisplayManager:  # pylint: disable=too-few-public-methods
                 resolution_match = re.match(r".*?(\d+x\d+).*", line)
                 if resolution_match:
                     return resolution_match.groups()[0].split("x")
-        return ("", "")
+        _log_vidmodes("Unable to find the current resolution from xrandr output")
+        return str(DEFAULT_RESOLUTION_WIDTH), str(DEFAULT_RESOLUTION_HEIGHT)
 
     @staticmethod
     def set_resolution(resolution):
