@@ -2,17 +2,22 @@
 import json
 import os
 import socket
+import ssl
 import urllib.error
 import urllib.parse
 import urllib.request
 from ssl import CertificateError
 
+import certifi
+
 from lutris.settings import PROJECT, SITE_URL, VERSION, read_setting
 from lutris.util import system
 from lutris.util.log import logger
 
 DEFAULT_TIMEOUT = read_setting("default_http_timeout") or 30
 
+ssl._create_default_https_context = lambda: ssl.create_default_context(cafile=certifi.where())
+
 
 class HTTPError(Exception):
     """Exception raised on request failures"""
@@ -22,7 +27,7 @@ class HTTPError(Exception):
         self.code = code
 
 
-class UnauthorizedAccess(Exception):
+class UnauthorizedAccessError(Exception):
     """Exception raised for 401 HTTP errors"""
 
 
@@ -79,10 +84,10 @@ class Request:
     def user_agent(self):
         return "{} {}".format(PROJECT, VERSION)
 
-    def get(self, data=None):
-        logger.debug("GET %s", self.url)
+    def _request(self, method, data=None):
+        logger.debug("%s %s", method, self.url)
         try:
-            req = urllib.request.Request(url=self.url, data=data, headers=self.headers)
+            req = urllib.request.Request(url=self.url, data=data, headers=self.headers, method=method)
         except ValueError as ex:
             raise HTTPError("Failed to create HTTP request to %s: %s" % (self.url, ex)) from ex
         try:
@@ -90,10 +95,12 @@ class Request:
                 request = self.opener.open(req, timeout=self.timeout)
             else:
                 request = urllib.request.urlopen(req, timeout=self.timeout)  # pylint: disable=consider-using-with
-        except (urllib.error.HTTPError, CertificateError) as error:
+        except (urllib.error.HTTPError) as error:
             if error.code == 401:
-                raise UnauthorizedAccess("Access to %s denied" % self.url) from error
+                raise UnauthorizedAccessError("Access to %s denied" % self.url) from error
             raise HTTPError("%s" % error, code=error.code) from error
+        except CertificateError as error:
+            raise HTTPError("%s" % error, code=0) from error
         except (socket.timeout, urllib.error.URLError) as error:
             raise HTTPError("Unable to connect to server %s: %s" % (self.url, error)) from error
 
@@ -126,8 +133,11 @@ class Request:
                 return
             yield chunk
 
-    def post(self, data):
-        raise NotImplementedError
+    def get(self, data=None):
+        return self._request("GET", data)
+
+    def post(self, data=None):
+        return self._request("POST", data)
 
     def write_to_file(self, path):
         content = self.content
