@@ -25,22 +25,24 @@ SYSTEM_COMPONENTS = {
         "fuser",
         "glxinfo",
         "vulkaninfo",
+        "fuser",
+        "7z",
+        "gtk-update-icon-cache",
+        "lspci",
+        "ldconfig",
+        "wine",
+        "fluidsynth",
+    ],
+    "OPTIONAL_COMMANDS": [
         "optirun",
         "primusrun",
         "pvkrun",
         "pulseaudio",
         "lsi-steam",
-        "fuser",
-        "7z",
-        "gtk-update-icon-cache",
-        "lspci",
-        "xgamma",
-        "ldconfig",
-        "strangle",
         "Xephyr",
         "nvidia-smi",
-        "wine",
-        "fluidsynth",
+        "strangle",
+        "xgamma",
     ],
     "TERMINALS": [
         "xterm",
@@ -107,7 +109,7 @@ class LinuxSystem:  # pylint: disable=too-many-public-methods
     optional_components = ["WINE", "GAMEMODE"]
 
     def __init__(self):
-        for key in ("COMMANDS", "TERMINALS"):
+        for key in ("COMMANDS", "OPTIONAL_COMMANDS", "TERMINALS"):
             self._cache[key] = {}
             for command in SYSTEM_COMPONENTS[key]:
                 command_path = shutil.which(command)
@@ -115,9 +117,11 @@ class LinuxSystem:  # pylint: disable=too-many-public-methods
                     command_path = self.get_sbin_path(command)
                 if command_path:
                     self._cache[key][command] = command_path
+                elif key == "COMMANDS":
+                    logger.warning("Command '%s' not found on your system", command)
 
         # Detect if system is 64bit capable
-        self.is_64_bit = sys.maxsize > 2**32
+        self.is_64_bit = sys.maxsize > 2 ** 32
         self.arch = self.get_arch()
         self.shared_libraries = self.get_shared_libraries()
         self.populate_libraries()
@@ -160,6 +164,8 @@ class LinuxSystem:  # pylint: disable=too-many-public-methods
     def get_drives():
         """Return a list of drives with their filesystems"""
         lsblk_output = system.read_process_output(["lsblk", "-f", "--json"])
+        if not lsblk_output:
+            return []
         return [
             drive
             for drive in json.loads(lsblk_output)["blockdevices"]
@@ -179,7 +185,7 @@ class LinuxSystem:  # pylint: disable=too-many-public-methods
     @staticmethod
     def get_dist_info():
         """Return distribution information"""
-        if linux_distribution:
+        if linux_distribution is not None:
             return linux_distribution()
         return "unknown"
 
@@ -208,7 +214,7 @@ class LinuxSystem:  # pylint: disable=too-many-public-methods
     def gamemode_available(self):
         """Return whether gamemode is available"""
         # Current versions of gamemode use gamemoderun
-        if system.find_executable("gamemoderun"):
+        if system.can_find_executable("gamemoderun"):
             return True
         # This is for old versions of gamemode only
         if self.is_feature_supported("GAMEMODE"):
@@ -222,18 +228,23 @@ class LinuxSystem:  # pylint: disable=too-many-public-methods
 
         # 515.43.04 was the first driver to support
         # VK_EXT_image_drm_format_modifier, required by gamescope.
-        minimum_nvidia_version_supported = 515
-        driver_info = drivers.get_nvidia_driver_info()
-        driver_version = driver_info["nvrm"]["version"]
-        major_version = int(driver_version.split(".")[0])
-        return major_version >= minimum_nvidia_version_supported
+        try:
+            minimum_nvidia_version_supported = 515
+            driver_info = drivers.get_nvidia_driver_info()
+            driver_version = driver_info["nvrm"]["version"]
+            if not driver_version:
+                return False
+            major_version = int(driver_version.split(".")[0])
+            return major_version >= minimum_nvidia_version_supported
+        except Exception as ex:
+            logger.exception("Unable to determine NVidia version: %s", ex)
+            return False
 
-    @property
     def has_steam(self):
         """Return whether Steam is installed locally"""
         return (
-            bool(system.find_executable("steam"))
-            or bool(system.find_executable("com.valvesoftware.Steam"))
+            system.can_find_executable("steam")
+            or system.can_find_executable("com.valvesoftware.Steam")
             or os.path.exists(os.path.expanduser("~/.steam/steam/ubuntu12_32/steam"))
         )
 
@@ -242,7 +253,6 @@ class LinuxSystem:  # pylint: disable=too-many-public-methods
         """Return the display server used"""
         return os.environ.get("XDG_SESSION_TYPE", "unknown")
 
-    @property
     def is_flatpak(self):
         """Check is we are running inside Flatpak sandbox"""
         return system.path_exists("/.flatpak-info")
@@ -384,6 +394,15 @@ class LinuxSystem:  # pylint: disable=too-many-public-methods
         """Return a dictionary of missing libraries"""
         return {req: self.get_missing_requirement_libs(req) for req in self.requirements}
 
+    def get_missing_lib_arch(self, requirement):
+        """Returns a list of architectures that are missing a library for a specific
+        requirement."""
+        missing_arch = []
+        for index, arch in enumerate(self.runtime_architectures):
+            if self.get_missing_requirement_libs(requirement)[index]:
+                missing_arch.append(arch)
+        return missing_arch
+
     def is_feature_supported(self, feature):
         """Return whether the system has the necessary libs to support a feature"""
         if feature == "ACO":
@@ -394,6 +413,9 @@ class LinuxSystem:  # pylint: disable=too-many-public-methods
                 return False
         return not self.get_missing_requirement_libs(feature)[0]
 
+    def is_vulkan_supported(self):
+        return not LINUX_SYSTEM.get_missing_lib_arch("VULKAN") and vkquery.is_vulkan_supported()
+
 
 class SharedLibrary:
     """Representation of a Linux shared library"""
@@ -458,7 +480,7 @@ def gather_system_info():
     return system_info
 
 
-def gather_system_info_str():
+def gather_system_info_dict():
     """Get all relevant system information already formatted as a string"""
     system_info = gather_system_info()
     system_info_readable = {}
@@ -506,15 +528,7 @@ def gather_system_info_str():
     else:
         graphics_dict["Vulkan"] = "Not Supported"
     system_info_readable["Graphics"] = graphics_dict
-
-    output = ''
-    for section, dictionary in system_info_readable.items():
-        output += '[%s]\n' % section
-        for key, value in dictionary.items():
-            tabs = " " * (16 - len(key))
-            output += '%s:%s%s\n' % (key, tabs, value)
-        output += '\n'
-    return output
+    return system_info_readable
 
 
 def get_terminal_apps():
