@@ -10,21 +10,14 @@ IGNORED_PROCESSES = (
 )
 
 
-class InvalidPid(Exception):
-
-    """Exception raised when an operation on a non-existent PID is called"""
-
-
 class Process:
-
     """Python abstraction a Linux process"""
 
     def __init__(self, pid):
         try:
             self.pid = int(pid)
-            self.error_cache = []
         except ValueError as err:
-            raise InvalidPid("'%s' is not a valid pid" % pid) from err
+            raise ValueError("'%s' is not a valid pid" % pid) from err
 
     def __repr__(self):
         return "Process {}".format(self.pid)
@@ -37,9 +30,12 @@ class Process:
         try:
             with open(file_path, encoding='utf-8') as proc_file:
                 content = proc_file.read()
-        except (ProcessLookupError, FileNotFoundError, PermissionError):
+        except PermissionError:
             return ""
-        return content
+        except (ProcessLookupError, FileNotFoundError) as ex:
+            logger.debug(ex)
+            return ""
+        return content.strip("\x00")
 
     def get_stat(self, parsed=True):
         stat_filename = "/proc/{}/stat".format(self.pid)
@@ -69,7 +65,7 @@ class Process:
         try:
             with open(children_path, encoding='utf-8') as children_file:
                 children_content = children_file.read()
-        except (FileNotFoundError, ProcessLookupError):
+        except (FileNotFoundError, ProcessLookupError, PermissionError):
             children_content = ""
         return children_content.strip().split()
 
@@ -112,15 +108,14 @@ class Process:
         """Return the process' environment variables"""
         environ_path = "/proc/{}/environ".format(self.pid)
         _environ_text = self._read_content(environ_path)
-        if not _environ_text:
-            return {}
-        try:
-            return dict([line.split("=", 1) for line in _environ_text.split("\x00") if line])
-        except ValueError:
-            if environ_path not in self.error_cache:
-                logger.error("Failed to parse environment variables: %s", _environ_text)
-                self.error_cache.append(environ_path)
+        if not _environ_text or "=" not in _environ_text:
             return {}
+        env_vars = []
+        for line in _environ_text.split("\x00"):
+            if "=" not in line:
+                continue
+            env_vars.append(line.split("=", 1))
+        return dict(env_vars)
 
     @property
     def children(self):
