@@ -1,5 +1,4 @@
 """Handle Steam configuration"""
-import glob
 import os
 from collections import OrderedDict
 
@@ -8,6 +7,7 @@ import requests
 from lutris import settings
 from lutris.util import system
 from lutris.util.log import logger
+from lutris.util.steam.steamid import SteamID
 from lutris.util.steam.vdfutils import vdf_parse
 
 STEAM_DATA_DIRS = (
@@ -15,11 +15,14 @@ STEAM_DATA_DIRS = (
     "~/.steam",
     "~/.local/share/steam",
     "~/.local/share/Steam",
+    "~/snap/steam/common/.local/share/Steam",
     "~/.steam/steam",
     "~/.var/app/com.valvesoftware.Steam/data/steam",
+    "~/.var/app/com.valvesoftware.Steam/data/Steam",
     "/usr/share/steam",
     "/usr/local/share/steam",
 )
+STEAM_ACCOUNT_SETTING = "active_steam_account"
 
 
 def get_steam_dir():
@@ -39,17 +42,6 @@ def search_in_steam_dirs(file):
             return path
 
 
-def search_recursive_in_steam_dirs(path_suffix):
-    """Perform a recursive search based on glob and returns a
-    list of hits"""
-    results = []
-    for candidate in STEAM_DATA_DIRS:
-        glob_path = os.path.join(os.path.expanduser(candidate), path_suffix)
-        for path in glob.glob(glob_path):
-            results.append(path)
-    return results
-
-
 def get_default_acf(appid, name):
     """Return a default configuration usable to
     create a runnable game in Steam"""
@@ -76,7 +68,7 @@ def read_user_config():
     return config
 
 
-def get_config_value(config, key):
+def get_config_value(config: dict, key: str):
     """Fetch a value from a configuration in a case insensitive way"""
     keymap = {k.lower(): k for k in config.keys()}
     if key not in keymap:
@@ -87,17 +79,56 @@ def get_config_value(config, key):
     return config[keymap[key.lower()]]
 
 
-def get_user_steam_id():
-    """Read user's SteamID from Steam config files"""
+def get_user_data_dirs():
+    """Return the list of available Steam user config directories (using a SteamID32)
+    and the base path where the settings are located (Returns the 1st location found)
+    """
+    for steam_dir in STEAM_DATA_DIRS:
+        userdata_path = os.path.join(os.path.expanduser(steam_dir), "userdata")
+        if not os.path.exists(userdata_path):
+            continue
+        user_ids = [f for f in os.listdir(userdata_path) if f.isnumeric()]
+        if user_ids:
+            return userdata_path, user_ids
+    return "", []
+
+
+def get_steam_users() -> list:
+    """Return a list of available Steam users.
+    Most recently used account is 1st in the list."""
+    steam_users = []
     user_config = read_user_config()
     if not user_config or "users" not in user_config:
-        return
-    last_steam_id = None
-    for steam_id in user_config["users"]:
-        last_steam_id = steam_id
-        if get_config_value(user_config["users"][steam_id], "mostrecent") == "1":
-            return steam_id
-    return last_steam_id
+        return []
+    most_recent = None
+    for steam_id, account in user_config["users"].items():
+        account["steamid64"] = steam_id
+        if get_config_value(account, "mostrecent") == "1":
+            most_recent = account
+        else:
+            steam_users.append(account)
+    if most_recent:
+        steam_users = [most_recent] + steam_users
+    return steam_users
+
+
+def get_active_steamid64() -> str:
+    """Return the currently active Steam ID"""
+    steam_ids = [u["steamid64"] for u in get_steam_users()]
+    active_steam_id = settings.read_setting(STEAM_ACCOUNT_SETTING)
+    if active_steam_id in steam_ids:
+        return active_steam_id
+    if steam_ids:
+        return steam_ids[0]
+    return ""
+
+
+def convert_steamid64_to_steamid32(steamid64: str) -> str:
+    """Return the 32bit variant of SteamIDs, used for folder names in userdata"""
+    if not steamid64.isnumeric():
+        return ""
+    steam_id = SteamID.from_steamid64(int(steamid64))
+    return str(steam_id.get_32_bit_community_id())
 
 
 def get_steam_library(steamid):
@@ -177,6 +208,54 @@ def read_library_folders(steam_data_dir):
         logger.error("Steam libraryfolders %s is empty: %s", library_filename, ex)
 
 
-def get_steamapps_paths():
-    from lutris.runners import steam  # pylint: disable=import-outside-toplevel
-    return steam.steam().get_steamapps_dirs()
+def get_steam_config():
+    """Return the "Steam" part of Steam's config.vdf as a dict."""
+    return read_config(get_steam_dir())
+
+
+def get_library_config():
+    """Return the "libraryfolders" part of Steam's libraryfolders.vdf as a dict """
+    return read_library_folders(get_steam_dir())
+
+
+def get_steamapps_dirs():
+    """Return a list of the Steam library main + custom folders."""
+    dirs = []
+    # Extra colon-separated compatibility tools dirs environment variable
+    if 'STEAM_EXTRA_COMPAT_TOOLS_PATHS' in os.environ:
+        dirs += os.getenv('STEAM_EXTRA_COMPAT_TOOLS_PATHS').split(':')
+    # Main steamapps dir and compatibilitytools.d dir
+    for data_dir in STEAM_DATA_DIRS:
+        for _dir in ["steamapps", "compatibilitytools.d"]:
+            abs_dir = os.path.join(os.path.expanduser(data_dir), _dir)
+            abs_dir = system.fix_path_case(abs_dir)
+            if abs_dir and os.path.isdir(abs_dir):
+                dirs.append(abs_dir)
+
+    # Custom dirs
+    steam_config = get_steam_config()
+    if steam_config:
+        i = 1
+        while "BaseInstallFolder_%s" % i in steam_config:
+            path = steam_config["BaseInstallFolder_%s" % i] + "/steamapps"
+            path = system.fix_path_case(path)
+            if path and os.path.isdir(path):
+                dirs.append(path)
+            i += 1
+
+    # New Custom dirs
+    library_config = get_library_config()
+    if library_config:
+        paths = []
+        for entry in library_config.values():
+            if "mounted" in entry:
+                if entry.get("path") and entry.get("mounted") == "1":
+                    path = system.fix_path_case(entry.get("path") + "/steamapps")
+                    paths.append(path)
+            else:
+                path = system.fix_path_case(entry.get("path") + "/steamapps")
+                paths.append(path)
+        for path in paths:
+            if path and os.path.isdir(path):
+                dirs.append(path)
+    return system.list_unique_folders(dirs)
