@@ -4,10 +4,12 @@ import os
 import shutil
 from gettext import gettext as _
 
+from lutris import settings
 from lutris.util import system
 from lutris.util.extract import extract_archive
 from lutris.util.http import download_file
 from lutris.util.log import logger
+from lutris.util.strings import parse_version
 from lutris.util.wine.prefix import WinePrefixManager
 
 
@@ -26,8 +28,6 @@ class DLLManager:
 
     def __init__(self, prefix=None, arch="win64", version=None):
         self.prefix = prefix
-        if not os.path.isdir(self.base_dir):
-            os.makedirs(self.base_dir)
         self._versions = []
         self._version = version
         self.wine_arch = arch
@@ -36,8 +36,10 @@ class DLLManager:
     def versions(self):
         """Return available versions"""
         self._versions = self.load_versions()
-        if not self._versions:
-            self._versions = os.listdir(self.base_dir)
+        if system.path_exists(self.base_dir):
+            for local_version in os.listdir(self.base_dir):
+                if os.path.isdir(os.path.join(self.base_dir, local_version)) and local_version not in self._versions:
+                    self._versions.append(local_version)
         return self._versions
 
     @property
@@ -45,8 +47,40 @@ class DLLManager:
         """Return version (latest known version if not provided)"""
         if self._version:
             return self._version
-        if self.versions:
-            return self.versions[0]
+        versions = self.versions
+        if versions:
+            def get_preference_key(v):
+                return not self.is_compatible_version(v), not self.is_recommended_version(v)
+
+            # Put the compatible versions first, and the recommended ones before unrecommended ones.
+            sorted_versions = sorted(versions, key=get_preference_key)
+            return sorted_versions[0]
+
+    def is_recommended_version(self, version):
+        """True if the version given should be usable as the default; false if it
+        should not be the default, but may be selected by the user. If only
+        non-recommended versions exist, we'll still default to one of them, however."""
+        return True
+
+    def is_compatible_version(self, version):
+        """True if the version of the component is compatible with this Lutris. We can tell only
+        once it is downloaded; if not this is always True.
+
+        This checks the file 'lutris.json', which may contain the lowest version of Lutris
+        the component version will work with. If this setting is absent, it is assumed compatible."""
+        dir_settings = settings.get_lutris_directory_settings(self.base_dir)
+
+        try:
+            if "min_lutris_version" in dir_settings:
+                min_lutris_version = parse_version(dir_settings["min_lutris_version"])
+                current_lutris_version = parse_version(settings.VERSION)
+                if current_lutris_version < min_lutris_version:
+                    return False
+        except TypeError as ex:
+            logger.exception("Invalid lutris.json: %s", ex)
+            return False
+
+        return True
 
     @property
     def path(self):
@@ -66,7 +100,7 @@ class DLLManager:
             _choices.append((version, version))
         return _choices
 
-    def load_versions(self):
+    def load_versions(self) -> list:
         if not system.path_exists(self.versions_path):
             return []
         with open(self.versions_path, "r", encoding='utf-8') as version_file:
@@ -144,10 +178,7 @@ class DLLManager:
                     shutil.move(wine_dll_path, wine_dll_path + ".orig")
                 else:
                     os.remove(wine_dll_path)
-            try:
-                os.symlink(dll_path, wine_dll_path)
-            except OSError:
-                logger.error("Failed linking %s to %s", dll_path, wine_dll_path)
+            system.create_symlink(dll_path, wine_dll_path)
         else:
             self.disable_dll(system_dir, arch, dll)
 
@@ -172,11 +203,7 @@ class DLLManager:
 
             if not os.path.isdir(wine_file_dir):
                 os.makedirs(wine_file_dir)
-
-            try:
-                os.symlink(source_path, wine_file_path)
-            except OSError:
-                logger.error("Failed linking %s to %s", source_path, wine_file_path)
+            system.create_symlink(source_path, wine_file_path)
         else:
             self.disable_user_file(appdata_dir, file_path)
 
@@ -210,6 +237,32 @@ class DLLManager:
                 filename = os.path.basename(file)
                 yield appdata_dir, file, filename
 
+    def setup(self, enable):
+        """Enable or disable DLLs"""
+
+        # manual version only sets the dlls to native (in get_enabling_dll_overrides())
+        manager_version = self.version
+        if not manager_version or manager_version.lower() != "manual":
+            if enable:
+                self.enable()
+            else:
+                self.disable()
+
+    def get_enabling_dll_overrides(self):
+        """Returns aa dll-override dict for the dlls in this manager; these options will
+        enable the manager's dll, so call this only for enabled managers."""
+        overrides = {}
+
+        for dll in self.managed_dlls:
+            # We have to make sure that the dll exists before setting it to native
+            if self.dll_exists(dll):
+                overrides[dll] = "n"
+
+        return overrides
+
+    def can_enable(self):
+        return True
+
     def enable(self):
         """Enable Dlls for the current prefix"""
         if not self.is_available():
@@ -238,10 +291,23 @@ class DLLManager:
         download_file(self.releases_url, self.versions_path, overwrite=True)
 
     def upgrade(self):
-        self.fetch_versions()
         if not self.is_available():
-            if self.version:
-                logger.info("Downloading %s %s...", self.component, self.version)
-                self.download()
-            else:
+            versions = self.load_versions()
+
+            if not versions:
                 logger.warning("Unable to download %s because version information was not available.", self.component)
+
+            # We prefer recommended versions, so download those first.
+            versions.sort(key=self.is_recommended_version, reverse=True)
+
+            for version in versions:
+                logger.info("Downloading %s %s...", self.component, version)
+                self.download()
+
+                if self.is_compatible_version(version):
+                    return  # got a compatible version, that'll do.
+
+                logger.warning("Version %s of %s is not compatible with this version of Lutris.", version,
+                               self.component)
+
+            # We found nothing compatible, and downloaded everything, we just give up.
